@@ -3,6 +3,7 @@
 import collections
 import hashlib
 import logging
+from typing import Any, Optional, Tuple, Union
 
 from pytype import compat
 from pytype import datatypes
@@ -12,12 +13,12 @@ from pytype.pytd import mro
 from pytype.typegraph import cfg
 from pytype.typegraph import cfg_utils
 
-import six
-
 log = logging.getLogger(__name__)
 
+# We can't import abstract.BaseValue here due to a circular dep.
+_BaseValue = Any
 
-# Type parameter names matching the ones in __builtin__.pytd and typing.pytd.
+# Type parameter names matching the ones in builtins.pytd and typing.pytd.
 T = "_T"
 T2 = "_T2"
 K = "_K"
@@ -25,7 +26,6 @@ V = "_V"
 ARGS = "_ARGS"
 RET = "_RET"
 
-
 # TODO(rechen): Stop supporting all variants except _HAS_DYNAMIC_ATTRIBUTES.
 DYNAMIC_ATTRIBUTE_MARKERS = [
     "HAS_DYNAMIC_ATTRIBUTES",
@@ -33,7 +33,6 @@ DYNAMIC_ATTRIBUTE_MARKERS = [
     "has_dynamic_attributes",
 ]
 
-
 # A dummy container object for use in instantiating type parameters.
 # A container is needed to preserve type parameter names for error messages
 # and for sub_(one_)annotation(s).
@@ -60,12 +59,12 @@ class GenericTypeError(Exception):
   """The error for user-defined generic types."""
 
   def __init__(self, annot, error):
-    super(GenericTypeError, self).__init__(annot, error)
+    super().__init__(annot, error)
     self.annot = annot
     self.error = error
 
 
-class AsInstance(object):
+class AsInstance:
   """Wrapper, used for marking things that we want to convert to an instance."""
 
   def __init__(self, cls):
@@ -81,16 +80,18 @@ LazyFormalTypeParameters = collections.namedtuple(
     "LazyFormalTypeParameters", ("template", "parameters", "subst"))
 
 
-_NONE = object()  # sentinel for get_atomic_value
+# Sentinel for get_atomic_value
+class _None:
+  pass
 
 
-def get_atomic_value(variable, constant_type=None, default=_NONE):
+def get_atomic_value(variable, constant_type=None, default=_None()):
   """Get the atomic value stored in this variable."""
   if len(variable.bindings) == 1:
     v, = variable.bindings
     if isinstance(v.data, constant_type or object):
       return v.data  # success
-  if default is not _NONE:
+  if not isinstance(default, _None):
     # If a default is specified, we return it instead of failing.
     return default
   # Determine an appropriate failure message.
@@ -122,9 +123,14 @@ def get_atomic_python_constant(variable, constant_type=None):
   return atomic.vm.convert.value_to_constant(atomic, constant_type)
 
 
-def get_views(variables, node, filter_strict=False):
+def get_views(variables, node):
   """Get all possible views of the given variables at a particular node.
 
+  For performance reasons, this method uses node.CanHaveCombination for
+  filtering. For a more precise check, you can call
+  node.HasCombination(list(view.values())). Do so judiciously, as the latter
+  method can be very slow.
+
   This function can be used either as a regular generator or in an optimized way
   to yield only functionally unique views:
     views = get_views(...)
@@ -141,8 +147,6 @@ def get_views(variables, node, filter_strict=False):
   Args:
     variables: The variables.
     node: The node.
-    filter_strict: If True, emit a view only when node.HasCombination is
-      satisfied; else, use the faster node.CanHaveCombination.
 
   Yields:
     A datatypes.AcessTrackingDict mapping variables to bindings.
@@ -155,21 +159,20 @@ def get_views(variables, node, filter_strict=False):
   seen = []  # the accessed subsets of previously seen views
   for combination in combinations:
     view = {value.variable: value for value in combination}
-    if any(subset <= six.viewitems(view) for subset in seen):
+    if any(subset <= view.items() for subset in seen):
       # Optimization: This view can be skipped because it matches the accessed
       # subset of a previous one.
       log.info("Skipping view (already seen): %r", view)
       continue
     combination = list(view.values())
-    check = node.HasCombination if filter_strict else node.CanHaveCombination
-    if not check(combination):
+    if not node.CanHaveCombination(combination):
       log.info("Skipping combination (unreachable): %r", combination)
       continue
     view = datatypes.AccessTrackingDict(view)
     skip_future = yield view
     if skip_future:
       # Skip future views matching this accessed subset.
-      seen.append(six.viewitems(view.accessed_subset))
+      seen.append(view.accessed_subset.items())
 
 
 def get_signatures(func):
@@ -200,10 +203,14 @@ def func_name_is_class_init(name):
 def has_type_parameters(node, val, seen=None):
   """Checks if the given object contains any TypeParameters.
 
+  This method differs from the `.formal` attribute on abstract values in that it
+  recurses into container instances to detect type parameters that have
+  (likely incorrectly) been inserted into runtime objects.
+
   Args:
     node: The current CFG node.
     val: The object to check for TypeParameters. Will likely be a
-      SimpleAbstractValue or a cfg.Variable with SimpleAbstractValues bindings.
+      SimpleValue or a cfg.Variable with SimpleValues bindings.
     seen: Optional. A set of already-visited objects, to avoid infinite loops.
 
   Returns:
@@ -220,7 +227,7 @@ def has_type_parameters(node, val, seen=None):
     return True
   elif val.isinstance_ParameterizedClass() or val.isinstance_Union():
     return val.formal
-  elif val.isinstance_SimpleAbstractValue():
+  elif val.isinstance_SimpleValue():
     return any((has_type_parameters(node, tp, seen)
                 for tp in val.instance_type_parameters.values()))
   else:
@@ -425,7 +432,7 @@ def compute_template(val):
   if the check fails.
 
   Args:
-    val: The abstract.AtomicAbstractValue to compute a template for.
+    val: The abstract.BaseValue to compute a template for.
 
   Returns:
     parsed type parameters
@@ -480,9 +487,9 @@ def compute_template(val):
         seqs.append(seq)
     try:
       template.extend(mro.MergeSequences(seqs))
-    except ValueError:
+    except ValueError as e:
       raise GenericTypeError(
-          val, "Illegal type parameter order in class %s" % val.name)
+          val, "Illegal type parameter order in class %s" % val.name) from e
 
   return template
 
@@ -590,7 +597,7 @@ def check_classes(var, check):
 
   Args:
     var: A cfg.Variable or empty.
-    check: (AtomicAbstractValue) -> bool.
+    check: (BaseValue) -> bool.
 
   Returns:
     Whether the check passes.
@@ -599,21 +606,16 @@ def check_classes(var, check):
       v.cls.isinstance_Class() and check(v.cls) for v in var.data if v.cls)
 
 
-def match_type_container(var, container_type_name):
+def match_type_container(typ, container_type_name: Union[str, Tuple[str, ...]]):
   """Unpack the type parameter from ContainerType[T]."""
-  if var is None:
+  if typ is None:
     return None
-  data = var.data[0]
-  if data.isinstance_Instance():
-    cls = data.cls
-  elif data.isinstance_Class():
-    cls = data
-  else:
+  if isinstance(container_type_name, str):
+    container_type_name = (container_type_name,)
+  if not (typ.isinstance_ParameterizedClass() and
+          typ.full_name in container_type_name):
     return None
-  if not (cls.isinstance_ParameterizedClass() and
-          cls.full_name == container_type_name):
-    return None
-  param = cls.get_formal_type_parameter(T)
+  param = typ.get_formal_type_parameter(T)
   return param
 
 
@@ -633,7 +635,113 @@ def get_annotations_dict(members):
     return None
   annots_var = members["__annotations__"]
   try:
-    annots = get_atomic_python_constant(annots_var, dict)
+    annots = get_atomic_value(annots_var)
   except ConversionError:
     return None
-  return annots
+  return annots if annots.isinstance_AnnotationsDict() else None
+
+
+class Local:
+  """A possibly annotated local variable."""
+
+  def __init__(self, node, op, typ, orig, vm):
+    self._ops = [op]
+    if typ:
+      self.typ = vm.program.NewVariable([typ], [], node)
+    else:
+      # Creating too many variables bloats the typegraph, hurting performance,
+      # so we use None instead of an empty variable.
+      self.typ = None
+    self.orig = orig
+    self.vm = vm
+
+  @property
+  def last_op(self):
+    # TODO(b/74434237): This property can be removed once the usage of it in
+    # dataclass_overlay is gone.
+    return self._ops[-1]
+
+  @property
+  def stack(self):
+    return self.vm.simple_stack(self.last_op)
+
+  def update(self, node, op, typ, orig):
+    """Update this variable's annotation and/or value."""
+    if op in self._ops:
+      return
+    self._ops.append(op)
+    if typ:
+      if self.typ:
+        self.typ.AddBinding(typ, [], node)
+      else:
+        self.typ = self.vm.program.NewVariable([typ], [], node)
+    if orig:
+      self.orig = orig
+
+  def get_type(self, node, name):
+    """Gets the variable's annotation."""
+    if not self.typ:
+      return None
+    values = self.typ.Data(node)
+    if len(values) > 1:
+      self.vm.errorlog.ambiguous_annotation(self.stack, values, name)
+      return self.vm.convert.unsolvable
+    elif values:
+      return values[0]
+    else:
+      return None
+
+
+def is_literal(annot: Optional[_BaseValue]):
+  if not annot:
+    return False
+  if annot.isinstance_Union():
+    return all(is_literal(o) for o in annot.options)
+  return annot.isinstance_LiteralClass()
+
+
+def is_indefinite_iterable(val: _BaseValue):
+  """True if val is a non-concrete instance of typing.Iterable."""
+  instance = val.isinstance_Instance()
+  concrete = (val.isinstance_PythonConstant() and
+              not getattr(val, "could_contain_anything", False))
+  cls_instance = val.cls and val.cls.isinstance_Class()
+  if not (instance and cls_instance and not concrete):
+    return False
+  for cls in val.cls.mro:
+    if cls.full_name == "builtins.str":
+      return False
+    elif cls.full_name == "builtins.tuple":
+      # A tuple's cls attribute may point to either PyTDClass(tuple) or
+      # TupleClass; only the former is indefinite.
+      return cls.isinstance_PyTDClass()
+    elif cls.full_name == "typing.Iterable":
+      return True
+  return False
+
+
+def is_var_indefinite_iterable(var):
+  """True if all bindings of var are indefinite sequences."""
+  return all(is_indefinite_iterable(x) for x in var.data)
+
+
+def merged_type_parameter(node, var, param):
+  if not var.bindings:
+    return node.program.NewVariable()
+  if is_var_splat(var):
+    var = unwrap_splat(var)
+  params = [v.get_instance_type_parameter(param) for v in var.data]
+  return var.data[0].vm.join_variables(node, params)
+
+
+def is_var_splat(var):
+  if var.data and var.data[0].isinstance_Splat():
+    # A splat should never have more than one binding, since we create and use
+    # it immediately.
+    assert len(var.bindings) == 1
+    return True
+  return False
+
+
+def unwrap_splat(var):
+  return var.data[0].iterable
