@@ -23,16 +23,22 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
   def sub_one_annotation(self, node, annot, substs, instantiate_unbound=True):
     """Apply type parameter substitutions to an annotation."""
     if isinstance(annot, abstract.TypeParameter):
-      def contains(subst, annot):
-        return (annot.full_name in subst and subst[annot.full_name].bindings and
-                not any(isinstance(v, abstract.AMBIGUOUS_OR_EMPTY)
-                        for v in subst[annot.full_name].data))
-      if all(contains(subst, annot) for subst in substs):
+      # We use the given substitutions to bind the annotation if
+      # (1) every subst provides at least one binding, and
+      # (2) none of the bindings are ambiguous, and
+      # (3) at least one binding is non-empty.
+      if all(annot.full_name in subst and subst[annot.full_name].bindings
+             for subst in substs):
         vals = sum((subst[annot.full_name].data for subst in substs), [])
-      elif instantiate_unbound:
-        vals = annot.instantiate(node).data
       else:
-        vals = [annot]
+        vals = None
+      if (vals is None or
+          any(isinstance(v, abstract.AMBIGUOUS) for v in vals) or
+          all(isinstance(v, abstract.Empty) for v in vals)):
+        if instantiate_unbound:
+          vals = annot.instantiate(node).data
+        else:
+          vals = [annot]
       return self.vm.convert.merge_classes(vals)
     elif isinstance(annot, mixin.NestedAnnotation):
       inner_types = [(key, self.sub_one_annotation(node, val, substs,
@@ -79,13 +85,15 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
         return new_annot
       return annot
     elif isinstance(annot, abstract.TupleClass):
-      annot.formal_type_parameters[abstract_utils.T] = self.add_scope(
+      params = dict(annot.formal_type_parameters)
+      params[abstract_utils.T] = self.add_scope(
           annot.formal_type_parameters[abstract_utils.T], types, module)
-      return annot
+      return abstract.TupleClass(
+          annot.base_cls, params, self.vm, annot.template)
     elif isinstance(annot, mixin.NestedAnnotation):
-      for key, typ in annot.get_inner_types():
-        annot.update_inner_type(key, self.add_scope(typ, types, module))
-      return annot
+      inner_types = [(key, self.add_scope(typ, types, module))
+                     for key, typ in annot.get_inner_types()]
+      return annot.replace(inner_types)
     return annot
 
   def get_type_parameters(self, annot, seen=None):
@@ -108,8 +116,11 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
     if isinstance(annot, abstract.TypeParameter):
       return [annot]
     elif isinstance(annot, abstract.TupleClass):
-      return self.get_type_parameters(
-          annot.formal_type_parameters[abstract_utils.T], seen)
+      annots = []
+      for idx in range(annot.tuple_length):
+        annots.extend(self.get_type_parameters(
+            annot.formal_type_parameters[idx], seen))
+      return annots
     elif isinstance(annot, mixin.NestedAnnotation):
       return sum((self.get_type_parameters(t, seen)
                   for _, t in annot.get_inner_types()), [])
