@@ -1,7 +1,6 @@
 """Functions for computing the execution order of bytecode."""
 
 import bisect
-import itertools
 
 from pytype.pyc import loadmarshal
 from pytype.pyc import opcodes
@@ -19,7 +18,7 @@ STORE_OPCODES = (
 CODE_LOADING_OPCODES = (opcodes.LOAD_CONST,)
 
 
-class OrderedCode(object):
+class OrderedCode:
   """Code object which knows about instruction ordering.
 
   Attributes:
@@ -44,6 +43,8 @@ class OrderedCode(object):
     python_version: The Python version this bytecode is from.
   """
 
+  _HAS_DYNAMIC_ATTRIBUTES = True
+
   def __init__(self, code, bytecode, order, python_version):
     # Copy all "co_*" attributes from code.
     # This is preferable to both inheritance (because we don't want to be
@@ -62,8 +63,7 @@ class OrderedCode(object):
       insn.code = self
 
   def has_opcode(self, op_type):
-    return any(isinstance(op, op_type)
-               for op in itertools.chain(*(block.code for block in self.order)))
+    return any(isinstance(op, op_type) for op in self.co_code)
 
   def has_iterable_coroutine(self):
     return bool(self.co_flags & loadmarshal.CodeType.CO_ITERABLE_COROUTINE)
@@ -98,7 +98,7 @@ class OrderedCode(object):
     return count
 
 
-class Block(object):
+class Block:
   """A block is a node in a directed graph.
 
   It has incoming and outgoing edges (jumps). Incoming jumps always jump
@@ -141,7 +141,7 @@ class Block(object):
     return self.code.__iter__()
 
 
-def add_pop_block_targets(bytecode):
+def add_pop_block_targets(bytecode, python_version):
   """Modifies bytecode so that each POP_BLOCK has a block_target.
 
   This is to achieve better initial ordering of try/except and try/finally code.
@@ -159,6 +159,7 @@ def add_pop_block_targets(bytecode):
 
   Args:
     bytecode: An array of bytecodes.
+    python_version: The target python version.
   """
   if not bytecode:
     return
@@ -166,6 +167,10 @@ def add_pop_block_targets(bytecode):
   for op in bytecode:
     op.block_target = None
 
+  if python_version >= (3, 8):
+    setup_except_op = opcodes.SETUP_FINALLY
+  else:
+    setup_except_op = opcodes.SETUP_EXCEPT
   todo = [(bytecode[0], ())]  # unordered queue of (position, block_stack)
   seen = set()
   while todo:
@@ -184,7 +189,7 @@ def add_pop_block_targets(bytecode):
       # Make "raise" statements jump to the innermost exception handler.
       # (If there's no exception handler, do nothing.)
       for b in reversed(block_stack):
-        if isinstance(b, opcodes.SETUP_EXCEPT):
+        if isinstance(b, setup_except_op):
           op.block_target = b.target
           break
     elif isinstance(op, opcodes.BREAK_LOOP):
@@ -196,7 +201,7 @@ def add_pop_block_targets(bytecode):
           assert b.target != op
           todo.append((op.block_target, block_stack[0:i]))
           break
-    elif isinstance(op, opcodes.SETUP_EXCEPT):
+    elif isinstance(op, setup_except_op):
       # Exceptions pop the block, so store the previous block stack.
       todo.append((op.target, block_stack))
       block_stack += (op,)
@@ -264,7 +269,7 @@ def compute_order(bytecode):
   return cfg_utils.order_nodes(blocks)
 
 
-class DisCodeVisitor(object):
+class DisCodeVisitor:
   """Visitor for disassembling code into Opcode objects."""
 
   def visit_code(self, code):
@@ -272,7 +277,7 @@ class DisCodeVisitor(object):
     return code
 
 
-def order_code(code):
+def order_code(code, python_version):
   """Split a CodeType object into ordered blocks.
 
   This takes a CodeType object (i.e., a piece of compiled Python code) and
@@ -280,27 +285,31 @@ def order_code(code):
 
   Args:
     code: A loadmarshal.CodeType object.
+    python_version: The target python version.
 
   Returns:
     A CodeBlocks instance.
   """
   bytecodes = code.co_code
-  add_pop_block_targets(bytecodes)
+  add_pop_block_targets(bytecodes, python_version)
   return OrderedCode(code, bytecodes, compute_order(bytecodes),
                      code.python_version)
 
 
-class OrderCodeVisitor(object):
+class OrderCodeVisitor:
   """Visitor for recursively changing all CodeType to OrderedCode.
 
   Depends on DisCodeVisitor having been run first.
   """
 
+  def __init__(self, python_version):
+    self._python_version = python_version
+
   def visit_code(self, code):
-    return order_code(code)
+    return order_code(code, self._python_version)
 
 
-class CollectAnnotationTargetsVisitor(object):
+class CollectAnnotationTargetsVisitor:
   """Collect opcodes that might have annotations attached.
 
   Depends on DisCodeVisitor having been run first.
@@ -383,17 +392,18 @@ def merge_annotations(code, annotations, docstrings):
   # To avoid associating a docstring with multiple functions when a one-line
   # function is followed by a body-less function with a docstring, we do not
   # search for a function's docstring past the start of the next function.
-  max_line = None
+  max_docstring_index = len(docstrings)
   for start, (end, op) in sorted(
       visitor.make_function_ops.items(), reverse=True):
+    # Check if the function has a docstring.
+    i = bisect.bisect_left(docstrings, start, hi=max_docstring_index)
     if start == end:
       # This is either a one-line function like `def f(): pass` or a function
-      # with no body code, just a docstring. If we find an associated docstring,
-      # use it as the new 'end'.
-      i = bisect.bisect_left(docstrings, start)
-      if i != len(docstrings):
-        end = min(max_line, docstrings[i]) if max_line else docstrings[i]
-    max_line = start
+      # with no body code, just a docstring. If it is the latter, use the
+      # docstring as the new 'end'.
+      if i != max_docstring_index:
+        end = docstrings[i]
+    max_docstring_index = i
 
     for i in range(start, end):
       # Take the first comment we find as the function typecomment.
@@ -404,5 +414,6 @@ def merge_annotations(code, annotations, docstrings):
   return code
 
 
-def process_code(code):
-  return pyc.visit(pyc.visit(code, DisCodeVisitor()), OrderCodeVisitor())
+def process_code(code, python_version):
+  return pyc.visit(
+      pyc.visit(code, DisCodeVisitor()), OrderCodeVisitor(python_version))
