@@ -18,6 +18,7 @@ from pytype.pyc import loadmarshal
 from pytype.pytd import mro
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
+from pytype.pytd.parse import parser_constants
 from pytype.typegraph import cfg
 
 
@@ -30,15 +31,17 @@ MAX_IMPORT_DEPTH = 12
 class Converter(utils.VirtualMachineWeakrefMixin):
   """Functions for creating the classes in abstract.py."""
 
+  unsolvable: abstract.Unsolvable
+
   # Define this error inside Converter so that it is exposed to abstract.py
   class TypeParameterError(Exception):
 
     def __init__(self, type_param_name):
-      super(Converter.TypeParameterError, self).__init__(type_param_name)
+      super().__init__(type_param_name)
       self.type_param_name = type_param_name
 
   def __init__(self, vm):
-    super(Converter, self).__init__(vm)
+    super().__init__(vm)
     self.vm.convert = self  # to make constant_to_value calls below work
     self.pytd_convert = output.Converter(vm)
 
@@ -71,13 +74,14 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     }
     self.primitive_class_names = [
         self._type_to_name(x) for x in self.primitive_classes]
-    self.none = abstract.AbstractOrConcreteValue(
-        None, self.primitive_classes[compat.NoneType], self.vm)
-    self.true = abstract.AbstractOrConcreteValue(
-        True, self.primitive_classes[bool], self.vm)
-    self.false = abstract.AbstractOrConcreteValue(
-        False, self.primitive_classes[bool], self.vm)
-    self.ellipsis = abstract.AbstractOrConcreteValue(
+    self.none = abstract.ConcreteValue(None,
+                                       self.primitive_classes[compat.NoneType],
+                                       self.vm)
+    self.true = abstract.ConcreteValue(True, self.primitive_classes[bool],
+                                       self.vm)
+    self.false = abstract.ConcreteValue(False, self.primitive_classes[bool],
+                                        self.vm)
+    self.ellipsis = abstract.ConcreteValue(
         Ellipsis, self.primitive_classes[compat.EllipsisType], self.vm)
 
     self.primitive_class_instances = {}
@@ -110,8 +114,6 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     self.tuple_type = self.constant_to_value(tuple)
     self.generator_type = self.constant_to_value(types.GeneratorType)
     self.iterator_type = self.constant_to_value(compat.IteratorType)
-    # TODO(ahxun): We should clean up/standardize how we initialize
-    # version-specific attributes.
     if self.vm.python_version >= (3, 5):
       self.coroutine_type = self.constant_to_value(compat.CoroutineType)
       self.awaitable_type = self.constant_to_value(compat.AwaitableType)
@@ -143,29 +145,27 @@ class Converter(utils.VirtualMachineWeakrefMixin):
   def _type_to_name(self, t):
     """Convert a type to its name."""
     assert t.__class__ is type
-    # TODO(rechen): We should use the target version-specific name of the
-    # builtins module rather than hard-coding __builtin__.
     if t is types.FunctionType:
       return "typing.Callable"
     elif t is compat.BytesType:
-      return "__builtin__.bytes"
+      return "builtins.bytes"
     elif t is compat.UnicodeType:
       if self.vm.PY2:
-        return "__builtin__.unicode"
+        return "builtins.unicode"
       else:
-        return "__builtin__.str"
+        return "builtins.str"
     elif t is compat.OldStyleClassType:
-      return "__builtin__.classobj"
+      return "builtins.classobj"
     elif t is compat.IteratorType:
-      return "__builtin__.object"
+      return "builtins.object"
     elif t is compat.CoroutineType:
-      return "__builtin__.coroutine"
+      return "builtins.coroutine"
     elif t is compat.AwaitableType:
       return "typing.Awaitable"
     elif t is compat.AsyncGeneratorType:
-      return "__builtin__.asyncgenerator"
+      return "builtins.asyncgenerator"
     else:
-      return "__builtin__." + t.__name__
+      return "builtins." + t.__name__
 
   def value_to_constant(self, val, constant_type):
     if (isinstance(val, mixin.PythonConstant) and
@@ -181,7 +181,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     else:
       pytd_cls = ast.Lookup(name)
     subst = subst or datatypes.AliasingDict()
-    return self.constant_to_value(pytd_cls, subst, self.vm.root_cfg_node)
+    return self.constant_to_value(pytd_cls, subst, self.vm.root_node)
 
   def tuple_to_value(self, content):
     """Create a VM tuple from the given sequence."""
@@ -220,10 +220,19 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     return var
 
   def build_slice(self, node, start, stop, step=None):
-    del start
-    del stop
-    del step
-    return self.primitive_class_instances[slice].to_variable(node)
+    const_types = (int, type(None))
+    try:
+      if start:
+        start = abstract_utils.get_atomic_python_constant(start, const_types)
+      if stop:
+        stop = abstract_utils.get_atomic_python_constant(stop, const_types)
+      if step:
+        step = abstract_utils.get_atomic_python_constant(step, const_types)
+    except abstract_utils.ConversionError:
+      return self.primitive_class_instances[slice].to_variable(node)
+    return abstract.ConcreteValue(
+        slice(start, stop, step), self.primitive_classes[slice],
+        self.vm).to_variable(node)
 
   def build_list(self, node, content):
     """Create a VM list from the given sequence."""
@@ -256,8 +265,8 @@ class Converter(utils.VirtualMachineWeakrefMixin):
   def get_maybe_abstract_instance(self, data):
     """Get an instance of the same type as the given data, abstract if possible.
 
-    Get an abstract instance of primitive data stored as an
-    AbstractOrConcreteValue. Return any other data as-is. This is used by
+    Get an abstract instance of primitive data stored as a
+    ConcreteValue. Return any other data as-is. This is used by
     constant_to_var to discard concrete values that have been kept
     around for InterpreterFunction.
 
@@ -310,7 +319,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     """Extract the element type of a vararg or kwarg."""
     if not isinstance(arg_type, abstract.ParameterizedClass):
       assert (isinstance(arg_type, mixin.Class) and
-              arg_type.full_name in ("__builtin__.dict", "__builtin__.tuple"))
+              arg_type.full_name in ("builtins.dict", "builtins.tuple"))
       return None
     elif arg_type.base_cls is self.dict_type:
       return arg_type.get_formal_type_parameter(abstract_utils.V)
@@ -329,10 +338,10 @@ class Converter(utils.VirtualMachineWeakrefMixin):
 
   def widen_type(self, container):
     """Widen a tuple to an iterable, or a dict to a mapping."""
-    if container.full_name == "__builtin__.tuple":
+    if container.full_name == "builtins.tuple":
       return self._copy_type_parameters(container, "typing.Iterable")
     else:
-      assert container.full_name == "__builtin__.dict", container.full_name
+      assert container.full_name == "builtins.dict", container.full_name
       return self._copy_type_parameters(container, "typing.Mapping")
 
   def merge_classes(self, instances):
@@ -341,12 +350,12 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     Args:
       instances: An iterable of instances.
     Returns:
-      An abstract.AtomicAbstractValue created by merging the instances' classes.
+      An abstract.BaseValue created by merging the instances' classes.
     """
     classes = set()
     for v in instances:
       cls = v.get_class()
-      if cls:
+      if cls and cls != self.empty:
         classes.add(cls)
     return self.vm.merge_values(classes)
 
@@ -374,9 +383,9 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       ValueError: if pytype is not of a known type.
     """
     source_sets = source_sets or [[]]
-    node = node or self.vm.root_cfg_node
+    node = node or self.vm.root_node
     if isinstance(pyval, pytd.NothingType):
-      return self.vm.program.NewVariable([], [], self.vm.root_cfg_node)
+      return self.vm.program.NewVariable([], [], self.vm.root_node)
     elif isinstance(pyval, pytd.Alias):
       return self.constant_to_var(pyval.type, subst, node, source_sets,
                                   discard_concrete_values)
@@ -416,20 +425,16 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     # e.g. a list of values instead of a list of types:
     assert pyval.__class__ != cfg.Variable, pyval
     if pyval.__class__ == tuple:
-      # TODO(ampere): This does not allow subclasses. Handle namedtuple
-      # correctly.
       # This case needs to go at the end because many things are actually also
       # tuples.
-      return self.build_tuple(
-          self.vm.root_cfg_node,
-          (self.constant_to_var(v, subst, node, source_sets,
-                                discard_concrete_values)
-           for i, v in enumerate(pyval)))
+      return self.build_tuple(self.vm.root_node, (self.constant_to_var(
+          v, subst, node, source_sets, discard_concrete_values)
+                                                  for i, v in enumerate(pyval)))
     raise ValueError(
         "Cannot convert {} to an abstract value".format(pyval.__class__))
 
   def constant_to_value(self, pyval, subst=None, node=None):
-    """Like constant_to_var, but convert to an abstract.AtomicAbstractValue.
+    """Like constant_to_var, but convert to an abstract.BaseValue.
 
     This also memoizes the results.  We don't memoize on name, as builtin types
     like str or list might be reinitialized under different names (e.g. "param
@@ -443,10 +448,11 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       pyval: The constant to convert.
       subst: The current type parameters.
       node: The current CFG node. (For instances)
+
     Returns:
-      The converted constant. (Instance of AtomicAbstractValue)
+      The converted constant. (Instance of BaseValue)
     """
-    node = node or self.vm.root_cfg_node
+    node = node or self.vm.root_node
     key = ("constant", pyval, type(pyval))
     if key in self._convert_cache:
       if self._convert_cache[key] is None:
@@ -463,7 +469,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         need_node[0] = True
         return node
       value = self._constant_to_value(pyval, subst, get_node)
-      if not need_node[0] or node is self.vm.root_cfg_node:
+      if not need_node[0] or node is self.vm.root_node:
         # Values that contain a non-root node cannot be cached. Otherwise,
         # we'd introduce bugs such as the following:
         #   if <condition>:
@@ -484,7 +490,6 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       ast = self.vm.loader.import_name(module)
       if ast is not None:
         try:
-          # TODO(kramm): Should this use pytd.py:ToType?
           cls = ast.Lookup(late_type.name)
         except KeyError:
           try:
@@ -508,15 +513,23 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     return abstract.Module(self.vm, ast.name, members, ast)
 
   def _get_literal_value(self, pyval):
-    if pyval == self.vm.lookup_builtin("__builtin__.True"):
+    if pyval == self.vm.lookup_builtin("builtins.True"):
       return True
-    elif pyval == self.vm.lookup_builtin("__builtin__.False"):
+    elif pyval == self.vm.lookup_builtin("builtins.False"):
       return False
+    elif isinstance(pyval, str):
+      prefix, value = parser_constants.STRING_RE.match(pyval).groups()[:2]
+      value = value[1:-1]  # remove quotation marks
+      if "b" in prefix and not self.vm.PY2:
+        value = compat.bytestring(value)
+      elif "u" in prefix and self.vm.PY2:
+        value = compat.UnicodeType(value)
+      return value
     else:
       return pyval
 
   def _constant_to_value(self, pyval, subst, get_node):
-    """Create a AtomicAbstractValue that represents a python constant.
+    """Create a BaseValue that represents a python constant.
 
     This supports both constant from code constant pools and PyTD constants such
     as classes. This also supports builtin python objects such as int and float.
@@ -525,6 +538,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       pyval: The python or PyTD value to convert.
       subst: The current type parameters.
       get_node: A getter function for the current node.
+
     Returns:
       A Value that represents the constant, or None if we couldn't convert.
     Raises:
@@ -535,25 +549,26 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       # We use a subclass of str, compat.BytesPy3, to mark Python 3
       # bytestrings, which are converted to abstract bytes instances.
       # compat.BytesType dispatches to this when appropriate.
-      return abstract.AbstractOrConcreteValue(pyval, self.str_type, self.vm)
+      return abstract.ConcreteValue(pyval, self.str_type, self.vm)
     elif isinstance(pyval, compat.UnicodeType):
-      return abstract.AbstractOrConcreteValue(pyval, self.unicode_type, self.vm)
+      return abstract.ConcreteValue(pyval, self.unicode_type, self.vm)
     elif isinstance(pyval, compat.BytesType):
-      return abstract.AbstractOrConcreteValue(pyval, self.bytes_type, self.vm)
+      return abstract.ConcreteValue(pyval, self.bytes_type, self.vm)
     elif isinstance(pyval, bool):
       return self.true if pyval else self.false
     elif isinstance(pyval, int) and -1 <= pyval <= MAX_IMPORT_DEPTH:
       # For small integers, preserve the actual value (for things like the
       # level in IMPORT_NAME).
-      return abstract.AbstractOrConcreteValue(pyval, self.int_type, self.vm)
+      return abstract.ConcreteValue(pyval, self.int_type, self.vm)
     elif isinstance(pyval, compat.LongType):
       # long is aliased to int
       return self.primitive_class_instances[int]
     elif pyval.__class__ in self.primitive_classes:
       return self.primitive_class_instances[pyval.__class__]
     elif isinstance(pyval, (loadmarshal.CodeType, blocks.OrderedCode)):
-      return abstract.AbstractOrConcreteValue(
-          pyval, self.primitive_classes[types.CodeType], self.vm)
+      return abstract.ConcreteValue(pyval,
+                                    self.primitive_classes[types.CodeType],
+                                    self.vm)
     elif pyval is super:
       return special_builtins.Super(self.vm)
     elif pyval is object:
@@ -573,9 +588,9 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       mod = self.vm.loader.import_name(pyval.module_name)
       return self._create_module(mod)
     elif isinstance(pyval, pytd.Class):
-      if pyval.name == "__builtin__.super":
+      if pyval.name == "builtins.super":
         return self.vm.special_builtins["super"]
-      elif pyval.name == "__builtin__.object":
+      elif pyval.name == "builtins.object":
         return self.object_type
       elif pyval.name == "types.ModuleType":
         return self.module_type
@@ -583,6 +598,8 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         # Python 3's typeshed uses a stub file indirection to define ModuleType
         # even though it is exported via types.pyi.
         return self.module_type
+      elif pyval.name == "types.FunctionType":
+        return self.function_type
       else:
         module, dot, base_name = pyval.name.rpartition(".")
         # typing.TypingContainer intentionally loads the underlying pytd types.
@@ -603,7 +620,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     elif isinstance(pyval, pytd.Function):
       signatures = [function.PyTDSignature(pyval.name, sig, self.vm)
                     for sig in pyval.signatures]
-      type_new = self.vm.lookup_builtin("__builtin__.type").Lookup("__new__")
+      type_new = self.vm.lookup_builtin("builtins.type").Lookup("__new__")
       if pyval is type_new:
         f_cls = special_builtins.TypeNew
       else:
@@ -613,7 +630,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       return f
     elif isinstance(pyval, pytd.ClassType):
       assert pyval.cls
-      return self.constant_to_value(pyval.cls, subst, self.vm.root_cfg_node)
+      return self.constant_to_value(pyval.cls, subst, self.vm.root_node)
     elif isinstance(pyval, pytd.NothingType):
       return self.empty
     elif isinstance(pyval, pytd.AnythingType):
@@ -622,21 +639,30 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           isinstance(pyval.type, pytd.AnythingType)):
       # We allow "X = ... # type: Any" to declare X as a type.
       return self.unsolvable
+    elif (isinstance(pyval, pytd.Constant) and
+          isinstance(pyval.type, pytd.GenericType) and
+          pyval.type.base_type.name == "builtins.type"):
+      # `X: Type[other_mod.X]` is equivalent to `X = other_mod.X`.
+      param, = pyval.type.parameters
+      return self.constant_to_value(param, subst, self.vm.root_node)
     elif isinstance(pyval, pytd.FunctionType):
-      return self.constant_to_value(
-          pyval.function, subst, self.vm.root_cfg_node)
+      return self.constant_to_value(pyval.function, subst, self.vm.root_node)
     elif isinstance(pyval, pytd.UnionType):
-      options = [self.constant_to_value(t, subst, self.vm.root_cfg_node)
-                 for t in pyval.type_list]
+      options = [
+          self.constant_to_value(t, subst, self.vm.root_node)
+          for t in pyval.type_list
+      ]
       if len(options) > 1:
         return abstract.Union(options, self.vm)
       else:
         return options[0]
     elif isinstance(pyval, pytd.TypeParameter):
-      constraints = tuple(self.constant_to_value(c, {}, self.vm.root_cfg_node)
-                          for c in pyval.constraints)
-      bound = (pyval.bound and
-               self.constant_to_value(pyval.bound, {}, self.vm.root_cfg_node))
+      constraints = tuple(
+          self.constant_to_value(c, {}, self.vm.root_node)
+          for c in pyval.constraints)
+      bound = (
+          pyval.bound and
+          self.constant_to_value(pyval.bound, {}, self.vm.root_node))
       return abstract.TypeParameter(
           pyval.name, self.vm, constraints=constraints,
           bound=bound, module=pyval.scope)
@@ -653,7 +679,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           cls.base_type.name == "typing.ClassVar"):
         param, = cls.parameters
         return self.constant_to_value(
-            abstract_utils.AsInstance(param), subst, self.vm.root_cfg_node)
+            abstract_utils.AsInstance(param), subst, self.vm.root_node)
       elif isinstance(cls, pytd.GenericType) or (isinstance(cls, pytd.Class) and
                                                  cls.template):
         # If we're converting a generic Class, need to create a new instance of
@@ -671,25 +697,24 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           assert isinstance(cls.base_type, pytd.ClassType)
           base_cls = cls.base_type.cls
         assert isinstance(base_cls, pytd.Class), base_cls
-        if base_cls.name == "__builtin__.type":
+        if base_cls.name == "builtins.type":
           c, = cls.parameters
           if isinstance(c, pytd.TypeParameter):
             if not subst or c.full_name not in subst:
               raise self.TypeParameterError(c.full_name)
             return self.merge_classes(subst[c.full_name].data)
           else:
-            return self.constant_to_value(c, subst, self.vm.root_cfg_node)
+            return self.constant_to_value(c, subst, self.vm.root_node)
         elif isinstance(cls, pytd.TupleType):
           content = tuple(self.constant_to_var(abstract_utils.AsInstance(p),
                                                subst, get_node())
                           for p in cls.parameters)
           return abstract.Tuple(content, self.vm)
         elif isinstance(cls, pytd.CallableType):
-          clsval = self.constant_to_value(cls, subst, self.vm.root_cfg_node)
+          clsval = self.constant_to_value(cls, subst, self.vm.root_node)
           return abstract.Instance(clsval, self.vm)
         else:
-          clsval = self.constant_to_value(
-              base_cls, subst, self.vm.root_cfg_node)
+          clsval = self.constant_to_value(base_cls, subst, self.vm.root_node)
           instance = abstract.Instance(clsval, self.vm)
           num_params = len(cls.parameters)
           assert num_params <= len(base_cls.template)
@@ -700,7 +725,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
                   abstract_utils.AsInstance(cls.parameters[i]), subst, node)
             else:
               # An omitted type parameter implies `Any`.
-              node = self.vm.root_cfg_node
+              node = self.vm.root_node
               p = self.unsolvable.to_variable(node)
             instance.merge_instance_type_parameter(node, formal.name, p)
           return instance
@@ -709,24 +734,24 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         # This key is also used in __init__
         key = (abstract.Instance, cls)
         if key not in self._convert_cache:
-          if cls.name in ["__builtin__.type", "__builtin__.property"]:
+          if cls.name in ["builtins.type", "builtins.property"]:
             # An instance of "type" or of an anonymous property can be anything.
             instance = self._create_new_unknown_value("type")
           else:
-            mycls = self.constant_to_value(cls, subst, self.vm.root_cfg_node)
+            mycls = self.constant_to_value(cls, subst, self.vm.root_node)
             instance = abstract.Instance(mycls, self.vm)
           log.info("New pytd instance for %s: %r", cls.name, instance)
           self._convert_cache[key] = instance
         return self._convert_cache[key]
       elif isinstance(cls, pytd.Literal):
         return self.constant_to_value(
-            self._get_literal_value(cls.value), subst, self.vm.root_cfg_node)
+            self._get_literal_value(cls.value), subst, self.vm.root_node)
       else:
-        return self.constant_to_value(cls, subst, self.vm.root_cfg_node)
+        return self.constant_to_value(cls, subst, self.vm.root_node)
     elif (isinstance(pyval, pytd.GenericType) and
           pyval.base_type.name == "typing.ClassVar"):
       param, = pyval.parameters
-      return self.constant_to_value(param, subst, self.vm.root_cfg_node)
+      return self.constant_to_value(param, subst, self.vm.root_node)
     elif isinstance(pyval, pytd.GenericType):
       if isinstance(pyval.base_type, pytd.LateType):
         actual = self._load_late_type(pyval.base_type)
@@ -734,18 +759,18 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           return self.unsolvable
         base = actual.cls
       else:
-        assert isinstance(pyval.base_type, pytd.ClassType)
+        assert isinstance(pyval.base_type, pytd.ClassType), pyval
         base = pyval.base_type.cls
       assert isinstance(base, pytd.Class), base
-      base_cls = self.constant_to_value(
-          base, subst, self.vm.root_cfg_node)
+      base_cls = self.constant_to_value(base, subst, self.vm.root_node)
       if not isinstance(base_cls, mixin.Class):
         # base_cls can be, e.g., an unsolvable due to an mro error.
         return self.unsolvable
       if isinstance(pyval, pytd.TupleType):
         abstract_class = abstract.TupleClass
         template = list(range(len(pyval.parameters))) + [abstract_utils.T]
-        parameters = pyval.parameters + (pytd.UnionType(pyval.parameters),)
+        combined_parameter = pytd_utils.JoinTypes(pyval.parameters)
+        parameters = pyval.parameters + (combined_parameter,)
       elif isinstance(pyval, pytd.CallableType):
         abstract_class = abstract.CallableClass
         template = list(range(len(pyval.args))) + [abstract_utils.ARGS,
@@ -768,13 +793,13 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       return abstract_class(base_cls, type_parameters, self.vm)
     elif isinstance(pyval, pytd.Literal):
       value = self.constant_to_value(
-          self._get_literal_value(pyval.value), subst, self.vm.root_cfg_node)
-      return abstract.LiteralClass(
-          self.name_to_value("typing.Literal"), value, self.vm)
+          self._get_literal_value(pyval.value), subst, self.vm.root_node)
+      return abstract.LiteralClass(value, self.vm)
     elif pyval.__class__ is tuple:  # only match raw tuple, not namedtuple/Node
-      return self.tuple_to_value([self.constant_to_var(item, subst,
-                                                       self.vm.root_cfg_node)
-                                  for i, item in enumerate(pyval)])
+      return self.tuple_to_value([
+          self.constant_to_var(item, subst, self.vm.root_node)
+          for i, item in enumerate(pyval)
+      ])
     else:
       raise NotImplementedError("Can't convert constant %s %r" %
                                 (type(pyval), pyval))
