@@ -1,4 +1,3 @@
-# Lint as: python3
 """Code and data structures for storing and displaying errors."""
 
 import collections
@@ -7,13 +6,14 @@ import csv
 import logging
 import re
 import sys
-from typing import Optional, Union
+from typing import Iterable, Optional, Union
 
 from pytype import abstract
 from pytype import debug
 from pytype import function
 from pytype import mixin
 from pytype import utils
+from pytype.pytd import escape
 from pytype.pytd import optimize
 from pytype.pytd import pytd_utils
 from pytype.pytd import slots
@@ -147,7 +147,7 @@ def _compare_traceback_strings(left, right):
 
 
 def _function_name(name, capitalize=False):
-  builtin_prefix = "__builtin__."
+  builtin_prefix = "builtins."
   if name.startswith(builtin_prefix):
     ret = "built-in function %s" % name[len(builtin_prefix):]
   else:
@@ -158,7 +158,7 @@ def _function_name(name, capitalize=False):
     return ret
 
 
-class CheckPoint(object):
+class CheckPoint:
   """Represents a position in an error log."""
 
   def __init__(self, errors):
@@ -171,7 +171,7 @@ class CheckPoint(object):
     self._errorlog_errors[:] = self._errorlog_errors[:self._position]
 
 
-class Error(object):
+class Error:
   """Representation of an error in the error log.
 
   Attributes:
@@ -325,7 +325,7 @@ class Error(object):
           traceback=None)
 
 
-class ErrorLogBase(object):
+class ErrorLogBase:
   """A stream of errors."""
 
   def __init__(self):
@@ -395,13 +395,13 @@ class ErrorLogBase(object):
     _log.info("Restored errorlog to checkpoint: %d errors reverted",
               len(checkpoint.errors))
 
-  def print_to_csv_file(self, filename):
+  def print_to_csv_file(self, filename, open_function=open):
     """Print the errorlog to a csv file."""
-    with open(filename, "w") as f:
+    with open_function(filename, "w") as f:
       csv_file = csv.writer(f, delimiter=",")
       for error in self.unique_sorted_errors():
         # pylint: disable=protected-access
-        # TODO(kramm): Add _methodname
+        # TODO(b/159038861): Add _methodname
         if error._details and error._traceback:
           details = error._details + "\n\n" + error._traceback
         elif error._traceback:
@@ -472,8 +472,8 @@ class ErrorLog(ErrorLogBase):
         pytd_type.Visit(visitors.RemoveUnknownClasses()))))
     # Clean up autogenerated namedtuple names, e.g. "namedtuple-X-a-_0-c"
     # becomes just "X", by extracting out just the type name.
-    if "namedtuple-" in name:
-      return re.sub(r"\bnamedtuple-([^-]+)-[-_\w]*", r"\1", name)
+    if "namedtuple" in name:
+      return escape.unpack_namedtuple(name)
     nested_class_match = re.search(r"_(?:\w+)_DOT_", name)
     if nested_class_match:
       # Pytype doesn't have true support for nested classes. Instead, for
@@ -489,13 +489,14 @@ class ErrorLog(ErrorLogBase):
       return name[:start] + name[start+1:].replace("_DOT_", ".")
     return name
 
-  def _print_as_expected_type(self, t, instance=None):
+  def _print_as_expected_type(self, t: abstract.BaseValue, instance=None):
     """Print abstract value t as a pytd type."""
     if t.is_late_annotation():
       return t.expr
     elif isinstance(t, (abstract.Unknown, abstract.Unsolvable, mixin.Class,
                         abstract.Union)):
-      with t.vm.convert.pytd_convert.produce_detailed_output():
+      with t.vm.convert.pytd_convert.set_output_mode(
+          t.vm.convert.pytd_convert.OutputMode.DETAILED):
         return self._pytd_print(t.get_instance_type(instance=instance))
     elif (isinstance(t, mixin.PythonConstant) and
           not getattr(t, "could_contain_anything", False)):
@@ -506,14 +507,43 @@ class ErrorLog(ErrorLogBase):
     else:
       return "<instance of %s>" % self._print_as_expected_type(t.cls, t)
 
-  def _print_as_actual_type(self, t):
-    with t.vm.convert.pytd_convert.produce_detailed_output():
+  def _print_as_actual_type(self, t, literal=False):
+    if literal:
+      output_mode = t.vm.convert.pytd_convert.OutputMode.LITERAL
+    else:
+      output_mode = t.vm.convert.pytd_convert.OutputMode.DETAILED
+    with t.vm.convert.pytd_convert.set_output_mode(output_mode):
       return self._pytd_print(t.to_type())
 
-  def _print_as_return_type(self, t):
-    ret = self._pytd_print(t)
+  def _print_as_generic_type(self, t):
+    generic = pytd_utils.MakeClassOrContainerType(
+        t.get_instance_type().base_type,
+        t.formal_type_parameters.keys(),
+        False)
+    with t.vm.convert.pytd_convert.set_output_mode(
+        t.vm.convert.pytd_convert.OutputMode.DETAILED):
+      return self._pytd_print(generic)
+
+  def _print_as_return_types(self, node, formal, actual, bad):
+    """Print the actual and expected values for a return type."""
+    convert = formal.vm.convert.pytd_convert
+    with convert.set_output_mode(convert.OutputMode.DETAILED):
+      expected = self._pytd_print(formal.get_instance_type(node))
+    if "Literal[" in expected:
+      output_mode = convert.OutputMode.LITERAL
+    else:
+      output_mode = convert.OutputMode.DETAILED
+    with convert.set_output_mode(output_mode):
+      bad_actual = self._pytd_print(pytd_utils.JoinTypes(
+          view[actual].data.to_type(node, view=view) for view in bad))
+      if len(actual.bindings) > len(bad):
+        full_actual = self._pytd_print(pytd_utils.JoinTypes(
+            v.to_type(node) for v in actual.data))
+      else:
+        full_actual = bad_actual
     # typing.NoReturn is a prettier alias for nothing.
-    return "NoReturn" if ret == "nothing" else ret
+    fmt = lambda ret: "NoReturn" if ret == "nothing" else ret
+    return fmt(expected), fmt(bad_actual), fmt(full_actual)
 
   def _join_printed_types(self, types):
     """Pretty-print the union of the printed types."""
@@ -551,7 +581,7 @@ class ErrorLog(ErrorLogBase):
         suffix = ": " + type_str + suffix
       yield prefix, name, suffix
 
-  def _iter_actual(self, sig, passed_args, bad_param):
+  def _iter_actual(self, sig, passed_args, bad_param, literal):
     """Yield the prefix, name and type information for actual parameters."""
     # We want to display the passed_args in the order they're defined in the
     # signature, unless there are starargs or starstarargs.
@@ -567,7 +597,7 @@ class ErrorLog(ErrorLogBase):
       return keys.get(arg_name, len(keys)+1)
     for name, arg in sorted(passed_args, key=key_f):
       if bad_param and name == bad_param.name:
-        suffix = ": " + self._print_as_actual_type(arg)
+        suffix = ": " + self._print_as_actual_type(arg, literal=literal)
       else:
         suffix = ""
       yield "", name, suffix
@@ -676,8 +706,9 @@ class ErrorLog(ErrorLogBase):
     """Log an invalid parameters error."""
     sig, passed_args, bad_param = bad_call
     expected = self._print_args(self._iter_expected(sig, bad_param), bad_param)
+    literal = "Literal[" in expected
     actual = self._print_args(
-        self._iter_actual(sig, passed_args, bad_param), bad_param)
+        self._iter_actual(sig, passed_args, bad_param, literal), bad_param)
     details = [
         "       Expected: (", expected, ")\n",
         "Actually passed: (", actual,
@@ -831,19 +862,23 @@ class ErrorLog(ErrorLogBase):
     self.error(stack, "Invalid base class: %s" % base_cls, keyword=base_cls)
 
   @_error_name("bad-return-type")
-  def bad_return_type(self, stack, actual_pytd, expected_pytd):
-    details = "".join([
-        "         Expected: ", self._print_as_return_type(expected_pytd), "\n",
-        "Actually returned: ", self._print_as_return_type(actual_pytd),
-    ])
-    self.error(stack, "bad option in return type", details)
+  def bad_return_type(self, stack, node, formal, actual, bad):
+    """Logs a [bad-return-type] error."""
+    expected, bad_actual, full_actual = self._print_as_return_types(
+        node, formal, actual, bad)
+    if full_actual == bad_actual:
+      message = "bad return type"
+    else:
+      message = f"bad option {bad_actual!r} in return type"
+    details = "".join(["         Expected: ", expected, "\n",
+                       "Actually returned: ", full_actual])
+    self.error(stack, message, details)
 
   @_error_name("bad-concrete-type")
-  def bad_concrete_type(self, stack, actual_pytd, expected_pytd):
-    details = "".join([
-        "       Expected: ", self._print_as_return_type(expected_pytd), "\n",
-        "Actually passed: ", self._print_as_return_type(actual_pytd),
-    ])
+  def bad_concrete_type(self, stack, node, formal, actual, bad):
+    expected, actual, _ = self._print_as_return_types(node, formal, actual, bad)
+    details = "".join(["       Expected: ", expected, "\n",
+                       "Actually passed: ", actual])
     self.error(stack, "Invalid instantiation of generic class", details)
 
   def unsupported_operands(self, stack, operator, var1, var2):
@@ -866,10 +901,12 @@ class ErrorLog(ErrorLogBase):
             " and ".join(repr(operand) for operand in operands)),
         details=details.get("details"))
 
-  def invalid_annotation(
-      self, stack, annot: Optional[Union[str, abstract.AtomicAbstractValue]],
-      details=None, name=None):
-    if isinstance(annot, abstract.AtomicAbstractValue):
+  def invalid_annotation(self,
+                         stack,
+                         annot: Optional[Union[str, abstract.BaseValue]],
+                         details=None,
+                         name=None):
+    if isinstance(annot, abstract.BaseValue):
       annot = self._print_as_expected_type(annot)
     self._invalid_annotation(stack, annot, details, name)
 
@@ -881,8 +918,17 @@ class ErrorLog(ErrorLogBase):
           container_name)
       self._invalid_annotation(stack, "Ellipsis", details, None)
 
-  def ambiguous_annotation(self, stack, options, name=None):
-    desc = " or ".join(sorted(self._print_as_expected_type(o) for o in options))
+  def ambiguous_annotation(
+      self,
+      stack,
+      options: Optional[Union[str, Iterable[abstract.BaseValue]]],
+      name=None):
+    """Log an ambiguous annotation."""
+    if isinstance(options, (str, type(None))):
+      desc = options
+    else:
+      desc = " or ".join(
+          sorted(self._print_as_expected_type(o) for o in options))
     self._invalid_annotation(stack, desc, "Must be constant", name)
 
   @_error_name("invalid-annotation")
@@ -1000,18 +1046,55 @@ class ErrorLog(ErrorLogBase):
     if annot is None:
       return
     annot_string = self._print_as_expected_type(annot)
-    actual_string = self._print_as_actual_type(binding.data)
+    literal = "Literal[" in annot_string
+    actual_string = self._print_as_actual_type(binding.data, literal=literal)
     details = ("Annotation: %s\n" % annot_string +
                "Assignment: %s" % actual_string)
     if len(binding.variable.bindings) > 1:
       # Joining the printed types rather than merging them before printing
       # ensures that we print all of the options when 'Any' is among them.
-      details += "\nIn assignment of type: %s" % self._join_printed_types(
-          self._print_as_actual_type(v) for v in binding.variable.data)
+      # We don't need to print this if there is only 1 unique type.
+      print_types = set(self._print_as_actual_type(v, literal=literal)
+                        for v in binding.variable.data)
+      if len(print_types) > 1:
+        details += "\nIn assignment of type: %s" % self._join_printed_types(
+            print_types)
     suffix = "" if name is None else " for " + name
     err_msg = "Type annotation%s does not match type of assignment" % suffix
     self.error(stack, err_msg, details=details)
 
+  @_error_name("container-type-mismatch")
+  def container_type_mismatch(self, stack, obj, mutations, name):
+    """Invalid combination of annotation and mutation.
+
+    Args:
+      stack: the frame stack
+      obj: the container instance being mutated
+      mutations: a dict of {parameter name: (annotated types, new types)}
+      name: the variable name (or None)
+    """
+    cls = obj.cls
+    annot_string = "%s (type parameters %s)" % (
+        self._print_as_expected_type(cls),
+        self._print_as_generic_type(cls))
+    details = "Annotation: %s\n" % annot_string
+    contained = ""
+    new_contained = ""
+    for formal in cls.formal_type_parameters.keys():
+      if formal in mutations:
+        params, values, _ = mutations[formal]
+        old_content = self._join_printed_types(
+            set(self._print_as_actual_type(v) for v in params.data))
+        new_content = self._join_printed_types(
+            set(self._print_as_actual_type(v) for v in values.data))
+        contained += "  %s: %s\n" % (formal, old_content)
+        new_contained += "  %s: %s\n" % (formal, new_content)
+    details += ("Contained types:\n" + contained +
+                "New contained types:\n" + new_contained)
+    suffix = "" if name is None else " for " + name
+    err_msg = "New container type%s does not match type annotation" % suffix
+    self.error(stack, err_msg, details=details)
+
   @_error_name("invalid-function-definition")
   def invalid_function_definition(self, stack, msg):
     """Invalid function constructed via metaprogramming."""
