@@ -10,8 +10,6 @@ from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
 
-import six
-
 log = logging.getLogger(__name__)
 
 
@@ -24,7 +22,7 @@ def _print(t):
   return pytd_utils.Print(t.get_instance_type())
 
 
-class Signature(object):
+class Signature:
   """Representation of a Python function signature.
 
   Attributes:
@@ -55,7 +53,7 @@ class Signature(object):
     self.annotations = annotations
     self.excluded_types = set()
     if postprocess_annotations:
-      for k, annot in six.iteritems(self.annotations):
+      for k, annot in self.annotations.items():
         self.annotations[k] = self._postprocess_annotation(k, annot)
 
   @property
@@ -64,7 +62,7 @@ class Signature(object):
 
   @property
   def has_param_annotations(self):
-    return bool(six.viewkeys(self.annotations) - {"return"})
+    return bool(self.annotations.keys() - {"return"})
 
   def add_scope(self, module):
     """Add scope for type parameters in annotations."""
@@ -93,7 +91,7 @@ class Signature(object):
     c = collections.Counter()
     for annot in self.annotations.values():
       c.update(annot.vm.annotations_util.get_type_parameters(annot))
-    for param, count in six.iteritems(c):
+    for param, count in c.items():
       if param.name in self.excluded_types:
         # skip all the type parameters in `excluded_types`
         continue
@@ -127,7 +125,8 @@ class Signature(object):
     pytd_annotations.append(("return", sig.return_type))
     def param_to_var(p):
       return vm.convert.constant_to_var(
-          p.type, subst=datatypes.AliasingDict(), node=vm.root_cfg_node)
+          p.type, subst=datatypes.AliasingDict(), node=vm.root_node)
+
     return cls(
         name=name,
         param_names=tuple(p.name for p in sig.params if not p.kwonly),
@@ -135,9 +134,11 @@ class Signature(object):
         kwonly_params=set(p.name for p in sig.params if p.kwonly),
         kwargs_name=None if sig.starstarargs is None else sig.starstarargs.name,
         defaults={p.name: param_to_var(p) for p in sig.params if p.optional},
-        annotations={name: vm.convert.constant_to_value(
-            typ, subst=datatypes.AliasingDict(), node=vm.root_cfg_node)
-                     for name, typ in pytd_annotations},
+        annotations={
+            name: vm.convert.constant_to_value(
+                typ, subst=datatypes.AliasingDict(), node=vm.root_node)
+            for name, typ in pytd_annotations
+        },
         postprocess_annotations=False,
     )
 
@@ -302,10 +303,21 @@ class Args(collections.namedtuple(
     """
     assert isinstance(posargs, tuple), posargs
     cls.replace = cls._replace
-    return super(Args, cls).__new__(
-        cls, posargs=posargs, namedargs=namedargs or {}, starargs=starargs,
+    return super().__new__(
+        cls,
+        posargs=posargs,
+        namedargs=namedargs or {},
+        starargs=starargs,
         starstarargs=starstarargs)
 
+  def is_empty(self):
+    if self.posargs or self.starargs or self.starstarargs:
+      return False
+    if isinstance(self.namedargs, dict):
+      return not self.namedargs
+    else:
+      return not self.namedargs.pyval
+
   def starargs_as_tuple(self, node, vm):
     try:
       args = self.starargs and abstract_utils.get_atomic_python_constant(
@@ -325,6 +337,69 @@ class Args(collections.namedtuple(
       args = None
     return args
 
+  def _unpack_and_match_args(self, node, vm, match_signature, starargs_tuple):
+    """Match args against a signature with unpacking."""
+
+    posargs = self.posargs
+    # As we have the function signature we will attempt to adjust the
+    # starargs into the missing posargs.
+    pre = []
+    post = []
+    stars = collections.deque(starargs_tuple)
+    while stars and not abstract_utils.is_var_splat(stars[0]):
+      pre.append(stars.popleft())
+    while stars and not abstract_utils.is_var_splat(stars[-1]):
+      post.append(stars.pop())
+    post.reverse()
+    n_matched = len(posargs) + len(pre) + len(post)
+    posarg_delta = len(match_signature.param_names) - n_matched
+    if stars and not post:
+      # If the invocation ends with `*args`, return it to match against *args in
+      # the function signature. For f(<k args>, *xs, ..., *ys), transform to
+      # f(<k args>, *ys) since ys is an indefinite tuple anyway and will match
+      # against all remaining posargs.
+      return posargs + tuple(pre), abstract_utils.unwrap_splat(stars[-1])
+    elif posarg_delta <= len(stars):
+      # We have too many args; don't do *xs expansion. Go back to matching from
+      # the start and treat every entry in starargs_tuple as length 1.
+      n_params = len(match_signature.param_names)
+      all_args = posargs + starargs_tuple
+      # Don't unwrap splats here because f(*xs, y) is not the same as f(xs, y).
+      # TODO(mdemello): Ideally, since we are matching call f(*xs, y) against
+      # sig f(x, y) we should raise an error here.
+      pos = _splats_to_any(all_args[:n_params], vm)
+      star = []
+      for var in all_args[n_params:]:
+        if abstract_utils.is_var_splat(var):
+          star.append(
+              abstract_utils.merged_type_parameter(node, var, abstract_utils.T))
+        else:
+          star.append(var)
+      if star:
+        return pos, vm.convert.tuple_to_value(star).to_variable(node)
+      else:
+        return pos, None
+    elif stars:
+      if len(stars) == 1:
+        # Special case (*xs, <post>) to fill in the type of xs in every arg
+        p = abstract_utils.merged_type_parameter(
+            node, stars[0], abstract_utils.T)
+        if not p.bindings:
+          # TODO(b/159052609): This shouldn't happen. For some reason,
+          # namedtuple instances don't have any bindings in T; see
+          # tests/py3/test_unpack:TestUnpack.test_unpack_namedtuple.
+          p.AddBinding(vm.convert.unsolvable)
+        mid = [p.AssignToNewVariable(node) for _ in range(posarg_delta)]
+      else:
+        # If we have (*xs, <k args>, *ys) remaining, and more than k+2 params to
+        # match, don't try to match the intermediate params to any range, just
+        # match all k+2 to Any
+        mid = [vm.new_unsolvable(node) for _ in range(posarg_delta)]
+      return posargs + tuple(pre + mid + post), None
+    else:
+      # We have **kwargs but no *args in the invocation
+      return posargs + tuple(pre), None
+
   def simplify(self, node, vm, match_signature=None):
     """Try to insert part of *args, **kwargs into posargs / namedargs."""
     # TODO(rechen): When we have type information about *args/**kwargs,
@@ -336,25 +411,28 @@ class Args(collections.namedtuple(
     starargs_as_tuple = self.starargs_as_tuple(node, vm)
     if starargs_as_tuple is not None:
       if match_signature:
-        # As we have the function signature we will attempt to adjust the
-        # starargs into the missing posargs.
-        missing_posarg_count = len(match_signature.param_names) - len(posargs)
-        starargs_list = list(starargs_as_tuple)
-        for _ in range(missing_posarg_count):
-          if starargs_list:
-            posargs += (starargs_list.pop(0),)
-          else:
-            break
-        starargs = vm.convert.tuple_to_value(starargs_list).to_variable(node)
+        posargs, starargs = self._unpack_and_match_args(
+            node, vm, match_signature, starargs_as_tuple)
+      elif (starargs_as_tuple and
+            abstract_utils.is_var_splat(starargs_as_tuple[-1])):
+        # If the last arg is an indefinite iterable keep it in starargs. Convert
+        # any other splats to Any.
+        # TODO(mdemello): If there are multiple splats should we just fall
+        # through to the next case (setting them all to Any), and only hit this
+        # case for a *single* splat in terminal position?
+        posargs = self.posargs + _splats_to_any(starargs_as_tuple[:-1], vm)
+        starargs = abstract_utils.unwrap_splat(starargs_as_tuple[-1])
       else:
-        posargs += starargs_as_tuple
+        # Don't try to unpack iterables in any other position since we don't
+        # have a signature to match. Just set all splats to Any.
+        posargs = self.posargs + _splats_to_any(starargs_as_tuple, vm)
         starargs = None
     starstarargs_as_dict = self.starstarargs_as_dict()
     if starstarargs_as_dict is not None:
       # Unlike varargs above, we do not adjust starstarargs into namedargs when
       # the function signature has matching param_names because we have not
       # found a benefit in doing so.
-      if namedargs is None:
+      if self.namedargs is None:
         namedargs = starstarargs_as_dict
       else:
         namedargs.update(node, starstarargs_as_dict)
@@ -370,11 +448,11 @@ class Args(collections.namedtuple(
     return variables
 
 
-class ReturnValueMixin(object):
+class ReturnValueMixin:
   """Mixin for exceptions that hold a return node and variable."""
 
   def __init__(self):
-    super(ReturnValueMixin, self).__init__()
+    super().__init__()
     self.return_node = None
     self.return_variable = None
 
@@ -399,7 +477,7 @@ class NotCallable(FailedFunctionCall):
   """For objects that don't have __call__."""
 
   def __init__(self, obj):
-    super(NotCallable, self).__init__()
+    super().__init__()
     self.obj = obj
 
 
@@ -407,7 +485,7 @@ class UndefinedParameterError(FailedFunctionCall):
   """Function called with an undefined variable."""
 
   def __init__(self, name):
-    super(UndefinedParameterError, self).__init__()
+    super().__init__()
     self.name = name
 
 
@@ -415,7 +493,7 @@ class DictKeyMissing(Exception, ReturnValueMixin):
   """When retrieving a key that does not exist in a dict."""
 
   def __init__(self, name):
-    super(DictKeyMissing, self).__init__()
+    super().__init__()
     self.name = name
 
   def __gt__(self, other):
@@ -432,7 +510,7 @@ class InvalidParameters(FailedFunctionCall):
   """Exception for functions called with an incorrect parameter combination."""
 
   def __init__(self, sig, passed_args, vm, bad_param=None):
-    super(InvalidParameters, self).__init__()
+    super().__init__()
     self.name = sig.name
     passed_args = [(name, vm.merge_values(arg.data))
                    for name, arg, _ in sig.iter_args(passed_args)]
@@ -456,7 +534,7 @@ class WrongKeywordArgs(InvalidParameters):
   """E.g. an arg "x" is passed to a function that doesn't have an "x" param."""
 
   def __init__(self, sig, passed_args, vm, extra_keywords):
-    super(WrongKeywordArgs, self).__init__(sig, passed_args, vm)
+    super().__init__(sig, passed_args, vm)
     self.extra_keywords = tuple(extra_keywords)
 
 
@@ -464,7 +542,7 @@ class DuplicateKeyword(InvalidParameters):
   """E.g. an arg "x" is passed to a function as both a posarg and a kwarg."""
 
   def __init__(self, sig, passed_args, vm, duplicate):
-    super(DuplicateKeyword, self).__init__(sig, passed_args, vm)
+    super().__init__(sig, passed_args, vm)
     self.duplicate = duplicate
 
 
@@ -472,7 +550,7 @@ class MissingParameter(InvalidParameters):
   """E.g. a function requires parameter 'x' but 'x' isn't passed."""
 
   def __init__(self, sig, passed_args, vm, missing_parameter):
-    super(MissingParameter, self).__init__(sig, passed_args, vm)
+    super().__init__(sig, passed_args, vm)
     self.missing_parameter = missing_parameter
 # pylint: enable=g-bad-exception-name
 
@@ -496,13 +574,14 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
   """
 
   def __init__(self, name, pytd_sig, vm):
-    super(PyTDSignature, self).__init__(vm)
+    super().__init__(vm)
     self.name = name
     self.pytd_sig = pytd_sig
     self.param_types = [
         self.vm.convert.constant_to_value(
-            p.type, subst=datatypes.AliasingDict(), node=self.vm.root_cfg_node)
-        for p in self.pytd_sig.params]
+            p.type, subst=datatypes.AliasingDict(), node=self.vm.root_node)
+        for p in self.pytd_sig.params
+    ]
     self.signature = Signature.from_pytd(vm, name, pytd_sig)
 
   def _map_args(self, args, view):
@@ -610,7 +689,7 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
     # do that now. The one exception is that Type[X] does not instantiate X, so
     # we do not call X.__init__.
     if (not isinstance(return_type, pytd.GenericType) or
-        return_type.base_type.name != "__builtin__.type"):
+        return_type.base_type.name != "builtins.type"):
       for param in pytd_utils.GetTypeParameters(return_type):
         if param.full_name in subst:
           node = self.vm.call_init(node, subst[param.full_name])
@@ -674,8 +753,7 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
     for formal in self.pytd_sig.params:
       actual = arg_dict[formal.name]
       arg = actual.data
-      if (formal.mutated_type is not None and
-          arg.isinstance_SimpleAbstractValue()):
+      if (formal.mutated_type is not None and arg.isinstance_SimpleValue()):
         if (isinstance(formal.type, pytd.GenericType) and
             isinstance(formal.mutated_type, pytd.GenericType) and
             formal.type.base_type == formal.mutated_type.base_type and
@@ -744,10 +822,17 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
     self.pytd_sig = new_sig
     self.param_types = [
         self.vm.convert.constant_to_value(
-            p.type, subst=datatypes.AliasingDict(), node=self.vm.root_cfg_node)
-        for p in self.pytd_sig.params]
+            p.type, subst=datatypes.AliasingDict(), node=self.vm.root_node)
+        for p in self.pytd_sig.params
+    ]
     self.signature = Signature.from_pytd(self.vm, self.name, self.pytd_sig)
     return self
 
   def __repr__(self):
     return pytd_utils.Print(self.pytd_sig)
+
+
+def _splats_to_any(seq, vm):
+  return tuple(
+      vm.new_unsolvable(vm.root_node) if abstract_utils.is_var_splat(v) else v
+      for v in seq)
