@@ -1,5 +1,10 @@
 """Load and link .pyi files."""
 
+# TODO(b/175443166): pytype cannot infer the type of Loader._modules correctly.
+# Once strict attribute checking is rolled out, we should be able to annotate
+# _modules and remove this disable.
+# pytype: disable=attribute-error
+
 import collections
 import logging
 import os
@@ -8,12 +13,12 @@ from pytype import file_utils
 from pytype import module_utils
 from pytype import utils
 from pytype.pyi import parser
+from pytype.pytd import builtins
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import serialize_ast
 from pytype.pytd import typeshed
 from pytype.pytd import visitors
-from pytype.pytd.parse import builtins
 from six.moves import cPickle
 
 log = logging.getLogger(__name__)
@@ -21,9 +26,10 @@ log = logging.getLogger(__name__)
 
 LOADER_ATTR_TO_CONFIG_OPTION_MAP = {
     "base_module": "module_name",
+    "imports_map": "imports_map",
+    "open_function": "open_function",
     "python_version": "python_version",
     "pythonpath": "pythonpath",
-    "imports_map": "imports_map",
     "use_typeshed": "typeshed",
 }
 
@@ -35,6 +41,10 @@ PICKLE_EXT = ".pickled"
 DEFAULT_PYI_PATH_SUFFIX = None
 
 
+# Always load this module from typeshed, even if we have it in the imports map
+_ALWAYS_PREFER_TYPESHED = frozenset({"typing_extensions"})
+
+
 def is_pickle(filename):
   return os.path.splitext(filename)[1].startswith(PICKLE_EXT)
 
@@ -69,7 +79,7 @@ ResolvedModule = collections.namedtuple(
     "ResolvedModule", ("module_name", "filename", "ast"))
 
 
-class Module(object):
+class Module:
   """Represents a parsed module.
 
   Attributes:
@@ -103,13 +113,13 @@ class BadDependencyError(Exception):
 
   def __init__(self, module_error, src=None):
     referenced = ", referenced from %r" % src if src else ""
-    super(BadDependencyError, self).__init__(module_error + referenced)
+    super().__init__(module_error + referenced)
 
   def __str__(self):
     return utils.message(self)
 
 
-class Loader(object):
+class Loader:
   """A cache for loaded PyTD files.
 
   Typically, you'll have one instance of this class, per module.
@@ -123,6 +133,7 @@ class Loader(object):
     pythonpath: The PYTHONPATH.
     imports_map: A short_path -> full_name mapping for imports.
     use_typeshed: Whether to use https://github.com/python/typeshed.
+    open_function: A custom file opening function.
   """
 
   PREFIX = "pytd:"  # for pytd files that ship with pytype
@@ -133,19 +144,21 @@ class Loader(object):
                pythonpath=(),
                imports_map=None,
                use_typeshed=True,
-               modules=None):
+               modules=None,
+               open_function=open):
     self.python_version = utils.normalize_version(python_version)
     self._modules = modules or self._base_modules(self.python_version)
-    if self._modules["__builtin__"].needs_unpickling():
-      self._unpickle_module(self._modules["__builtin__"])
+    if self._modules["builtins"].needs_unpickling():
+      self._unpickle_module(self._modules["builtins"])
     if self._modules["typing"].needs_unpickling():
       self._unpickle_module(self._modules["typing"])
-    self.builtins = self._modules["__builtin__"].ast
+    self.builtins = self._modules["builtins"].ast
     self.typing = self._modules["typing"].ast
     self.base_module = base_module
     self.pythonpath = pythonpath
     self.imports_map = imports_map
     self.use_typeshed = use_typeshed
+    self.open_function = open_function
     self._concatenated = None
     self._import_name_cache = {}  # performance cache
     self._aliases = {}
@@ -159,14 +172,17 @@ class Loader(object):
     # We assume that the Loader is in a consistent state here. In particular, we
     # assume that for every module in _modules, all the transitive dependencies
     # have been loaded.
-    items = tuple((name, serialize_ast.StoreAst(module.ast))
-                  for name, module in sorted(self._modules.items()))
+    items = tuple(
+        (name, serialize_ast.StoreAst(
+            module.ast, open_function=self.open_function))
+        for name, module in sorted(self._modules.items()))
     # Preparing an ast for pickling clears its class pointers, making it
     # unsuitable for reuse, so we have to discard the builtins cache.
     builtins.InvalidateCache(self.python_version)
     # Now pickle the pickles. We keep the "inner" modules as pickles as a
     # performance optimization - unpickling is slow.
-    pytd_utils.SavePickle(items, filename, compress=True)
+    pytd_utils.SavePickle(
+        items, filename, compress=True, open_function=self.open_function)
 
   def _unpickle_module(self, module):
     raise NotImplementedError()  # overwritten in PickledPyiLoader
@@ -174,8 +190,8 @@ class Loader(object):
   def _base_modules(self, python_version):
     bltins, typing = builtins.GetBuiltinsAndTyping(python_version)
     return {
-        "__builtin__":
-        Module("__builtin__", self.PREFIX + "__builtin__", bltins, dirty=False),
+        "builtins":
+        Module("builtins", self.PREFIX + "builtins", bltins, dirty=False),
         "typing":
         Module("typing", self.PREFIX + "typing", typing, dirty=False)
     }
@@ -237,8 +253,9 @@ class Loader(object):
     if existing:
       return existing
     if not ast:
-      ast = parser.parse_file(filename=filename, name=module_name,
-                              python_version=self.python_version)
+      with self.open_function(filename, "r") as f:
+        ast = parser.parse_string(f.read(), filename=filename, name=module_name,
+                                  python_version=self.python_version)
     return self._process_module(module_name, filename, ast)
 
   def _process_module(self, module_name, filename, ast):
@@ -333,9 +350,16 @@ class Loader(object):
           if base_name == "*":
             continue
           full_name = "%s.%s" % (name, base_name)
+          # Check whether full_name is a submodule based on whether it is
+          # defined in the __init__ file.
           try:
-            other_ast.Lookup(full_name)
+            attr = other_ast.Lookup(full_name)
           except KeyError:
+            attr = None
+          # 'from . import submodule as submodule' produces
+          # Alias(submodule, NamedType(submodule)).
+          if attr is None or (isinstance(attr, pytd.Alias) and
+                              attr.name == getattr(attr.type, "name", None)):
             # Don't check the import result - _resolve_external_types will raise
             # a better error.
             self._import_name(full_name)
@@ -347,7 +371,7 @@ class Loader(object):
           self._get_module_map(), self_name=name,
           module_alias_map=self._aliases))
     except KeyError as e:
-      raise BadDependencyError(utils.message(e), name)
+      raise BadDependencyError(utils.message(e), name) from e
     return pyval
 
   def _finish_pyi(self, pyval, ast=None):
@@ -359,7 +383,7 @@ class Loader(object):
     try:
       pyval.Visit(visitors.VerifyLookup(ignore_late_types=True))
     except ValueError as e:
-      raise BadDependencyError(utils.message(e), ast_name or pyval.name)
+      raise BadDependencyError(utils.message(e), ast_name or pyval.name) from e
     pyval.Visit(visitors.VerifyContainers())
 
   def resolve_type(self, pyval, ast):
@@ -506,6 +530,8 @@ class Loader(object):
         # nothing else supplies the module AST.
         default = self._modules.get(module_name)
         del self._modules[module_name]
+      elif module_name in _ALWAYS_PREFER_TYPESHED:
+        del self._modules[module_name]
       else:
         return file_ast
 
@@ -521,6 +547,7 @@ class Loader(object):
 
     # Now return the default module if we have found nothing better.
     if file_ast:
+      assert default
       self._modules[module_name] = default
       return file_ast
 
@@ -633,11 +660,13 @@ class PickledPyiLoader(Loader):
   """A Loader which always loads pickle instead of PYI, for speed."""
 
   def __init__(self, *args, **kwargs):
-    super(PickledPyiLoader, self).__init__(*args, **kwargs)
+    super().__init__(*args, **kwargs)
 
   @classmethod
   def load_from_pickle(cls, filename, base_module, **kwargs):
-    items = pytd_utils.LoadPickle(filename, compress=True)
+    items = pytd_utils.LoadPickle(
+        filename, compress=True,
+        open_function=kwargs.get("open_function", open))
     modules = {
         name: Module(name, filename=None, ast=None, pickle=pickle, dirty=False)
         for name, pickle in items
@@ -675,11 +704,12 @@ class PickledPyiLoader(Loader):
   def load_file(self, module_name, filename, ast=None):
     """Load (or retrieve from cache) a module and resolve its dependencies."""
     if not is_pickle(filename):
-      return super(PickledPyiLoader, self).load_file(module_name, filename, ast)
+      return super().load_file(module_name, filename, ast)
     existing = self._get_existing_ast(module_name)
     if existing:
       return existing
-    loaded_ast = pytd_utils.LoadPickle(filename)
+    loaded_ast = pytd_utils.LoadPickle(
+        filename, open_function=self.open_function)
     # At this point ast.name and module_name could be different.
     # They are later synced in ProcessAst.
     dependencies = {d: names for d, names in loaded_ast.dependencies
@@ -691,7 +721,7 @@ class PickledPyiLoader(Loader):
       ast = serialize_ast.ProcessAst(loaded_ast, self._get_module_map())
     except serialize_ast.UnrestorableDependencyError as e:
       del self._modules[module_name]
-      raise BadDependencyError(utils.message(e), module_name)
+      raise BadDependencyError(utils.message(e), module_name) from e
     # Mark all the module's late dependencies as explicitly imported.
     for d, _ in loaded_ast.late_dependencies:
       if d != loaded_ast.ast.name:
