@@ -1,6 +1,7 @@
 """Tests for load_pytd.py."""
 
 import collections
+import io
 import os
 import textwrap
 
@@ -48,7 +49,7 @@ class ImportPathsTest(test_base.UnitTest):
 
   def test_basic(self):
     with file_utils.Tempdir() as d:
-      d.create_file("path/to/some/module.pyi", "def foo(x:int) -> str")
+      d.create_file("path/to/some/module.pyi", "def foo(x:int) -> str: ...")
       loader = load_pytd.Loader(
           "base", self.python_version, pythonpath=[d.path])
       ast = loader.import_name("path.to.some.module")
@@ -57,8 +58,8 @@ class ImportPathsTest(test_base.UnitTest):
   def test_path(self):
     with file_utils.Tempdir() as d1:
       with file_utils.Tempdir() as d2:
-        d1.create_file("dir1/module1.pyi", "def foo1() -> str")
-        d2.create_file("dir2/module2.pyi", "def foo2() -> str")
+        d1.create_file("dir1/module1.pyi", "def foo1() -> str: ...")
+        d2.create_file("dir2/module2.pyi", "def foo2() -> str: ...")
         loader = load_pytd.Loader(
             "base", self.python_version, pythonpath=[d1.path, d2.path])
         module1 = loader.import_name("dir1.module1")
@@ -79,8 +80,8 @@ class ImportPathsTest(test_base.UnitTest):
       loader = load_pytd.Loader(
           "base", self.python_version, pythonpath=[d.path])
       mod = loader.import_name("foo")
-      self.assertEqual("__builtin__.int", mod.Lookup("foo.x").type.cls.name)
-      self.assertEqual("__builtin__.int", mod.Lookup("foo.x").type.name)
+      self.assertEqual("builtins.int", mod.Lookup("foo.x").type.cls.name)
+      self.assertEqual("builtins.int", mod.Lookup("foo.x").type.name)
 
   def test_no_init(self):
     with file_utils.Tempdir() as d:
@@ -104,7 +105,7 @@ class ImportPathsTest(test_base.UnitTest):
 
   def test_deep_dependency(self):
     with file_utils.Tempdir() as d:
-      d.create_file("module1.pyi", "def get_bar() -> module2.Bar")
+      d.create_file("module1.pyi", "def get_bar() -> module2.Bar: ...")
       d.create_file("module2.pyi", "class Bar:\n  pass")
       loader = load_pytd.Loader(
           "base", self.python_version, pythonpath=[d.path])
@@ -115,12 +116,12 @@ class ImportPathsTest(test_base.UnitTest):
   def test_circular_dependency(self):
     with file_utils.Tempdir() as d:
       d.create_file("foo.pyi", """
-        def get_bar() -> bar.Bar
+        def get_bar() -> bar.Bar: ...
         class Foo:
           pass
       """)
       d.create_file("bar.pyi", """
-        def get_foo() -> foo.Foo
+        def get_foo() -> foo.Foo: ...
         class Bar:
           pass
       """)
@@ -135,10 +136,10 @@ class ImportPathsTest(test_base.UnitTest):
 
   def test_relative(self):
     with file_utils.Tempdir() as d:
-      d.create_file("__init__.pyi", "base = ...  # type: ?")
-      d.create_file("path/__init__.pyi", "path = ...  # type: ?")
-      d.create_file("path/to/__init__.pyi", "to = ...  # type: ?")
-      d.create_file("path/to/some/__init__.pyi", "some = ...  # type: ?")
+      d.create_file("__init__.pyi", "base = ...  # type: str")
+      d.create_file("path/__init__.pyi", "path = ...  # type: str")
+      d.create_file("path/to/__init__.pyi", "to = ...  # type: str")
+      d.create_file("path/to/some/__init__.pyi", "some = ...  # type: str")
       d.create_file("path/to/some/module.pyi", "")
       loader = load_pytd.Loader("path.to.some.module",
                                 self.python_version,
@@ -155,6 +156,22 @@ class ImportPathsTest(test_base.UnitTest):
     loader = load_pytd.Loader("base", self.python_version)
     self.assertTrue(loader.import_name("urllib.request"))
 
+  def test_prefer_typeshed(self):
+    with file_utils.Tempdir() as d:
+      # Override two modules from typeshed
+      d.create_file("typing_extensions/__init__.pyi", "foo: str = ...")
+      d.create_file("crypt/__init__.pyi", "foo: str = ...")
+      loader = load_pytd.Loader("x", self.python_version, pythonpath=[d.path])
+      # typing_extensions should ignore the override, crypt should not.
+      ast1 = loader.import_name("typing_extensions")
+      ast2 = loader.import_name("crypt")
+      self.assertTrue(ast1.Lookup("typing_extensions.Literal"))
+      self.assertTrue(ast2.Lookup("crypt.foo"))
+      with self.assertRaises(KeyError):
+        ast1.Lookup("typing_extensions.foo")
+      with self.assertRaises(KeyError):
+        ast2.Lookup("crypt.crypt")
+
   def test_resolve_alias(self):
     with file_utils.Tempdir() as d:
       d.create_file("module1.pyi", """
@@ -162,7 +179,7 @@ class ImportPathsTest(test_base.UnitTest):
           x = List[int]
       """)
       d.create_file("module2.pyi", """
-          def f() -> module1.x
+          def f() -> module1.x: ...
       """)
       loader = load_pytd.Loader(
           "base", self.python_version, pythonpath=[d.path])
@@ -249,11 +266,11 @@ class ImportPathsTest(test_base.UnitTest):
 
   def test_get_resolved_modules(self):
     with file_utils.Tempdir() as d:
-      filename = d.create_file("dir/module.pyi", "def foo() -> str")
+      filename = d.create_file("dir/module.pyi", "def foo() -> str: ...")
       loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
       ast = loader.import_name("dir.module")
       modules = loader.get_resolved_modules()
-      self.assertEqual(set(modules), {"__builtin__", "typing", "dir.module"})
+      self.assertEqual(set(modules), {"builtins", "typing", "dir.module"})
       module = modules["dir.module"]
       self.assertEqual(module.module_name, "dir.module")
       self.assertEqual(module.filename, filename)
@@ -275,7 +292,7 @@ class ImportPathsTest(test_base.UnitTest):
       loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
       ast = loader.import_name("os2.path")
       self.assertEqual(ast.Lookup("os2.path._PathType").type.name,
-                       "__builtin__.bytes")
+                       "builtins.bytes")
 
   def test_circular_import_with_external_type(self):
     with file_utils.Tempdir() as d:
@@ -344,6 +361,55 @@ class ImportPathsTest(test_base.UnitTest):
       cls = a.Lookup("a.A")
       self.assertEqual("UserDict.UserDict", pytd_utils.Print(cls.parents[0]))
 
+  def test_open_function(self):
+    def mock_open(*unused_args, **unused_kwargs):
+      return io.StringIO("x: int")
+    loader = load_pytd.Loader(
+        "base", self.python_version, open_function=mock_open)
+    a = loader.load_file("a", "a.pyi")
+    self.assertEqual("int", pytd_utils.Print(a.Lookup("a.x").type))
+
+  def test_submodule_reexport(self):
+    with file_utils.Tempdir() as d:
+      d.create_file("foo/bar.pyi", "")
+      d.create_file("foo/__init__.pyi", """
+        from . import bar as bar
+      """)
+      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      foo = loader.import_name("foo")
+      self.assertEqual(pytd_utils.Print(foo), "import foo.bar")
+
+  def test_submodule_rename(self):
+    with file_utils.Tempdir() as d:
+      d.create_file("foo/bar.pyi", "")
+      d.create_file("foo/__init__.pyi", """
+        from . import bar as baz
+      """)
+      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      foo = loader.import_name("foo")
+      self.assertEqual(pytd_utils.Print(foo), "import foo.bar as foo.baz")
+
+  def test_typing_reexport(self):
+    with file_utils.Tempdir() as d:
+      d.create_file("foo.pyi", """
+        from typing import List as List
+      """)
+      d.create_file("bar.pyi", """
+        from foo import *
+        def f() -> List[int]: ...
+      """)
+      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      foo = loader.import_name("foo")
+      bar = loader.import_name("bar")
+      self.assertEqual(pytd_utils.Print(foo), "foo.List = list")
+      self.assertEqual(pytd_utils.Print(bar), textwrap.dedent("""
+        from typing import List
+
+        bar.List = list
+
+        def bar.f() -> List[int]: ...
+      """).strip())
+
 
 class ImportTypeMacroTest(test_base.UnitTest):
 
