@@ -2,6 +2,7 @@
 
 import collections
 import contextlib
+import enum
 import logging
 
 from pytype import abstract
@@ -18,7 +19,7 @@ from six import moves
 
 log = logging.getLogger(__name__)
 
-TOP_LEVEL_IGNORE = {
+TOP_LEVEL_IGNORE = frozenset({
     "__builtins__",
     "__doc__",
     "__file__",
@@ -27,9 +28,9 @@ TOP_LEVEL_IGNORE = {
     "__name__",
     "__annotations__",
     "google_type_annotations",
-}
+})
 
-CLASS_LEVEL_IGNORE = {
+CLASS_LEVEL_IGNORE = frozenset({
     "__builtins__",
     "__class__",
     "__module__",
@@ -37,31 +38,47 @@ CLASS_LEVEL_IGNORE = {
     "__qualname__",
     "__slots__",
     "__annotations__",
-}
+})
 
 
 class Converter(utils.VirtualMachineWeakrefMixin):
   """Functions for converting abstract classes into PyTD."""
 
+  class OutputMode(enum.IntEnum):
+    """Controls the level of detail in pytd types. See set_output_mode."""
+    NORMAL = 0
+    DETAILED = 1
+    LITERAL = 2
+
   def __init__(self, vm):
-    super(Converter, self).__init__(vm)
-    self._detailed = False
+    super().__init__(vm)
+    self._output_mode = Converter.OutputMode.NORMAL
 
   @contextlib.contextmanager
-  def produce_detailed_output(self):
-    """Produce more detailed pytd types, which may be unsafe for pyi files.
+  def set_output_mode(self, mode):
+    """Change the level of detail in pytd types.
 
-    With this setting, the converter will do things like using the names of
-    inner classes rather than Any and including the known argument types for a
-    callable even if the argument count is unknown. Useful for error messages.
+    Args:
+      mode: Converter.OutputMode option controlling the level of detail to use.
+        NORMAL - the default, safe for pyi files.
+        DETAILED - more detail, unsafe for pyi files. The converter will do
+          things like using the names of inner classes rather than Any and
+          including the known argument types for a callable even if the argument
+          count is unknown. Useful for error messages.
+        LITERAL - like DETAILED, but bool, int, str, and bytes constants will be
+          emitted as Literal[<constant>] rather than their type.
 
     Yields:
       None.
     """
-    old = self._detailed
-    self._detailed = True
+    old = self._output_mode
+    self._output_mode = mode
     yield
-    self._detailed = old
+    self._output_mode = old
+
+  @property
+  def _detailed(self):
+    return self._output_mode >= Converter.OutputMode.DETAILED
 
   def _get_values(self, node, var, view):
     if var.bindings and view is not None:
@@ -93,10 +110,12 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         v = instance = None
       template = new_template
     if instance is None and isinstance(v, abstract.ParameterizedClass):
+      assert v
       return [self.value_instance_to_pytd_type(
           node, v.get_formal_type_parameter(t), None, seen, view)
               for t in template]
-    elif isinstance(instance, abstract.SimpleAbstractValue):
+    elif isinstance(instance, abstract.SimpleValue):
+      assert instance
       type_arguments = []
       for t in template:
         if isinstance(instance, abstract.Tuple):
@@ -106,7 +125,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
               node, instance.get_instance_type_parameter(t), view)
         elif isinstance(v, abstract.CallableClass):
           param_values = v.get_formal_type_parameter(t).instantiate(
-              node or self.vm.root_cfg_node).data
+              node or self.vm.root_node).data
         else:
           param_values = [self.vm.convert.unsolvable]
         if (param_values == [self.vm.convert.unsolvable] and
@@ -145,6 +164,26 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     elif isinstance(v, abstract.AnnotationContainer):
       return self.value_instance_to_pytd_type(
           node, v.base_cls, instance, seen, view)
+    elif isinstance(v, abstract.LiteralClass):
+      if not v.value:
+        # TODO(b/123775699): Remove this workaround once we support literal
+        # enums.
+        return pytd.AnythingType()
+      if isinstance(v.value.pyval, (str, bytes)):
+        # Strings are stored as strings of their representations, prefix and
+        # quotes and all.
+        value = repr(v.value.pyval)
+      elif isinstance(v.value.pyval, bool):
+        # True and False are stored as pytd constants.
+        value = self.vm.lookup_builtin(f"builtins.{v.value.pyval}")
+      else:
+        # Ints are stored as their literal values. Note that Literal[None] or a
+        # nested literal will never appear here, since we simplified it to None
+        # or unnested it, respectively, in typing_overlay. Literal[<enum>] does
+        # not appear here yet because it is unsupported.
+        assert isinstance(v.value.pyval, int), v.value.pyval
+        value = v.value.pyval
+      return pytd.Literal(value)
     elif isinstance(v, mixin.Class):
       if not self._detailed and v.official_name is None:
         return pytd.AnythingType()
@@ -164,11 +203,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           node, v, instance, type_params, seen, view)
       base = pytd_utils.NamedTypeWithModule(v.official_name or v.name, v.module)
       if self._is_tuple(v, instance):
-        if type_arguments:
-          homogeneous = False
-        else:
-          homogeneous = True
-          type_arguments = [pytd.NothingType()]
+        homogeneous = False
       elif v.full_name == "typing.Callable":
         homogeneous = not isinstance(v, abstract.CallableClass)
       else:
@@ -182,7 +217,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     elif isinstance(v, typing_overlay.NoReturn):
       return pytd.NothingType()
     else:
-      log.info("Using ? for instance of %s", v.name)
+      log.info("Using Any for instance of %s", v.name)
       return pytd.AnythingType()
 
   def value_to_pytd_type(self, node, v, seen, view):
@@ -216,7 +251,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       else:
         return pytd.AnythingType()
     elif isinstance(v, typing_overlay.TypeVar):
-      return pytd.NamedType("__builtin__.type")
+      return pytd.NamedType("builtins.type")
     elif isinstance(v, dataclass_overlay.FieldInstance):
       if not v.default:
         return pytd.AnythingType()
@@ -242,23 +277,29 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       return pytd.NamedType("typing.Callable")
     elif isinstance(v, mixin.Class):
       param = self.value_instance_to_pytd_type(node, v, None, seen, view)
-      return pytd.GenericType(base_type=pytd.NamedType("__builtin__.type"),
+      return pytd.GenericType(base_type=pytd.NamedType("builtins.type"),
                               parameters=(param,))
     elif isinstance(v, abstract.Module):
-      return pytd.NamedType("__builtin__.module")
-    elif isinstance(v, abstract.SimpleAbstractValue):
+      return pytd.NamedType("builtins.module")
+    elif (self._output_mode >= Converter.OutputMode.LITERAL and
+          isinstance(v, abstract.ConcreteValue) and
+          isinstance(v.pyval, (int, str, bytes))):
+      # LITERAL mode is used only for pretty-printing, so we just stringify the
+      # inner value rather than properly converting it.
+      return pytd.Literal(repr(v.pyval))
+    elif isinstance(v, abstract.SimpleValue):
       if v.cls:
         ret = self.value_instance_to_pytd_type(
             node, v.cls, v, seen=seen, view=view)
         ret.Visit(visitors.FillInLocalPointers(
-            {"__builtin__": self.vm.loader.builtins}))
+            {"builtins": self.vm.loader.builtins}))
         return ret
       else:
         # We don't know this type's __class__, so return AnythingType to
         # indicate that we don't know anything about what this is.
         # This happens e.g. for locals / globals, which are returned from the
         # code in class declarations.
-        log.info("Using ? for %s", v.name)
+        log.info("Using Any for %s", v.name)
         return pytd.AnythingType()
     elif isinstance(v, abstract.Union):
       opts = []
@@ -272,7 +313,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         opts.append(opt)
       return pytd.UnionType(tuple(opts))
     elif isinstance(v, special_builtins.SuperInstance):
-      return pytd.NamedType("__builtin__.super")
+      return pytd.NamedType("builtins.super")
     elif isinstance(v, abstract.TypeParameter):
       # Arguably, the type of a type parameter is NamedType("typing.TypeVar"),
       # but pytype doesn't know how to handle that, so let's just go with Any
@@ -303,8 +344,8 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     """
     base_cls = self.vm.convert.function_type
     ret = sig.annotations.get("return", self.vm.convert.unsolvable)
-    if self._detailed or (
-        sig.mandatory_param_count() == sig.maximum_param_count()):
+    if not sig.kwonly_params and (self._detailed or (
+        sig.mandatory_param_count() == sig.maximum_param_count())):
       # If self._detailed is false, we throw away the argument types if the
       # function takes a variable number of arguments, which is correct for pyi
       # generation but undesirable for, say, error message printing.
@@ -315,8 +356,8 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       params.update(enumerate(args))
       return abstract.CallableClass(base_cls, params, self.vm)
     else:
-      # The only way to indicate a variable number of arguments in a Callable
-      # is to not specify argument types at all.
+      # The only way to indicate kwonly arguments or a variable number of
+      # arguments in a Callable is to not specify argument types at all.
       params = {abstract_utils.ARGS: self.vm.convert.unsolvable,
                 abstract_utils.RET: ret}
       return abstract.ParameterizedClass(base_cls, params, self.vm)
@@ -416,7 +457,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         annot = func.signature.annotations[func.signature.varargs_name]
         typ = annot.get_instance_type(node_after)
       else:
-        typ = pytd.NamedType("__builtin__.tuple")
+        typ = pytd.NamedType("builtins.tuple")
       starargs = pytd.Parameter(
           func.signature.varargs_name, typ, False, True, None)
     else:
@@ -426,7 +467,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         annot = func.signature.annotations[func.signature.kwargs_name]
         typ = annot.get_instance_type(node_after)
       else:
-        typ = pytd.NamedType("__builtin__.dict")
+        typ = pytd.NamedType("builtins.dict")
       starstarargs = pytd.Parameter(
           func.signature.kwargs_name, typ, False, True, None)
     else:
@@ -479,7 +520,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     if sig.has_return_annotation:
       ret_type = sig.annotations["return"].get_instance_type(node)
     else:
-      ret_type = pytd.NamedType("__builtin__.NoneType")
+      ret_type = pytd.NamedType("builtins.NoneType")
     pytd_sig = pytd.Signature(
         params=tuple(params+kwonly),
         starargs=star,
@@ -641,6 +682,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
                      methods=tuple(methods.values()),
                      constants=tuple(constants),
                      classes=(),
+                     decorators=(),
                      slots=v.slots,
                      template=())
     for base in missing_bases:
