@@ -30,15 +30,15 @@ class AttrOverlay(overlay.Overlay):
         "Factory": Factory.make,
     }
     ast = vm.loader.import_name("attr")
-    super(AttrOverlay, self).__init__(vm, "attr", member_map, ast)
+    super().__init__(vm, "attr", member_map, ast)
 
 
 class Attrs(classgen.Decorator):
   """Implements the @attr.s decorator."""
 
   @classmethod
-  def make(cls, name, vm):
-    return super(Attrs, cls).make(name, vm, "attr")
+  def make(cls, vm):
+    return super().make("s", vm, "attr")
 
   def init_name(self, attr):
     # attrs removes leading underscores from attrib names when generating kwargs
@@ -58,29 +58,33 @@ class Attrs(classgen.Decorator):
     for name, local in ordered_locals.items():
       typ, orig = local.get_type(node, name), local.orig
       if is_attrib(orig):
-        if typ and orig.data[0].has_type:
+        attrib = orig.data[0]
+        if typ and attrib.has_type:
           # We cannot have both a type annotation and a type argument.
           self.vm.errorlog.invalid_annotation(self.vm.frames, typ)
           attr = Attribute(
               name=name,
               typ=self.vm.convert.unsolvable,
-              init=orig.data[0].init,
-              default=orig.data[0].default)
+              init=attrib.init,
+              kw_only=attrib.kw_only,
+              default=attrib.default)
         elif not typ:
           # Replace the attrib in the class dict with its type.
           attr = Attribute(
               name=name,
-              typ=orig.data[0].typ,
-              init=orig.data[0].init,
-              default=orig.data[0].default)
+              typ=attrib.typ,
+              init=attrib.init,
+              kw_only=attrib.kw_only,
+              default=attrib.default)
           cls.members[name] = classgen.instantiate(node, name, attr.typ)
         else:
           # cls.members[name] has already been set via a typecomment
           attr = Attribute(
               name=name,
               typ=typ,
-              init=orig.data[0].init,
-              default=orig.data[0].default)
+              init=attrib.init,
+              kw_only=attrib.kw_only,
+              default=attrib.default)
         self.vm.check_annotation_type_mismatch(
             node, attr.name, attr.typ, attr.default, local.stack,
             allow_none=True)
@@ -89,7 +93,8 @@ class Attrs(classgen.Decorator):
         if not match_classvar(typ):
           self.vm.check_annotation_type_mismatch(
               node, name, typ, orig, local.stack, allow_none=True)
-          attr = Attribute(name=name, typ=typ, init=True, default=orig)
+          attr = Attribute(
+              name=name, typ=typ, init=True, kw_only=False, default=orig)
           if not orig:
             cls.members[name] = classgen.instantiate(node, name, typ)
           own_attrs.append(attr)
@@ -105,15 +110,16 @@ class Attrs(classgen.Decorator):
       cls.members["__init__"] = init_method
 
 
-class AttribInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
+class AttribInstance(abstract.SimpleValue, mixin.HasSlots):
   """Return value of an attr.ib() call."""
 
-  def __init__(self, vm, typ, has_type, init, default):
-    super(AttribInstance, self).__init__("attrib", vm)
+  def __init__(self, vm, typ, has_type, init, kw_only, default):
+    super().__init__("attrib", vm)
     mixin.HasSlots.init_mixin(self)
     self.typ = typ
     self.has_type = has_type
     self.init = init
+    self.kw_only = kw_only
     self.default = default
     # TODO(rechen): attr.ib() returns an instance of attr._make._CountingAttr.
     self.cls = vm.convert.unsolvable
@@ -157,8 +163,8 @@ class Attrib(classgen.FieldConstructor):
   """Implements attr.ib."""
 
   @classmethod
-  def make(cls, name, vm):
-    return super(Attrib, cls).make(name, vm, "attr")
+  def make(cls, vm):
+    return super().make("ib", vm, "attr")
 
   def call(self, node, unused_func, args):
     """Returns a type corresponding to an attr."""
@@ -166,6 +172,7 @@ class Attrib(classgen.FieldConstructor):
     node, default_var = self._get_default_var(node, args)
     type_var = args.namedargs.get("type")
     init = self.get_kwarg(args, "init", True)
+    kw_only = self.get_kwarg(args, "kw_only", False)
     has_type = type_var is not None
     if type_var:
       typ = self.vm.annotations_util.extract_annotation(
@@ -174,8 +181,8 @@ class Attrib(classgen.FieldConstructor):
       typ = get_type_from_default(default_var, self.vm)
     else:
       typ = self.vm.convert.unsolvable
-    typ = AttribInstance(self.vm, typ, has_type, init,
-                         default_var).to_variable(node)
+    typ = AttribInstance(
+        self.vm, typ, has_type, init, kw_only, default_var).to_variable(node)
     return node, typ
 
   def _get_default_var(self, node, args):
@@ -207,12 +214,17 @@ def match_classvar(typ):
 
 
 def get_type_from_default(default_var, vm):
-  if default_var and default_var.data == [vm.convert.none]:
+  """Get the type of an attribute from its default value."""
+  if default_var.data == [vm.convert.none]:
     # A default of None doesn't give us any information about the actual type.
     return vm.convert.unsolvable
   typ = vm.convert.merge_classes(default_var.data)
   if typ == vm.convert.empty:
     return vm.convert.unsolvable
+  elif isinstance(typ, abstract.TupleClass) and not typ.tuple_length:
+    # The type of an attribute whose default is an empty tuple should be
+    # Tuple[Any, ...], not Tuple[()].
+    return vm.convert.tuple_type
   return typ
 
 
@@ -220,5 +232,5 @@ class Factory(abstract.PyTDFunction):
   """Implementation of attr.Factory."""
 
   @classmethod
-  def make(cls, name, vm):
-    return super(Factory, cls).make(name, vm, "attr")
+  def make(cls, vm):
+    return super().make("Factory", vm, "attr")
