@@ -1,5 +1,10 @@
 """Implementation of the types in Python 3's typing.py."""
 
+# We should be able to enable pytype on this file once we switch to the
+# typed_ast-based pyi parser. The current parser can't handle a constant called
+# 'namedtuple':
+# pytype: skip-file
+
 # pylint's detection of this is error-prone:
 # pylint: disable=unpacking-non-sequence
 
@@ -15,7 +20,6 @@ from pytype.overlays import collections_overlay
 from pytype.pytd import pep484
 from pytype.pytd import pytd
 from pytype.pytd import visitors
-import six
 from six import moves
 
 
@@ -28,22 +32,22 @@ class TypingOverlay(overlay.Overlay):
 
   def __init__(self, vm):
     # Make sure we have typing available as a dependency
-    if not vm.loader.can_see("typing"):
+    if vm.python_version < (3, 5) and not vm.loader.can_see("typing"):
       vm.errorlog.import_error(vm.frames, "typing")
-    member_map = typing_overload.copy()
+    member_map = typing_overlay.copy()
     ast = vm.loader.typing
     for cls in ast.classes:
       _, name = cls.name.rsplit(".", 1)
       if name not in member_map and pytd.IsContainer(cls) and cls.template:
-        member_map[name] = TypingContainer
-    super(TypingOverlay, self).__init__(vm, "typing", member_map, ast)
+        member_map[name] = overlay.build(name, TypingContainer)
+    super().__init__(vm, "typing", member_map, ast)
 
 
 class Union(abstract.AnnotationClass):
   """Implementation of typing.Union[...]."""
 
-  def __init__(self, name, vm, options=()):
-    super(Union, self).__init__(name, vm)
+  def __init__(self, vm, options=()):
+    super().__init__("Union", vm)
     self.options = options
 
   def _build_value(self, node, inner, ellipses):
@@ -55,11 +59,11 @@ class TypingContainer(abstract.AnnotationContainer):
 
   def __init__(self, name, vm):
     if name in pep484.PEP484_CAPITALIZED:
-      pytd_name = "__builtin__." + name.lower()
+      pytd_name = "builtins." + name.lower()
     else:
       pytd_name = "typing." + name
     base = vm.convert.name_to_value(pytd_name)
-    super(TypingContainer, self).__init__(name, vm, base)
+    super().__init__(name, vm, base)
 
 
 class Tuple(TypingContainer):
@@ -69,7 +73,7 @@ class Tuple(TypingContainer):
     if ellipses:
       # An ellipsis may appear at the end of the parameter list as long as it is
       # not the only parameter.
-      return super(Tuple, self)._get_value_info(
+      return super()._get_value_info(
           inner, ellipses, allowed_ellipses={len(inner) - 1} - {0})
     else:
       template = list(moves.range(len(inner))) + [abstract_utils.T]
@@ -89,7 +93,7 @@ class Callable(TypingContainer):
       self.vm.errorlog.invalid_ellipses(
           self.vm.frames, inner_ellipses, args.name)
     else:
-      if args.cls and args.cls.full_name == "__builtin__.list":
+      if args.cls and args.cls.full_name == "builtins.list":
         self.vm.errorlog.ambiguous_annotation(self.vm.frames, [args])
       elif 0 not in ellipses or not isinstance(args, abstract.Unsolvable):
         self.vm.errorlog.invalid_annotation(
@@ -109,15 +113,14 @@ class Callable(TypingContainer):
       return template, inner, abstract.CallableClass
     else:
       # An ellipsis may take the place of the ARGS list.
-      return super(Callable, self)._get_value_info(
-          inner, ellipses, allowed_ellipses={0})
+      return super()._get_value_info(inner, ellipses, allowed_ellipses={0})
 
 
 class TypeVarError(Exception):
   """Raised if an error is encountered while initializing a TypeVar."""
 
   def __init__(self, message, bad_call=None):
-    super(TypeVarError, self).__init__(message)
+    super().__init__(message)
     self.bad_call = bad_call
 
 
@@ -127,9 +130,10 @@ class TypeVar(abstract.PyTDFunction):
   def _get_constant(self, var, name, arg_type, arg_type_desc=None):
     try:
       ret = abstract_utils.get_atomic_python_constant(var, arg_type)
-    except abstract_utils.ConversionError:
-      raise TypeVarError("%s must be %s" % (
-          name, arg_type_desc or "a constant " + arg_type.__name__))
+    except abstract_utils.ConversionError as e:
+      raise TypeVarError(
+          "%s must be %s" %
+          (name, arg_type_desc or "a constant " + arg_type.__name__)) from e
     return ret
 
   def _get_annotation(self, node, var, name):
@@ -157,12 +161,12 @@ class TypeVar(abstract.PyTDFunction):
     try:
       self.match_args(node, args)
     except function.InvalidParameters as e:
-      raise TypeVarError("wrong arguments", e.bad_call)
-    except function.FailedFunctionCall:
+      raise TypeVarError("wrong arguments", e.bad_call) from e
+    except function.FailedFunctionCall as e:
       # It is currently impossible to get here, since the only
       # FailedFunctionCall that is not an InvalidParameters is NotCallable.
-      raise TypeVarError("initialization failed")
-    name = self._get_constant(args.posargs[0], "name", six.string_types,
+      raise TypeVarError("initialization failed") from e
+    name = self._get_constant(args.posargs[0], "name", str,
                               arg_type_desc="a constant str")
     constraints = tuple(
         self._get_annotation(node, c, "constraint") for c in args.posargs[1:])
@@ -207,18 +211,17 @@ class Cast(abstract.PyTDFunction):
             self.vm.frames, "cannot pass a TypeVar to a function")
         return node, self.vm.new_unsolvable(node)
       return self.vm.init_class(node, annot)
-    return super(Cast, self).call(node, func, args)
+    return super().call(node, func, args)
 
 
 class NoReturn(abstract.Singleton):
   """Implements typing.NoReturn as a singleton."""
 
   def __init__(self, vm):
-    super(NoReturn, self).__init__("NoReturn", vm)
+    super().__init__("NoReturn", vm)
 
 
-def build_any(name, vm):
-  del name
+def build_any(vm):
   return vm.convert.unsolvable
 
 
@@ -226,28 +229,27 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
   """Factory for creating typing.NamedTuple classes."""
 
   @classmethod
-  def make(cls, name, vm):
+  def make(cls, vm):
     typing_ast = vm.loader.import_name("typing")
     # Because NamedTuple is a special case for the pyi parser, typing.pytd has
     # "_NamedTuple" instead. Replace the name of the returned function so that
     # error messages will correctly display "typing.NamedTuple".
     pyval = typing_ast.Lookup("typing._NamedTuple")
     pyval = pyval.Replace(name="typing.NamedTuple")
-    self = super(NamedTupleFuncBuilder, cls).make(name, vm, pyval)
+    self = super().make("NamedTuple", vm, pyval)
     # NamedTuple's fields arg has type Sequence[Sequence[Union[str, type]]],
     # which doesn't provide precise enough type-checking, so we have to do
     # some of our own in _getargs. _NamedTupleFields is an alias to
     # List[Tuple[str, type]], which gives a more understandable error message.
     fields_pyval = typing_ast.Lookup("typing._NamedTupleFields").type
-    fields_type = vm.convert.constant_to_value(
-        fields_pyval, {}, vm.root_cfg_node)
+    fields_type = vm.convert.constant_to_value(fields_pyval, {}, vm.root_node)
     # pylint: disable=protected-access
     self._fields_param = function.BadParam(name="fields", expected=fields_type)
     return self
 
   def _is_str_instance(self, val):
     return (isinstance(val, abstract.Instance) and
-            val.full_name in ("__builtin__.str", "__builtin__.unicode"))
+            val.full_name in ("builtins.str", "builtins.unicode"))
 
   def _getargs(self, node, args):
     self.match_args(node, args)
@@ -257,18 +259,18 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     name_var = callargs["typename"]
     fields_var = callargs["fields"]
     fields = abstract_utils.get_atomic_python_constant(fields_var)
-    if isinstance(fields, six.string_types):
+    if isinstance(fields, str):
       # Since str matches Sequence, we have to manually check for it.
       raise function.WrongArgTypes(
           sig.signature, args, self.vm, self._fields_param)
     # The fields is a list of tuples, so we need to deeply unwrap them.
     fields = [abstract_utils.get_atomic_python_constant(t) for t in fields]
-    # We need the actual string for the field names and the AtomicAbstractValue
+    # We need the actual string for the field names and the BaseValue
     # for the field types.
     names = []
     types = []
     for field in fields:
-      if isinstance(field, six.string_types):
+      if isinstance(field, str):
         # Since str matches Sequence, we have to manually check for it.
         raise function.WrongArgTypes(
             sig.signature, args, self.vm, self._fields_param)
@@ -478,16 +480,16 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
 
   _special = ("__module__", "__name__", "__qualname__", "__annotations__")
 
-  def __init__(self, name, vm):
+  def __init__(self, vm):
     typing_ast = vm.loader.import_name("typing")
     pyval = typing_ast.Lookup("typing._NamedTupleClass")
     pyval = pyval.Replace(name="typing.NamedTuple")
-    super(NamedTupleClassBuilder, self).__init__(name, pyval, vm)
+    super().__init__("NamedTuple", pyval, vm)
     # Prior to python 3.6, NamedTuple is a function. Although NamedTuple is a
     # class in python 3.6+, we can still use it like a function. Hold the
     # an instance of 'NamedTupleFuncBuilder' so that we can reuse the
     # old implementation to implement the NamedTuple in python 3.6+
-    self.namedtuple = NamedTupleFuncBuilder.make(name, vm)
+    self.namedtuple = NamedTupleFuncBuilder.make(vm)
 
   def call(self, node, _, args):
     posargs = args.posargs
@@ -571,7 +573,7 @@ class NewType(abstract.PyTDFunction):
   """Implementation of typing.NewType as a function."""
 
   def __init__(self, name, signatures, kind, vm):
-    super(NewType, self).__init__(name, signatures, kind, vm)
+    super().__init__(name, signatures, kind, vm)
     assert len(self.signatures) == 1, "NewType has more than one signature."
     signature = self.signatures[0].signature
     self._name_arg_name = signature.param_names[0]
@@ -664,54 +666,83 @@ class Optional(abstract.AnnotationClass):
     return abstract.Union((self.vm.convert.none_type,) + inner, self.vm)
 
 
+class Literal(TypingContainer):
+  """Implementation of typing.Literal."""
+
+  def _build_value(self, node, inner, ellipses):
+    values = []
+    errors = []
+    for i, param in enumerate(inner):
+      # TODO(b/123775699): Once pytype has proper support for enums, we should
+      # stop allowing unsolvable and handle enums here.
+      if (param == self.vm.convert.none or
+          isinstance(param, abstract.LiteralClass) or
+          param == self.vm.convert.unsolvable and i not in ellipses):
+        value = param
+      elif (isinstance(param, abstract.ConcreteValue) and
+            isinstance(param.pyval, (int, str, bytes))):
+        value = abstract.LiteralClass(param, self.vm)
+      else:
+        if i in ellipses:
+          invalid_param = "..."
+        else:
+          invalid_param = param.name
+        errors.append((invalid_param, i))
+        value = self.vm.convert.unsolvable
+      values.append(value)
+    if errors:
+      self.vm.errorlog.invalid_annotation(
+          self.vm.frames, self,
+          "\n".join("Bad parameter %r at index %d" % e for e in errors))
+    return self.vm.merge_values(values)
+
+
 def not_supported_yet(name, vm):
   vm.errorlog.not_supported_yet(vm.frames, "typing." + name)
   return vm.convert.unsolvable
 
 
-def build_namedtuple(name, vm):
+def build_namedtuple(vm):
   if vm.python_version < (3, 6):
-    return NamedTupleFuncBuilder.make(name, vm)
+    return NamedTupleFuncBuilder.make(vm)
   else:
-    return NamedTupleClassBuilder(name, vm)
+    return NamedTupleClassBuilder(vm)
 
 
-def build_newtype(name, vm):
-  return NewType.make(name, vm, "typing")
+def build_newtype(vm):
+  return NewType.make("NewType", vm, "typing")
 
 
-def build_noreturn(name, vm):
-  del name
+def build_noreturn(vm):
   return vm.convert.no_return
 
 
-def build_overload(name, vm):
-  return Overload.make(name, vm, "typing")
+def build_overload(vm):
+  return Overload.make("overload", vm, "typing")
 
 
-def build_typevar(name, vm):
-  return TypeVar.make(name, vm, "typing", pyval_name="_typevar_new")
+def build_typevar(vm):
+  return TypeVar.make("TypeVar", vm, "typing", pyval_name="_typevar_new")
 
 
-def build_typechecking(name, vm):
-  del name
+def build_typechecking(vm):
   return vm.convert.true
 
 
-def build_cast(name, vm):
-  return Cast.make(name, vm, "typing")
+def build_cast(vm):
+  return Cast.make("cast", vm, "typing")
 
 
-typing_overload = {
+typing_overlay = {
     "Any": build_any,
-    "Callable": Callable,
-    "Generic": Generic,
-    "Literal": not_supported_yet,
+    "Callable": overlay.build("Callable", Callable),
+    "Generic": overlay.build("Generic", Generic),
+    "Literal": overlay.build("Literal", Literal),
     "NamedTuple": build_namedtuple,
     "NewType": build_newtype,
     "NoReturn": build_noreturn,
-    "Optional": Optional,
-    "Tuple": Tuple,
+    "Optional": overlay.build("Optional", Optional),
+    "Tuple": overlay.build("Tuple", Tuple),
     "TypeVar": build_typevar,
     "Union": Union,
     "TYPE_CHECKING": build_typechecking,
