@@ -7,6 +7,7 @@ from pytype import file_utils
 from pytype import module_utils
 from pytype import utils
 from pytype.pyi import parser_ext  # pytype: disable=import-error
+from pytype.pytd import escape
 from pytype.pytd import pep484
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
@@ -36,7 +37,7 @@ _SlotDecl = collections.namedtuple("_", ["slots"])
 _Property = collections.namedtuple("_", ["precedence", "arity"])
 
 
-class _ConditionScope(object):
+class _ConditionScope:
   """State associated with a condition if/elif/else block."""
 
   def __init__(self, parent):
@@ -83,7 +84,7 @@ class ParseError(Exception):
   """Exceptions raised by the parser."""
 
   def __init__(self, msg, line=None, filename=None, column=None, text=None):
-    super(ParseError, self).__init__(msg)
+    super().__init__(msg)
     self._line = line
     self._filename = filename
     self._column = column
@@ -114,7 +115,7 @@ class OverloadedDecoratorError(ParseError):
   def __init__(self, name, typ, *args, **kwargs):
     msg = "Overloaded signatures for %s disagree on %sdecorators" % (
         name, (typ + " " if typ else ""))
-    super(OverloadedDecoratorError, self).__init__(msg, *args, **kwargs)
+    super().__init__(msg, *args, **kwargs)
 
 
 class _Mutator(visitors.Visitor):
@@ -130,7 +131,7 @@ class _Mutator(visitors.Visitor):
   """
 
   def __init__(self, name, new_type):
-    super(_Mutator, self).__init__()
+    super().__init__()
     self.name = name
     self.new_type = new_type
     self.successful = False
@@ -150,7 +151,7 @@ class _InsertTypeParameters(visitors.Visitor):
   """Visitor for inserting TypeParameter instances."""
 
   def __init__(self, type_params):
-    super(_InsertTypeParameters, self).__init__()
+    super().__init__()
     self.type_params = {p.name: p for p in type_params}
 
   def VisitNamedType(self, node):
@@ -164,7 +165,7 @@ class _VerifyMutators(visitors.Visitor):
   """Visitor for verifying TypeParameters used in mutations are in scope."""
 
   def __init__(self):
-    super(_VerifyMutators, self).__init__()
+    super().__init__()
     # A stack of type parameters introduced into the scope. The top of the stack
     # contains the currently accessible parameter set.
     self.type_params_in_scope = [set()]
@@ -215,7 +216,7 @@ class _ContainsAnyType(visitors.Visitor):
   """Check if a pytd object contains a type of any of the given names."""
 
   def __init__(self, type_names):
-    super(_ContainsAnyType, self).__init__()
+    super().__init__()
     self._type_names = set(type_names)
     self.found = False
 
@@ -268,7 +269,7 @@ class _PropertyToConstant(visitors.Visitor):
         return True
 
 
-class _Parser(object):
+class _Parser:
   """A class used to parse a single PYI file.
 
   The PYI parser is split into two parts: a low level parser implemented in
@@ -435,7 +436,7 @@ class _Parser(object):
         except IndexError:
           text = None
         raise ParseError(utils.message(e), line=line, filename=self._filename,
-                         column=self._error_location[1], text=text)
+                         column=self._error_location[1], text=text) from e
       else:
         raise e
 
@@ -452,8 +453,8 @@ class _Parser(object):
       ast = ast.Replace(name=hashlib.md5(src.encode("utf-8")).hexdigest())
     ast = ast.Visit(visitors.StripExternalNamePrefix())
 
-    # Typeshed files that explicitly import and refer to "builtins" need to have
-    # that rewritten to __builtin__
+    # Typeshed files that explicitly import and refer to "__builtin__" need to
+    # have that rewritten to builtins
     return ast.Visit(visitors.RenameBuiltinsPrefix())
 
   def _build_type_decl_unit(self, defs):
@@ -464,15 +465,19 @@ class _Parser(object):
     assert not aliases  # We handle top-level aliases in add_alias_or_constant.
     constants.extend(self._constants)
 
+    if self._ast_name == "builtins":
+      constants.extend(_builtin_keyword_constants())
+
     generated_classes = sum(self._generated_classes.values(), [])
 
     classes = generated_classes + classes
     functions = _merge_method_signatures(functions)
 
     name_to_class = {c.name: c for c in classes}
+    name_to_constant = {c.name: c for c in constants}
     aliases = []
     for a in self._aliases.values():
-      t = _maybe_resolve_alias(a, name_to_class)
+      t = _maybe_resolve_alias(a, name_to_class, name_to_constant)
       if t is None:
         continue
       elif isinstance(t, pytd.Function):
@@ -567,7 +572,7 @@ class _Parser(object):
       try:
         actual = self._version[key]
       except IndexError as e:
-        raise ParseError(utils.message(e))
+        raise ParseError(utils.message(e)) from e
       if isinstance(key, slice):
         actual = _three_tuple(actual)
         value = _three_tuple(value)
@@ -645,6 +650,7 @@ class _Parser(object):
     return pytd.Constant(name, t)
 
   def new_alias_or_constant(self, name_and_value):
+    """Build an alias or constant."""
     name, value = name_and_value
     if name == "__slots__":
       if not isinstance(value, list):
@@ -652,6 +658,11 @@ class _Parser(object):
       return _SlotDecl(tuple(_handle_string_literal(s) for s in value))
     elif value in [pytd.NamedType("True"), pytd.NamedType("False")]:
       return pytd.Constant(name, pytd.NamedType("bool"))
+    elif isinstance(value, list):
+      if name != "__all__":
+        raise ParseError("Only __slots__ and __all__ can be literal lists")
+      return pytd.Constant(name, pytd.GenericType(
+          pytd.NamedType("typing.List"), (pytd.NamedType("str"),)))
     else:
       return pytd.Alias(name, value)
 
@@ -723,7 +734,7 @@ class _Parser(object):
           assert new_name == name
           new_name = t.name
         self._type_map[new_name] = t
-        if (new_name != name or
+        if (isinstance(item, tuple) or
             from_package != "typing" or
             self._ast_name == "protocols"):
           self._aliases[new_name] = pytd.Alias(new_name, t)
@@ -770,7 +781,7 @@ class _Parser(object):
         resolved_type = visitors.MaybeSubstituteParameters(
             base_type, parameters)
       except ValueError as e:
-        raise ParseError(str(e))
+        raise ParseError(str(e)) from e
       if resolved_type:
         return resolved_type
     if parameters is not None:
@@ -809,7 +820,7 @@ class _Parser(object):
 
   def _is_tuple_base_type(self, t):
     return isinstance(t, pytd.NamedType) and (
-        t.name == "tuple" or self._matches_full_name(t, "__builtin__.tuple") or
+        t.name == "tuple" or self._matches_full_name(t, "builtins.tuple") or
         self._matches_full_name(t, "typing.Tuple"))
 
   def _is_callable_base_type(self, t):
@@ -822,22 +833,13 @@ class _Parser(object):
         self._matches_full_name(t, "typing_extensions.Literal"))
 
   def _heterogeneous_tuple(self, base_type, parameters):
-    if parameters:
-      return pytd.TupleType(base_type=base_type, parameters=parameters)
-    else:
-      return pytd.GenericType(base_type=base_type,
-                              parameters=(pytd.NothingType(),))
+    return pytd.TupleType(base_type=base_type, parameters=parameters)
 
   def _is_empty_tuple(self, t):
-    return (isinstance(t, pytd.GenericType) and
-            self._is_tuple_base_type(t.base_type) and
-            t.parameters == (pytd.NothingType(),))
+    return isinstance(t, pytd.TupleType) and not t.parameters
 
   def _is_heterogeneous_tuple(self, t):
-    # An empty tuple is represented as a GenericType rather than a TupleType,
-    # but we still consider it heterogeneous because we know exactly what the
-    # parameters are (there are none).
-    return isinstance(t, pytd.TupleType) or self._is_empty_tuple(t)
+    return isinstance(t, pytd.TupleType)
 
   def _is_any(self, t):
     return isinstance(t, pytd.AnythingType) or t == pytd.NamedType("typing.Any")
@@ -854,13 +856,24 @@ class _Parser(object):
     if self._is_literal_base_type(base_type):
       literal_parameters = []
       for p in parameters:
-        if self._is_none(p):
+        if self._is_none(p) or isinstance(p, pytd.Literal):
           literal_parameters.append(p)
-        elif isinstance(p, pytd.NamedType) and p.name not in ("True", "False"):
-          # TODO(b/123775699): support enums.
-          literal_parameters.append(pytd.AnythingType())
-        else:
+        elif isinstance(p, pytd.NamedType):
+          if p.name in ("True", "False"):
+            literal_parameters.append(pytd.Literal(p))
+          else:
+            # TODO(b/123775699): support enums.
+            literal_parameters.append(pytd.AnythingType())
+        elif isinstance(p, (int, str, bytes)):
           literal_parameters.append(pytd.Literal(p))
+        elif isinstance(p, pytd.UnionType):
+          for t in p.type_list:
+            if isinstance(t, pytd.Literal):
+              literal_parameters.append(t)
+            else:
+              raise ParseError(f"Literal[{t}] not supported")
+        else:
+          raise ParseError(f"Literal[{p}] not supported")
       return pytd_utils.JoinTypes(literal_parameters)
     elif any(isinstance(p, (int, str)) for p in parameters):
       parameters = ", ".join(
@@ -962,7 +975,7 @@ class _Parser(object):
       try:
         signature = signature.Visit(mutator)
       except NotImplementedError as e:
-        raise ParseError(utils.message(e))
+        raise ParseError(utils.message(e)) from e
       if not mutator.successful:
         raise ParseError("No parameter named %s" % mutator.name)
 
@@ -1013,7 +1026,7 @@ class _Parser(object):
     """Build an __init__ method for a namedtuple.
 
     Builds a dummy __init__ that accepts any arguments. Needed because our
-    model of __builtin__.tuple uses __init__.
+    model of builtins.tuple uses __init__.
 
     Returns:
       A _NameAndSig object for an __init__ method.
@@ -1037,7 +1050,7 @@ class _Parser(object):
     fields = [(_handle_string_literal(n), t) for n, t in fields]
     # Handle previously defined NamedTuples with the same name
     prev_list = self._generated_classes[base_name]
-    class_name = "namedtuple-%s-%d" % (base_name, len(prev_list))
+    class_name = escape.pack_namedtuple_base_class(base_name, len(prev_list))
     class_parent = self._heterogeneous_tuple(pytd.NamedType("tuple"),
                                              tuple(t for _, t in fields))
     class_constants = tuple(pytd.Constant(n, t) for n, t in fields)
@@ -1052,6 +1065,7 @@ class _Parser(object):
                           parents=(class_parent,),
                           methods=tuple(methods),
                           constants=class_constants,
+                          decorators=(),
                           classes=(),
                           slots=tuple(n for n, _ in fields),
                           template=())
@@ -1059,6 +1073,49 @@ class _Parser(object):
     self._generated_classes[base_name].append(nt_class)
     return pytd.NamedType(nt_class.name)
 
+  def new_new_type(self, name, typ):
+    """Returns a type for a NewType."""
+    name = _handle_string_literal(name)
+    args = [("self", pytd.AnythingType(), None), ("val", typ, None)]
+    ret = pytd.NamedType("NoneType")
+    methods = _merge_method_signatures(
+        [self.new_function((), False, "__init__", args, ret, ())])
+    cls_name = escape.pack_newtype_base_class(
+        name, len(self._generated_classes[name]))
+    cls = pytd.Class(name=cls_name,
+                     metaclass=None,
+                     parents=(typ,),
+                     methods=tuple(methods),
+                     constants=(),
+                     decorators=(),
+                     classes=(),
+                     slots=None,
+                     template=())
+    self._generated_classes[name].append(cls)
+    return pytd.NamedType(cls_name)
+
+  def new_typed_dict(self, name, items, total):
+    """Returns a type for a TypedDict.
+
+    This method is currently called only for TypedDict objects defined via
+    the following function-based syntax:
+
+      Foo = TypeDict('Foo', {'a': int, 'b': str}, total=False)
+
+    rather than the recommended class-based syntax.
+
+    Args:
+      name: the name of the TypedDict instance, e.g., "'Foo'".
+      items: a {key: value_type} dict, e.g., {"'a'": "int", "'b'": "str"}.
+      total: A tuple of a single kwarg, e.g., ("total", NamedType("False")), or
+        None when no kwarg is passed.
+    """
+    # TODO(b/157603915): Add real support for TypedDict.
+    del name, items, total  # unused
+    return pytd.GenericType(
+        pytd.NamedType("typing.Dict"),
+        (pytd.NamedType("str"), pytd.NamedType("typing.Any")))
+
   def register_class_name(self, class_name):
     """Register a class name so that it can shadow aliases."""
     if not self._current_condition.active:
@@ -1085,10 +1142,22 @@ class _Parser(object):
       ParseError: if defs contains duplicate names (excluding multiple
           definitions of a function, which is allowed).
     """
-    unsupported_decorators = [d for d in decorators if d != "type_check_only"]
+    # Drop the @type_check_only decorator from classes
+    # TODO(mdemello): Workaround for the bug that typing.foo class decorators
+    # don't add the import, since typing.type_check_only is the only one.
+    decorators = [x for x in decorators if x != "type_check_only"]
+
+    # Check for some function/method-only decorators
+    nonclass = {"property", "classmethod", "staticmethod", "overload"}
+    unsupported_decorators = set(decorators) & nonclass
     if unsupported_decorators:
       raise ParseError("Unsupported class decorators: %s" % ", ".join(
           unsupported_decorators))
+
+    # Convert decorators to named types
+    decorators = [self._type_map.get(d) or pytd.NamedType(d)
+                  for d in decorators]
+
     # Process parent_args, extracting parents and possibly a metaclass.
     parents = []
     metaclass = None
@@ -1195,6 +1264,7 @@ class _Parser(object):
                       methods=tuple(methods),
                       constants=tuple(constants),
                       classes=tuple(classes),
+                      decorators=tuple(decorators),
                       slots=slots,
                       template=())
 
@@ -1267,13 +1337,6 @@ def parse_string(src, python_version, name=None, filename=None, platform=None):
       src, name, filename)
 
 
-def parse_file(filename, python_version, name=None, platform=None):
-  with open(filename, "r") as fi:
-    src = fi.read()
-  return _Parser(version=python_version, platform=platform).parse(
-      src, name, filename)
-
-
 def canonical_pyi(pyi, python_version, multiline_args=False):
   ast = parse_string(pyi, python_version=python_version)
   ast = ast.Visit(visitors.ClassTypeToNamedType())
@@ -1563,7 +1626,7 @@ def _merge_method_signatures(signatures):
     is_coroutine = name_to_is_coroutine[name]
     if name == "__new__" or decorator == "staticmethod":
       kind = pytd.STATICMETHOD
-    elif decorator == "classmethod":
+    elif name == "__init_subclass__" or decorator == "classmethod":
       kind = pytd.CLASSMETHOD
     elif _is_property(name, decorator, sigs[0]):
       kind = pytd.PROPERTY
@@ -1586,12 +1649,13 @@ def _merge_method_signatures(signatures):
   return methods
 
 
-def _maybe_resolve_alias(alias, name_to_class):
+def _maybe_resolve_alias(alias, name_to_class, name_to_constant):
   """Resolve the alias if possible.
 
   Args:
     alias: A pytd.Alias
     name_to_class: A class map used for resolution.
+    name_to_constant: A constant map used for resolution.
 
   Returns:
     None, if the alias pointed to an un-aliasable type.
@@ -1609,12 +1673,21 @@ def _maybe_resolve_alias(alias, name_to_class):
     # as constants to distinguish them from imports.
     return alias
   parts = alias.type.name.split(".")
-  if parts[0] not in name_to_class:
+  if parts[0] not in name_to_class and parts[0] not in name_to_constant:
     return alias
-  value = name_to_class[parts[0]]
+  prev_value = None
+  value = name_to_class.get(parts[0]) or name_to_constant[parts[0]]
   for part in parts[1:]:
+    prev_value = value
     # We can immediately return upon encountering an error, as load_pytd will
     # complain when it can't resolve the alias.
+    if isinstance(value, pytd.Constant):
+      if (not isinstance(value.type, pytd.NamedType) or
+          value.type.name not in name_to_class):
+        # TODO(rechen): Parameterized constants of generic classes should
+        # probably also be allowed.
+        return alias
+      value = name_to_class[value.type.name]
     if not isinstance(value, pytd.Class):
       return alias
     try:
@@ -1624,6 +1697,17 @@ def _maybe_resolve_alias(alias, name_to_class):
   if isinstance(value, pytd.Class):
     return pytd.Constant(
         alias.name, _make_type_type(pytd.NamedType(alias.type.name)))
+  elif isinstance(value, pytd.Function):
+    # We allow module-level aliases of methods from classes and class instances.
+    # When a static method is aliased, or a normal method is aliased from a
+    # class (not an instance), the entire method signature is copied. Otherwise,
+    # the first parameter ('self' or 'cls') is dropped.
+    new_value = value.Replace(name=alias.name).Replace(kind="method")
+    if value.kind == "staticmethod" or (
+        value.kind == "method" and isinstance(prev_value, pytd.Class)):
+      return new_value
+    return new_value.Replace(signatures=tuple(
+        s.Replace(params=s.params[1:]) for s in new_value.signatures))
   else:
     return value.Replace(name=alias.name)
 
@@ -1639,3 +1723,13 @@ def _handle_string_literal(value):
   if not match:
     return value
   return match.groups()[1][1:-1]
+
+
+def _builtin_keyword_constants():
+  defs = [
+      ("True", "bool"),
+      ("False", "bool"),
+      ("None", "NoneType"),
+      ("__debug__", "bool")
+  ]
+  return [pytd.Constant(name, pytd.NamedType(typ)) for name, typ in defs]
