@@ -1,5 +1,4 @@
 import hashlib
-import os
 import re
 import textwrap
 
@@ -16,10 +15,8 @@ IGNORE = object()
 
 
 def get_builtins_source(python_version):
-  filename = "builtins/%d/__builtin__.pytd" % python_version[0]
-  pytd_dir = os.path.dirname(pytd.__file__)
-  with open(os.path.join(pytd_dir, filename)) as f:
-    return f.read()
+  return pytd_utils.GetPredefinedFile(
+      "builtins/%d" % python_version[0], "builtins")[1]
 
 
 class _ParserTestBase(test_base.UnitTest):
@@ -516,6 +513,15 @@ class ParserTest(_ParserTestBase):
       x: Callable[[int, int], Any]
     """)
 
+  def test_all(self):
+    self.check("""
+      __all__ = ['a']
+    """, """
+      from typing import List
+
+      __all__: List[str]
+    """)
+
 
 class HomogeneousTypeTest(_ParserTestBase):
 
@@ -546,13 +552,6 @@ class HomogeneousTypeTest(_ParserTestBase):
     self.check("""
       from typing import Callable
 
-      x = ...  # type: Callable[[nothing], bool]""", """
-      from typing import Callable
-
-      x: Callable[[], bool]""")
-    self.check("""
-      from typing import Callable
-
       x = ...  # type: Callable[[int]]""", """
       from typing import Any, Callable
 
@@ -633,11 +632,11 @@ class NamedTupleTest(_ParserTestBase):
     self.check("x = ...  # type: NamedTuple('foo', [])", """
       from typing import Any, Tuple, Type, TypeVar
 
-      x: `namedtuple-foo-0`
+      x: namedtuple_foo_0
 
-      _Tnamedtuple-foo-0 = TypeVar('_Tnamedtuple-foo-0', bound=`namedtuple-foo-0`)
+      _Tnamedtuple_foo_0 = TypeVar('_Tnamedtuple_foo_0', bound=namedtuple_foo_0)
 
-      class `namedtuple-foo-0`(Tuple[()]):
+      class namedtuple_foo_0(Tuple[()]):
           __slots__ = []
           _asdict: Any
           __dict__: Any
@@ -646,7 +645,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-foo-0`]) -> `_Tnamedtuple-foo-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_foo_0]) -> _Tnamedtuple_foo_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
       """)
 
@@ -654,11 +653,11 @@ class NamedTupleTest(_ParserTestBase):
     expected = """
       from typing import Any, Tuple, Type, TypeVar
 
-      x: `namedtuple-foo-0`
+      x: namedtuple_foo_0
 
-      _Tnamedtuple-foo-0 = TypeVar('_Tnamedtuple-foo-0', bound=`namedtuple-foo-0`)
+      _Tnamedtuple_foo_0 = TypeVar('_Tnamedtuple_foo_0', bound=namedtuple_foo_0)
 
-      class `namedtuple-foo-0`(Tuple[int, str]):
+      class namedtuple_foo_0(Tuple[int, str]):
           __slots__ = ["a", "b"]
           a: int
           b: str
@@ -669,7 +668,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-foo-0`], a: int, b: str) -> `_Tnamedtuple-foo-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_foo_0], a: int, b: str) -> _Tnamedtuple_foo_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
     """
     self.check("x = ...  # type: NamedTuple('foo', [('a', int), ('b', str)])",
@@ -687,13 +686,13 @@ class NamedTupleTest(_ParserTestBase):
                """
       from typing import Any, Tuple, Type, TypeVar
 
-      x: `namedtuple-foo-0`
-      y: `namedtuple-foo-1`
+      x: namedtuple_foo_0
+      y: namedtuple_foo_1
 
-      _Tnamedtuple-foo-0 = TypeVar('_Tnamedtuple-foo-0', bound=`namedtuple-foo-0`)
-      _Tnamedtuple-foo-1 = TypeVar('_Tnamedtuple-foo-1', bound=`namedtuple-foo-1`)
+      _Tnamedtuple_foo_0 = TypeVar('_Tnamedtuple_foo_0', bound=namedtuple_foo_0)
+      _Tnamedtuple_foo_1 = TypeVar('_Tnamedtuple_foo_1', bound=namedtuple_foo_1)
 
-      class `namedtuple-foo-0`(Tuple[int]):
+      class namedtuple_foo_0(Tuple[int]):
           __slots__ = ["a"]
           a: int
           _asdict: Any
@@ -703,10 +702,10 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-foo-0`], a: int) -> `_Tnamedtuple-foo-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_foo_0], a: int) -> _Tnamedtuple_foo_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
 
-      class `namedtuple-foo-1`(Tuple[str]):
+      class namedtuple_foo_1(Tuple[str]):
           __slots__ = ["b"]
           b: str
           _asdict: Any
@@ -716,7 +715,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-foo-1`], b: str) -> `_Tnamedtuple-foo-1`: ...
+          def __new__(cls: Type[_Tnamedtuple_foo_1], b: str) -> _Tnamedtuple_foo_1: ...
           def __init__(self, *args, **kwargs) -> None: ...
         """)
   # pylint: enable=line-too-long
@@ -725,11 +724,11 @@ class NamedTupleTest(_ParserTestBase):
     self.check("X = NamedTuple('X', [])", """
       from typing import Any, Tuple, Type, TypeVar
 
-      X = `namedtuple-X-0`
+      X = namedtuple_X_0
 
-      _Tnamedtuple-X-0 = TypeVar('_Tnamedtuple-X-0', bound=`namedtuple-X-0`)
+      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
 
-      class `namedtuple-X-0`(Tuple[()]):
+      class namedtuple_X_0(Tuple[()]):
           __slots__ = []
           _asdict: Any
           __dict__: Any
@@ -738,7 +737,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-X-0`]) -> `_Tnamedtuple-X-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_X_0]) -> _Tnamedtuple_X_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
     """)
 
@@ -746,9 +745,9 @@ class NamedTupleTest(_ParserTestBase):
     self.check("class X(NamedTuple('X', [])): ...", """
       from typing import Any, Tuple, Type, TypeVar
 
-      _Tnamedtuple-X-0 = TypeVar('_Tnamedtuple-X-0', bound=`namedtuple-X-0`)
+      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
 
-      class `namedtuple-X-0`(Tuple[()]):
+      class namedtuple_X_0(Tuple[()]):
           __slots__ = []
           _asdict: Any
           __dict__: Any
@@ -757,10 +756,10 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-X-0`]) -> `_Tnamedtuple-X-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_X_0]) -> _Tnamedtuple_X_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
 
-      class X(`namedtuple-X-0`): ...
+      class X(namedtuple_X_0): ...
     """)
 
   def test_trailing_comma(self):
@@ -776,11 +775,11 @@ class NamedTupleTest(_ParserTestBase):
     """, """
       from typing import Any, Tuple, Type, TypeVar
 
-      Foo = `namedtuple-Foo-0`
+      Foo = namedtuple_Foo_0
 
-      _Tnamedtuple-Foo-0 = TypeVar('_Tnamedtuple-Foo-0', bound=`namedtuple-Foo-0`)
+      _Tnamedtuple_Foo_0 = TypeVar('_Tnamedtuple_Foo_0', bound=namedtuple_Foo_0)
 
-      class `namedtuple-Foo-0`(Tuple[int, str]):
+      class namedtuple_Foo_0(Tuple[int, str]):
           __slots__ = ["a", "b"]
           a: int
           b: str
@@ -791,7 +790,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-Foo-0`], a: int, b: str) -> `_Tnamedtuple-Foo-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_Foo_0], a: int, b: str) -> _Tnamedtuple_Foo_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
     """)
 
@@ -809,11 +808,11 @@ class NamedTupleTest(_ParserTestBase):
       from typing import Any, Tuple, Type, TypeVar
 
       from collections import namedtuple
-      Foo = `namedtuple-Foo-0`
+      Foo = namedtuple_Foo_0
 
-      _Tnamedtuple-Foo-0 = TypeVar('_Tnamedtuple-Foo-0', bound=`namedtuple-Foo-0`)
+      _Tnamedtuple_Foo_0 = TypeVar('_Tnamedtuple_Foo_0', bound=namedtuple_Foo_0)
 
-      class `namedtuple-Foo-0`(Tuple[Any, Any]):
+      class namedtuple_Foo_0(Tuple[Any, Any]):
           __slots__ = ["a", "b"]
           a: Any
           b: Any
@@ -824,7 +823,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-Foo-0`], a, b) -> `_Tnamedtuple-Foo-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_Foo_0], a, b) -> _Tnamedtuple_Foo_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
     """)
 
@@ -833,11 +832,11 @@ class NamedTupleTest(_ParserTestBase):
       from typing import Any, Tuple, Type, TypeVar
 
       from collections import namedtuple
-      X = `namedtuple-X-0`
+      X = namedtuple_X_0
 
-      _Tnamedtuple-X-0 = TypeVar('_Tnamedtuple-X-0', bound=`namedtuple-X-0`)
+      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
 
-      class `namedtuple-X-0`(Tuple[Any]):
+      class namedtuple_X_0(Tuple[Any]):
           __slots__ = ["y"]
           y: Any
           _asdict: Any
@@ -847,7 +846,7 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-X-0`], y) -> `_Tnamedtuple-X-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_X_0], y) -> _Tnamedtuple_X_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
     """
     self.check("""
@@ -868,9 +867,9 @@ class NamedTupleTest(_ParserTestBase):
     """, """
       from typing import Any, Tuple, Type, TypeVar
 
-      _Tnamedtuple-X-0 = TypeVar('_Tnamedtuple-X-0', bound=`namedtuple-X-0`)
+      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
 
-      class `namedtuple-X-0`(Tuple[int, str]):
+      class namedtuple_X_0(Tuple[int, str]):
           __slots__ = ["y", "z"]
           y: int
           z: str
@@ -881,10 +880,10 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-X-0`], y: int, z: str) -> `_Tnamedtuple-X-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_X_0], y: int, z: str) -> _Tnamedtuple_X_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
 
-      class X(`namedtuple-X-0`): ...
+      class X(namedtuple_X_0): ...
     """)
 
   def test_typing_namedtuple_class_with_method(self):
@@ -897,9 +896,9 @@ class NamedTupleTest(_ParserTestBase):
     """, """
       from typing import Any, Tuple, Type, TypeVar
 
-      _Tnamedtuple-X-0 = TypeVar('_Tnamedtuple-X-0', bound=`namedtuple-X-0`)
+      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
 
-      class `namedtuple-X-0`(Tuple[int, str]):
+      class namedtuple_X_0(Tuple[int, str]):
           __slots__ = ["y", "z"]
           y: int
           z: str
@@ -910,10 +909,10 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-X-0`], y: int, z: str) -> `_Tnamedtuple-X-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_X_0], y: int, z: str) -> _Tnamedtuple_X_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
 
-      class X(`namedtuple-X-0`):
+      class X(namedtuple_X_0):
           def foo(self) -> None: ...
     """)
 
@@ -926,9 +925,9 @@ class NamedTupleTest(_ParserTestBase):
     """, """
       from typing import Any, Tuple, Type, TypeVar
 
-      _Tnamedtuple-X-0 = TypeVar('_Tnamedtuple-X-0', bound=`namedtuple-X-0`)
+      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
 
-      class `namedtuple-X-0`(Tuple[int, str]):
+      class namedtuple_X_0(Tuple[int, str]):
           __slots__ = ["y", "z"]
           y: int
           z: str
@@ -939,10 +938,10 @@ class NamedTupleTest(_ParserTestBase):
           __getstate__: Any
           _make: Any
           _replace: Any
-          def __new__(cls: Type[`_Tnamedtuple-X-0`], y: int, z: str) -> `_Tnamedtuple-X-0`: ...
+          def __new__(cls: Type[_Tnamedtuple_X_0], y: int, z: str) -> _Tnamedtuple_X_0: ...
           def __init__(self, *args, **kwargs) -> None: ...
 
-      class X(dict, `namedtuple-X-0`): ...
+      class X(dict, namedtuple_X_0): ...
     """)
 
   def test_multi_namedtuple_parent(self):
@@ -1046,6 +1045,37 @@ class FunctionTest(_ParserTestBase):
       def f(x: int) -> None: ...
     """)
 
+  def test_typeignore_alias(self):
+    self.check("""
+      class Foo:
+          def f(self) -> None: ...
+          g = f  # type: ignore
+    """, """
+      class Foo:
+          def f(self) -> None: ...
+          def g(self) -> None: ...
+    """)
+
+  def test_typeignore_slots(self):
+    self.check("""
+      class Foo:
+          __slots__ = ["a", "b"]  # type: ignore
+    """, """
+      class Foo:
+          __slots__ = ["a", "b"]
+    """)
+
+  def test_typeignore_errorcode(self):
+    self.check("""
+      def f() -> None: ...  # type: ignore[override]
+      def g() -> None: ...  # type: ignore[var-annotated]
+      def h() -> None: ...  # type: ignore[abstract, no-untyped-def]
+    """, """
+      def f() -> None: ...
+      def g() -> None: ...
+      def h() -> None: ...
+    """)
+
   def test_decorators(self):
     # These tests are a bit questionable because most of the decorators only
     # make sense for methods of classes.  But this at least gives us some
@@ -1161,6 +1191,28 @@ class FunctionTest(_ParserTestBase):
       class Foo: ...
     """, "class Foo: ...")
 
+  def test_decorated_class(self):
+    self.check("""
+      from dataclasses import dataclass
+
+      @dataclass
+      class Foo: ...
+    """, """
+    import dataclasses
+
+    from dataclasses import dataclass
+
+    @dataclasses.dataclass
+    class Foo: ...
+    """)
+
+  def test_multiple_class_decorators(self):
+    self.check("""
+      @decorator1
+      @decorator2
+      class Foo: ...
+    """)
+
   def test_bad_decorated_class(self):
     self.check_error("""
       @classmethod
@@ -1216,6 +1268,11 @@ class FunctionTest(_ParserTestBase):
       def foo() -> int:
           raise Bar.Error()""", prologue="import Bar")
 
+  def test_invalid_body(self):
+    self.check_error("""
+      def foo(x) -> int:
+        a: str""", 2, "syntax error")
+
   def test_return(self):
     self.check("def foo() -> int: ...")
     self.check("def foo(): ...",
@@ -1288,44 +1345,6 @@ class ClassTest(_ParserTestBase):
       class Foo(metaclass=Meta, Bar): ...
       """, 1, "non-keyword arguments cannot follow keyword arguments")
 
-  def test_typed_dict_kwarg(self):
-    self.check("""
-      from typing import TypedDict
-
-      class Foo(TypedDict, total=False): ...
-    """, """
-      from typing import TypedDict
-
-      class Foo(TypedDict): ...
-    """)
-    self.check_error("""
-      class Foo(object, total=False): ...
-    """, 1, "'total' allowed as classdef kwarg only for TypedDict subclasses")
-
-  def test_typing_extensions_typed_dict(self):
-    self.check("""
-      from typing_extensions import TypedDict
-
-      class Foo(TypedDict, total=False): ...
-    """, """
-      import typing_extensions
-
-      from typing_extensions import TypedDict
-
-      class Foo(typing_extensions.TypedDict): ...
-    """)
-
-  def test_multiple_classdef_kwargs(self):
-    self.check("""
-      from typing import TypedDict
-
-      class Foo(TypedDict, total=False, metaclass=Meta): ...
-    """, """
-      from typing import TypedDict
-
-      class Foo(TypedDict, metaclass=Meta): ...
-    """)
-
   def test_shadow_pep484(self):
     self.check("""
       class List:
@@ -1922,31 +1941,31 @@ class PropertyDecoratorTest(_ParserTestBase):
 
   def test_property_decorator_bad_syntax(self):
     self.check_error("""
-      class A(object):
+      class A:
           @property
           def name(self, bad_arg): ...
     """, 1, "Unhandled decorator: property")
 
     self.check_error("""
-      class A(object):
+      class A:
           @name.setter
           def name(self): ...
       """, 1, "Unhandled decorator: name.setter")
 
     self.check_error("""
-      class A(object):
+      class A:
           @name.foo
           def name(self): ...
       """, 1, "Unhandled decorator: name.foo")
 
     self.check_error("""
-      class A(object):
+      class A:
           @notname.deleter
           def name(self): ...
       """, 1, "Unhandled decorator: notname.deleter")
 
     self.check_error("""
-      class A(object):
+      class A:
           @property
           @staticmethod
           def name(self): ...
@@ -2035,7 +2054,7 @@ class MergeSignaturesTest(_ParserTestBase):
 
   def test_method_and_property_error(self):
     self.check_error("""
-      class A(object):
+      class A:
           @property
           def name(self): ...
 
@@ -2044,7 +2063,7 @@ class MergeSignaturesTest(_ParserTestBase):
 
   def test_overloaded_signatures_disagree(self):
     self.check_error("""
-      class A(object):
+      class A:
           @staticmethod
           def foo(x: int): ...
           @classmethod
@@ -2109,7 +2128,7 @@ class MergeSignaturesTest(_ParserTestBase):
 
   def test_abstractmethod_conflict(self):
     self.check_error("""
-      class A(object):
+      class A:
           @abstractmethod
           def foo(x: int) -> str: ...
           def foo(x: int, y: int) -> str: ...
@@ -2400,9 +2419,9 @@ class LiteralTest(_ParserTestBase):
 
       x: Literal[True, 0, b"", u"", None]
     """, """
-      from typing import Literal, Optional, Union
+      from typing import Literal, Optional
 
-      x: Optional[Union[Literal[True], Literal[0], Literal[b""], Literal[u""]]]
+      x: Optional[Literal[True, 0, b"", u""]]
     """)
 
   def test_stray_number(self):
@@ -2440,6 +2459,212 @@ class LiteralTest(_ParserTestBase):
       x: Literal[42]
     """)
 
+  def test_unnest(self):
+    self.check("""
+      from typing import Literal
+      MyLiteralAlias = Literal[42]
+      x: Literal[MyLiteralAlias, Literal[Literal[True]], None]
+      y: Literal[1, Literal[2, Literal[3]]]
+    """, """
+      from typing import Literal, Optional
+
+      MyLiteralAlias = Literal[42]
+
+      x: Optional[Literal[42, True]]
+      y: Literal[1, 2, 3]
+    """)
+
+  def test_bad_value(self):
+    self.check_error("""
+      from typing import Literal
+      x: Literal[0.0]
+    """, 2, "Literal[0.0] not supported")
+
+
+class TypedDictTest(_ParserTestBase):
+
+  def test_assign(self):
+    self.check("""
+      from typing_extensions import TypedDict
+      X = TypedDict('X', {})
+    """, """
+      from typing import Any, Dict
+
+      from typing_extensions import TypedDict
+      X = Dict[str, Any]
+    """)
+
+  def test_assign_with_items(self):
+    self.check("""
+      from typing_extensions import TypedDict
+      X = TypedDict('X', {'a': int, 'b': str})
+    """, """
+      from typing import Any, Dict
+
+      from typing_extensions import TypedDict
+      X = Dict[str, Any]
+    """)
+
+  def test_assign_with_kwarg(self):
+    self.check("""
+      from typing_extensions import TypedDict
+      X = TypedDict('X', {}, total=False)
+    """, """
+      from typing import Any, Dict
+
+      from typing_extensions import TypedDict
+      X = Dict[str, Any]
+    """)
+
+  def test_trailing_comma(self):
+    self.check("""
+      from typing_extensions import TypedDict
+      X = TypedDict('X', {
+          'a': int,
+          'b': str,
+      },)
+    """, """
+      from typing import Any, Dict
+
+      from typing_extensions import TypedDict
+      X = Dict[str, Any]
+    """)
+
+  def test_kwarg(self):
+    self.check("""
+      from typing import TypedDict
+
+      class Foo(TypedDict, total=False): ...
+    """, """
+      from typing import TypedDict
+
+      class Foo(TypedDict): ...
+    """)
+    self.check_error("""
+      class Foo(object, total=False): ...
+    """, 1, "'total' allowed as classdef kwarg only for TypedDict subclasses")
+
+  def test_typing_extensions(self):
+    self.check("""
+      from typing_extensions import TypedDict
+
+      class Foo(TypedDict, total=False): ...
+    """, """
+      import typing_extensions
+
+      from typing_extensions import TypedDict
+
+      class Foo(typing_extensions.TypedDict): ...
+    """)
+
+  def test_multiple_classdef_kwargs(self):
+    self.check("""
+      from typing import TypedDict
+
+      class Foo(TypedDict, total=False, metaclass=Meta): ...
+    """, """
+      from typing import TypedDict
+
+      class Foo(TypedDict, metaclass=Meta): ...
+    """)
+
+
+class NewTypeTest(_ParserTestBase):
+
+  def test_basic(self):
+    self.check("""
+      from typing import NewType
+      X = NewType('X', int)
+    """, """
+      X = newtype_X_0
+
+      class newtype_X_0(int):
+          def __init__(self, val: int) -> None: ...
+    """)
+
+  def test_fullname(self):
+    self.check("""
+      import typing
+      X = typing.NewType('X', int)
+    """, """
+      X = newtype_X_0
+
+      class newtype_X_0(int):
+          def __init__(self, val: int) -> None: ...
+    """)
+
+
+class MethodAliasTest(_ParserTestBase):
+
+  def test_normal_method(self):
+    self.check("""
+      class Foo:
+          def f(self, x: int) -> None: ...
+      _foo: Foo
+      f1 = Foo.f
+      f2 = _foo.f
+    """, """
+      _foo: Foo
+
+      class Foo:
+          def f(self, x: int) -> None: ...
+
+      def f1(self, x: int) -> None: ...
+      def f2(x: int) -> None: ...
+    """)
+
+  def test_classmethod(self):
+    self.check("""
+      class Foo:
+          @classmethod
+          def f(cls, x: int) -> None: ...
+      _foo: Foo
+      f1 = Foo.f
+      f2 = _foo.f
+    """, """
+      _foo: Foo
+
+      class Foo:
+          @classmethod
+          def f(cls, x: int) -> None: ...
+
+      def f1(x: int) -> None: ...
+      def f2(x: int) -> None: ...
+    """)
+
+  def test_staticmethod(self):
+    self.check("""
+      class Foo:
+          @staticmethod
+          def f(x: int) -> None: ...
+      _foo: Foo
+      f1 = Foo.f
+      f2 = _foo.f
+    """, """
+      _foo: Foo
+
+      class Foo:
+          @staticmethod
+          def f(x: int) -> None: ...
+
+      def f1(x: int) -> None: ...
+      def f2(x: int) -> None: ...
+    """)
+
+  def test_nested_constant(self):
+    self.check("""
+      class Foo:
+          foo: Foo
+          def f(self, x: int) -> None: ...
+      f = Foo.foo.f
+    """, """
+      class Foo:
+          foo: Foo
+          def f(self, x: int) -> None: ...
+
+      def f(x: int) -> None: ...
+    """)
+
 
 if __name__ == "__main__":
   unittest.main()
