@@ -1,20 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-
 """Functions for optimizing pytd syntax trees.
 
    pytd files come from various sources, and are typically redundant (duplicate
@@ -29,6 +12,7 @@ import logging
 from pytype import utils
 from pytype.pytd import abc_hierarchy
 from pytype.pytd import booleq
+from pytype.pytd import escape
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import type_match
@@ -42,13 +26,13 @@ class RenameUnknowns(visitors.Visitor):
   """Give unknowns that map to the same set of concrete types the same name."""
 
   def __init__(self, mapping):
-    super(RenameUnknowns, self).__init__()
+    super().__init__()
     self.name_to_cls = {name: hash(cls) for name, cls in mapping.items()}
     self.cls_to_canonical_name = {
         cls: name for name, cls in self.name_to_cls.items()}
 
   def VisitClassType(self, node):
-    if node.name.startswith("~unknown"):
+    if escape.is_unknown(node.name):
       return pytd.ClassType(
           self.cls_to_canonical_name[self.name_to_cls[node.name]], None)
     else:
@@ -78,15 +62,15 @@ class RemoveRedundantSignatures(visitors.Visitor):
 
   For example, this transforms
     def f(x: int) -> float
-    def f(x: int or float) -> float
+    def f(x: Union[int, float]) -> float
   to
-    def f(x: int or float) -> float
+    def f(x: Union[int, float]) -> float
   In order to be removed, a signature has to be "contained" (a subclass of)
   an existing one.
   """
 
   def __init__(self, hierarchy):
-    super(RemoveRedundantSignatures, self).__init__()
+    super().__init__()
     self.match = type_match.TypeMatch(hierarchy.GetSuperClasses(),
                                       any_also_is_bottom=False)
     self.subst = {}
@@ -127,20 +111,20 @@ class SimplifyUnions(visitors.Visitor):
   """Remove duplicate or redundant entries in union types.
 
   For example, this transforms
-    a: int or int
-    b: int or ?
-    c: int or (int or float)
+    a: Union[int, int]
+    b: Union[int, Any]
+    c: Union[int, int, float]
   to
     a: int
-    b: ?
-    c: int or float
+    b: Any
+    c: Union[int, float]
   """
 
   def VisitUnionType(self, union):
     return pytd_utils.JoinTypes(union.type_list)
 
 
-class _ReturnsAndExceptions(object):
+class _ReturnsAndExceptions:
   """Mutable class for collecting return types and exceptions of functions.
 
   The collecting is stable: Items are kept in the order in which they were
@@ -175,7 +159,7 @@ class CombineReturnsAndExceptions(visitors.Visitor):
     def f(x: int) -> int:
       raise IndexError()
   to
-    def f(x: int) -> float or int:
+    def f(x: int) -> Union[float, int]:
       raise IndexError()
       raise OverflowError()
   """
@@ -232,14 +216,14 @@ class CombineContainers(visitors.Visitor):
   """Change unions of containers to containers of unions.
 
   For example, this transforms
-    list[int] or list[float]
+    Union[list[int], list[float]]
   to
-    list[int or float]
+    list[Union[int, float]]
   .
   """
 
   _CONTAINER_NAMES = {
-      pytd.TupleType: ("__builtin__.tuple", "typing.Tuple"),
+      pytd.TupleType: ("builtins.tuple", "typing.Tuple"),
       pytd.CallableType: ("typing.Callable",),
   }
 
@@ -303,7 +287,7 @@ class CombineContainers(visitors.Visitor):
       for t in union.type_list:
         if merge_tuples and isinstance(t, pytd.TupleType):
           t = pytd.GenericType(base_type=t.base_type,
-                               parameters=(pytd.UnionType(t.parameters),))
+                               parameters=(pytd_utils.JoinTypes(t.parameters),))
         elif merge_callables and isinstance(t, pytd.CallableType):
           t = pytd.GenericType(base_type=t.base_type,
                                parameters=(pytd.AnythingType(), t.ret))
@@ -349,7 +333,7 @@ class Factorize(visitors.Visitor):
     def f(x: float, y: int)
     def f(x: float, y: float)
   to
-    def f(x: int or float, y: int or float)
+    def f(x: Union[int, float], y: Union[int, float])
   """
 
   def _GroupByOmittedArg(self, signatures, i):
@@ -488,7 +472,7 @@ class ApplyOptionalArguments(visitors.Visitor):
     return f.Replace(signatures=tuple(new_signatures))
 
 
-class SuperClassHierarchy(object):
+class SuperClassHierarchy:
   """Utility class for optimizations working with superclasses."""
 
   def __init__(self, superclasses):
@@ -558,7 +542,7 @@ class SimplifyUnionsWithSuperclasses(visitors.Visitor):
   """Simplify Unions with superclasses.
 
   E.g., this changes
-    int or bool
+    Union[int, bool]
   to
     int
   since bool is a subclass of int.
@@ -568,13 +552,13 @@ class SimplifyUnionsWithSuperclasses(visitors.Visitor):
   """
 
   def __init__(self, hierarchy):
-    super(SimplifyUnionsWithSuperclasses, self).__init__()
+    super().__init__()
     self.hierarchy = hierarchy
 
   def VisitUnionType(self, union):
     c = collections.Counter()
     for t in set(union.type_list):
-      # TODO(rechen): How can we make this work with GenericType?
+      # TODO(b/159052794): How can we make this work with GenericType?
       if isinstance(t, pytd.GENERIC_BASE_TYPE):
         c += collections.Counter(self.hierarchy.ExpandSubClasses(str(t)))
     # Below, c[str[t]] can be zero - that's the default for non-existent items
@@ -588,13 +572,13 @@ class FindCommonSuperClasses(visitors.Visitor):
   """Find common super classes. Optionally also uses abstract base classes.
 
   E.g., this changes
-    def f(x: list or tuple, y: frozenset or set) -> int or float
+    def f(x: Union[list, tuple], y: Union[frozenset, set]) -> Union[int, float]
   to
     def f(x: Sequence, y: Set) -> Real
   """
 
   def __init__(self, hierarchy):
-    super(FindCommonSuperClasses, self).__init__()
+    super().__init__()
     self.hierarchy = hierarchy
 
   def VisitUnionType(self, union):
@@ -602,7 +586,7 @@ class FindCommonSuperClasses(visitors.Visitor):
 
     This is a lossy optimization that tries to map a list of types to a common
     base type. For example, int and bool are both base classes of int, so it
-    would convert "int or bool" to "int".
+    would convert "Union[int, bool]" to "int".
 
     Arguments:
       union: A union type.
@@ -644,8 +628,8 @@ class CollapseLongUnions(visitors.Visitor):
   """
 
   def __init__(self, max_length=7):
-    assert isinstance(max_length, six.integer_types)
-    super(CollapseLongUnions, self).__init__()
+    assert isinstance(max_length, int)
+    super().__init__()
     self.generic_type = pytd.AnythingType()
     self.max_length = max_length
 
@@ -662,8 +646,8 @@ class AdjustGenericType(visitors.Visitor):
   """Changes the generic type from "object" to "Any"."""
 
   def __init__(self):
-    super(AdjustGenericType, self).__init__()
-    self.old_generic_type = pytd.ClassType("__builtin__.object")
+    super().__init__()
+    self.old_generic_type = pytd.ClassType("builtins.object")
     self.new_generic_type = pytd.AnythingType()
 
   def VisitClassType(self, t):
@@ -711,14 +695,14 @@ class AddInheritedMethods(visitors.Visitor):
     if any(base for base in cls.parents if isinstance(base, pytd.NamedType)):
       raise AssertionError("AddInheritedMethods needs a resolved AST")
     # Filter out only the types we can reason about.
-    # TODO(kramm): Do we want handle UnionTypes and GenericTypes at some point?
+    # TODO(b/159052794): Do we want handle UnionTypes and GenericTypes?
     bases = [base.cls
              for base in cls.parents
              if isinstance(base, pytd.ClassType)]
     # Don't pull in methods that are named the same as existing methods in
     # this class, local methods override parent class methods.
     names = {m.name for m in cls.methods} | {c.name for c in cls.constants}
-    # TODO(kramm): This should do full-blown MRO.
+    # TODO(b/159052405): This should do full-blown MRO.
     adjust_self = visitors.AdjustSelf(force=True)
     adjust_self.class_types.append(visitors.ClassAsType(cls))
     new_methods = list(cls.methods)
@@ -750,7 +734,7 @@ class PullInMethodClasses(visitors.Visitor):
   """
 
   def __init__(self):
-    super(PullInMethodClasses, self).__init__()
+    super().__init__()
     self._module = None
     self._total_count = collections.defaultdict(int)
     self._processed_count = collections.defaultdict(int)
@@ -807,7 +791,7 @@ class PullInMethodClasses(visitors.Visitor):
     """
     if not self._processed_count[cls.name]:
       # Leave standalone classes alone. E.g. the pytd files in
-      # pytd/builtins/ defines classes not used by anything else.
+      # stubs/builtins/ defines classes not used by anything else.
       return False
     return self._processed_count[cls.name] == self._total_count[cls.name]
 
@@ -853,11 +837,11 @@ class AbsorbMutableParameters(visitors.Visitor):
 
   For example, this will change
     def f(x: list[int]):
-      x = list[int or float]
+      x = list[Union[int, float]]
   to
-    def f(x: list[int] or list[int or float])
+    def f(x: Union[list[int], list[Union[int, float]])
   .
-  (Use optimize.CombineContainers to then change x to list[int or float].)
+  (Use optimize.CombineContainers to then change x to list[Union[int, float]].)
 
   This also works for methods - it will then potentially change the type of
   "self". The resulting AST is temporary and needs careful handling.
@@ -897,7 +881,7 @@ class TypeParameterScope(visitors.Visitor):
   """Common superclass for optimizations that track type parameters."""
 
   def __init__(self):
-    super(TypeParameterScope, self).__init__()
+    super().__init__()
     self.type_params_stack = [{}]
 
   def EnterClass(self, cls):
@@ -930,7 +914,7 @@ class MergeTypeParameters(TypeParameterScope):
 
   For example, this will change
     class A(typing.Generic(T)):
-      def append(self, T or T2) -> T2
+      def append(self, Union[T, T2]) -> T2
   to
     class A(typing.Generic(T)):
       def append(self, T) -> T
@@ -941,16 +925,16 @@ class MergeTypeParameters(TypeParameterScope):
   MergeTypeParameters transforms
     class list(typing.Generic(T)):
       def append(self, v: T2) -> NoneType:
-        self = T or T2
+        self = Union[T, T2]
   to
     class list(typing.Generic(T')):
       def append(self, V:T') -> NoneType
   by creating a *new* template variable T' that propagates the
-  mutations to the outermost level (in this example, T' = T or T2)
+  mutations to the outermost level (in this example, T' = Union[T, T2])
   """
 
   def __init__(self):
-    super(MergeTypeParameters, self).__init__()
+    super().__init__()
     self.type_param_union = None
 
   def _AppendNew(self, l1, l2):
@@ -962,13 +946,13 @@ class MergeTypeParameters(TypeParameterScope):
 
   def EnterSignature(self, node):
     # Necessary because TypeParameterScope also defines this function
-    super(MergeTypeParameters, self).EnterSignature(node)
+    super().EnterSignature(node)
     assert self.type_param_union is None
     self.type_param_union = collections.defaultdict(list)
 
   def LeaveSignature(self, node):
     # Necessary because TypeParameterScope also defines this function
-    super(MergeTypeParameters, self).LeaveSignature(node)
+    super().LeaveSignature(node)
     self.type_param_union = None
 
   def VisitUnionType(self, u):
@@ -1011,7 +995,6 @@ class MergeTypeParameters(TypeParameterScope):
     else:
       # It's a function type parameter that appears in a union with other
       # function type parameters.
-      # TODO(kramm): We could merge those, too.
       return [item]
 
   def VisitSignature(self, sig):
@@ -1043,7 +1026,7 @@ def Optimize(node,
     builtins: Definitions of all of the external types in node.
     lossy: Allow optimizations that change the meaning of the pytd.
     use_abcs: Use abstract base classes to represent unions like
-        e.g. "float or int" as "Real".
+        e.g. "Union[float, int]" as "Real".
     max_union: How many types we allow in a union before we simplify
         it to just "object".
     remove_mutable: Whether to simplify mutable parameters to normal
