@@ -1,25 +1,5 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-# Our way of using namedtuple is confusing pylint.
-# pylint: disable=no-member
-
 """AST representation of a pytd file."""
 
-
 import collections
 import itertools
 
@@ -32,7 +12,7 @@ from pytype.pytd.parse import preconditions
 # Note that the mixin must be registered with preconditions.register() below.
 
 
-class Type(object):
+class Type:
 
   __slots__ = ()
 
@@ -124,6 +104,7 @@ class Class(node.Node('name: str',
                       'methods: tuple[Function]',
                       'constants: tuple[Constant]',
                       'classes: tuple[Class]',
+                      'decorators: tuple[Function or {Type}]',
                       'slots: None or tuple[str]',
                       'template: tuple[TemplateItem]')):
   """Represents a class declaration.
@@ -289,7 +270,7 @@ class TypeParameter(node.Node('name: str',
 
   def __lt__(self, other):
     try:
-      return super(TypeParameter, self).__lt__(other)
+      return super().__lt__(other)
     except TypeError:
       # In Python 3, str and None are not comparable. Declare None to be less
       # than every str so that visitors.AdjustTypeParameters.VisitTypeDeclUnit
