@@ -1,17 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 """Utilities for pytd.
 
 This provides a utility function to access data files in a way that works either
@@ -41,9 +27,10 @@ from six.moves import cPickle
 _PICKLE_PROTOCOL = cPickle.HIGHEST_PROTOCOL
 _PICKLE_RECURSION_LIMIT_AST = 40000
 
-
 ANON_PARAM = re.compile(r"_[0-9]+")
 
+_TUPLE_NAMES = ("builtins.tuple", "typing.Tuple")
+
 
 def UnpackUnion(t):
   """Return the type list for union type, or a list with the type itself."""
@@ -55,13 +42,13 @@ def UnpackUnion(t):
 
 def MakeClassOrContainerType(base_type, type_arguments, homogeneous):
   """If we have type params, build a generic type, a normal type otherwise."""
-  if not type_arguments:
+  if not type_arguments and (homogeneous or base_type.name not in _TUPLE_NAMES):
     return base_type
   if homogeneous:
     container_type = pytd.GenericType
   elif base_type.name == "typing.Callable":
     container_type = pytd.CallableType
-  elif base_type.name in ("__builtin__.tuple", "typing.Tuple"):
+  elif base_type.name in _TUPLE_NAMES:
     container_type = pytd.TupleType
   else:
     container_type = pytd.GenericType
@@ -137,7 +124,7 @@ def disabled_function(*unused_args, **unused_kwargs):
   raise AssertionError("Cannot call disabled function.")
 
 
-class TypeMatcher(object):
+class TypeMatcher:
   """Base class for modules that match types against each other.
 
   Maps pytd node types (<type1>, <type2>) to a method "match_<type1>_<type2>".
@@ -273,7 +260,7 @@ def _check_intersection(items1, items2, name1, name2):
                     (", ".join(map(repr, sorted(items))), name1, name2))
 
 
-class TypeBuilder(object):
+class TypeBuilder:
   """Utility class for building union types."""
 
   def __init__(self):
@@ -314,7 +301,7 @@ class OrderedSet(collections.OrderedDict):
   """A simple ordered set."""
 
   def __init__(self, iterable=None):
-    super(OrderedSet, self).__init__((item, None) for item in (iterable or []))
+    super().__init__((item, None) for item in (iterable or []))
 
   def add(self, item):
     self[item] = None
@@ -340,7 +327,7 @@ def WrapsDict(member_name, writable=False, implement_len=False):
   """
   src = "if True:\n"  # To allow the code below to be indented
   src += """
-    class WrapsDict(object):
+    class WrapsDict:
 
       def __getitem__(self, key):
         return self.{member_name}[key]
@@ -420,12 +407,12 @@ def WrapsDict(member_name, writable=False, implement_len=False):
   return namespace["WrapsDict"]  # pytype: disable=key-error
 
 
-def GetPredefinedFile(pytd_subdir, module, extension=".pytd",
+def GetPredefinedFile(stubs_subdir, module, extension=".pytd",
                       as_package=False):
   """Get the contents of a predefined PyTD, typically with a file name *.pytd.
 
   Arguments:
-    pytd_subdir: the directory, typically "builtins" or "stdlib"
+    stubs_subdir: the directory, typically "builtins" or "stdlib"
     module: module name (e.g., "sys" or "__builtins__")
     extension: either ".pytd" or ".py"
     as_package: try the module as a directory with an __init__ file
@@ -438,28 +425,28 @@ def GetPredefinedFile(pytd_subdir, module, extension=".pytd",
   if as_package:
     parts.append("__init__")
   mod_path = os.path.join(*parts) + extension
-  path = os.path.join("pytd", pytd_subdir, mod_path)
-  return path, pytype_source_utils.load_pytype_file(path)
+  path = os.path.join("stubs", stubs_subdir, mod_path)
+  return path, pytype_source_utils.load_text_file(path)
 
 
-def LoadPickle(filename, compress=False):
-  if compress:
-    with gzip.GzipFile(filename, "rb") as fi:
-      # TODO(b/117797409): Remove the disable once the typeshed bug is fixed.
-      return cPickle.load(fi)  # pytype: disable=wrong-arg-types
-  else:
-    with open(filename, "rb") as fi:
+def LoadPickle(filename, compress=False, open_function=open):
+  with open_function(filename, "rb") as fi:
+    if compress:
+      with gzip.GzipFile(fileobj=fi) as zfi:
+        # TODO(b/117797409): Remove the disable once the typeshed bug is fixed.
+        return cPickle.load(zfi)  # pytype: disable=wrong-arg-types
+    else:
       return cPickle.load(fi)
 
 
-def SavePickle(data, filename=None, compress=False):
+def SavePickle(data, filename=None, compress=False, open_function=open):
   """Pickle the data."""
   recursion_limit = sys.getrecursionlimit()
   sys.setrecursionlimit(_PICKLE_RECURSION_LIMIT_AST)
   assert not compress or filename, "gzip only supported with a filename"
   try:
     if compress:
-      with open(filename, mode="wb") as fi:
+      with open_function(filename, mode="wb") as fi:
         # We blank the filename and set the mtime explicitly to produce
         # deterministic gzip files.
         with gzip.GzipFile(filename="", mode="wb",
@@ -467,7 +454,7 @@ def SavePickle(data, filename=None, compress=False):
           # TODO(b/117797409): Remove disable once typeshed bug is fixed.
           cPickle.dump(data, zfi, _PICKLE_PROTOCOL)  # pytype: disable=wrong-arg-types
     elif filename is not None:
-      with open(filename, "wb") as fi:
+      with open_function(filename, "wb") as fi:
         cPickle.dump(data, fi, _PICKLE_PROTOCOL)
     else:
       return cPickle.dumps(data, _PICKLE_PROTOCOL)
@@ -564,8 +551,10 @@ def MergeBaseClass(cls, base):
   class_names = [c.name for c in cls.classes]
   classes = cls.classes + tuple(c for c in base.classes
                                 if c.name not in class_names)
+  # Keep decorators from the base class only if the derived class has none
+  decorators = cls.decorators or base.decorators
   if cls.slots:
-    slots = cls.clots + tuple(s for s in base.slots or () if s not in cls.slots)
+    slots = cls.slots + tuple(s for s in base.slots or () if s not in cls.slots)
   else:
     slots = base.slots
   return pytd.Class(name=cls.name,
@@ -574,5 +563,6 @@ def MergeBaseClass(cls, base):
                     methods=methods,
                     constants=constants,
                     classes=classes,
+                    decorators=decorators,
                     slots=slots,
                     template=cls.template or base.template)
