@@ -1,19 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-
 import os
 import textwrap
 
@@ -35,9 +19,10 @@ class TestUtils(parser_test_base.ParserTest):
   def test_unpack_union(self):
     """Test for UnpackUnion."""
     ast = self.Parse("""
-      c1 = ...  # type: int or float
+      from typing import Union
+      c1 = ...  # type: Union[int, float]
       c2 = ...  # type: int
-      c3 = ...  # type: list[int or float]""")
+      c3 = ...  # type: list[Union[int, float]]""")
     c1 = ast.Lookup("c1").type
     c2 = ast.Lookup("c2").type
     c3 = ast.Lookup("c3").type
@@ -50,7 +35,7 @@ class TestUtils(parser_test_base.ParserTest):
     ast1 = self.Parse("""
       c1 = ...  # type: int
 
-      def f1() -> int
+      def f1() -> int: ...
 
       class Class1(object):
         pass
@@ -58,7 +43,7 @@ class TestUtils(parser_test_base.ParserTest):
     ast2 = self.Parse("""
       c2 = ...  # type: int
 
-      def f2() -> int
+      def f2() -> int: ...
 
       class Class2(object):
         pass
@@ -67,8 +52,8 @@ class TestUtils(parser_test_base.ParserTest):
       c1 = ...  # type: int
       c2 = ...  # type: int
 
-      def f1() -> int
-      def f2() -> int
+      def f1() -> int: ...
+      def f2() -> int: ...
 
       class Class1(object):
           pass
@@ -94,11 +79,11 @@ class TestUtils(parser_test_base.ParserTest):
 
   def test_concat_type_parameters(self):
     """Test for concatenating ASTs with type parameters."""
-    ast1 = self.Parse("""T = TypeVar("T")""", name="__builtin__")
+    ast1 = self.Parse("""T = TypeVar("T")""", name="builtins")
     ast2 = self.Parse("""T = TypeVar("T")""")
     combined = pytd_utils.Concat(ast1, ast2)
-    self.assertEqual(combined.Lookup("__builtin__.T"),
-                     pytd.TypeParameter("T", scope="__builtin__"))
+    self.assertEqual(combined.Lookup("builtins.T"),
+                     pytd.TypeParameter("T", scope="builtins"))
     self.assertEqual(combined.Lookup("T"), pytd.TypeParameter("T", scope=None))
 
   def test_join_types(self):
@@ -131,7 +116,7 @@ class TestUtils(parser_test_base.ParserTest):
     self.assertIsInstance(pytd_utils.JoinTypes([]), pytd.NothingType)
 
   def test_join_anything_types(self):
-    """Test that JoinTypes() simplifies unions containing '?'."""
+    """Test that JoinTypes() simplifies unions containing 'Any'."""
     types = [pytd.AnythingType(), pytd.NamedType("a")]
     self.assertIsInstance(pytd_utils.JoinTypes(types), pytd.AnythingType)
 
@@ -160,22 +145,6 @@ class TestUtils(parser_test_base.ParserTest):
         pytd.Function("f2", (s2, s2), pytd.METHOD),
         mykeyword="foobar"))
 
-  def test_print(self):
-    """Smoketest for printing pytd."""
-    ast = self.Parse("""
-      c1 = ...  # type: int
-      T = TypeVar('T')
-      class A(typing.Generic[T], object):
-        bar = ...  # type: T
-        def foo(self, x: list[int], y: T) -> list[T] or float:
-          raise ValueError()
-      X = TypeVar('X')
-      Y = TypeVar('Y')
-      def bar(x: X or Y) -> ?
-    """)
-    # TODO(kramm): Do more extensive testing.
-    pytd_utils.Print(ast)
-
   def test_named_type_with_module(self):
     """Test NamedTypeWithModule()."""
     self.assertEqual(pytd_utils.NamedTypeWithModule("name"),
@@ -195,15 +164,15 @@ class TestUtils(parser_test_base.ParserTest):
     """Test WrapTypeDeclUnit."""
     ast1 = self.Parse("""
       c = ...  # type: int
-      def f(x: int) -> int
-      def f(x: float) -> float
+      def f(x: int) -> int: ...
+      def f(x: float) -> float: ...
       class A(object):
         pass
     """)
     ast2 = self.Parse("""
       c = ...  # type: float
       d = ...  # type: int
-      def f(x: complex) -> complex
+      def f(x: complex) -> complex: ...
       class B(object):
         pass
     """)
@@ -212,11 +181,12 @@ class TestUtils(parser_test_base.ParserTest):
         ast1.classes + ast1.functions + ast1.constants +
         ast2.classes + ast2.functions + ast2.constants)
     expected = textwrap.dedent("""
-      c = ...  # type: int or float
+      from typing import Union
+      c = ...  # type: Union[int, float]
       d = ...  # type: int
-      def f(x: int) -> int
-      def f(x: float) -> float
-      def f(x: complex) -> complex
+      def f(x: int) -> int: ...
+      def f(x: float) -> float: ...
+      def f(x: complex) -> complex: ...
       class A(object):
         pass
       class B(object):
@@ -400,18 +370,20 @@ class TestUtils(parser_test_base.ParserTest):
     # different. The union types are different (int,str) vs (str,int) but the
     # ordering is ignored when testing for equality (which ASTeq uses).
     src1 = textwrap.dedent("""
-        def foo(a: int or str) -> C
+        from typing import Union
+        def foo(a: Union[int, str]) -> C: ...
         T = TypeVar('T')
         class C(typing.Generic[T], object):
-            def bar(x: T) -> NoneType
+            def bar(x: T) -> NoneType: ...
         CONSTANT = ...  # type: C[float]
         """)
     src2 = textwrap.dedent("""
+        from typing import Union
         CONSTANT = ...  # type: C[float]
         T = TypeVar('T')
         class C(typing.Generic[T], object):
-            def bar(x: T) -> NoneType
-        def foo(a: str or int) -> C
+            def bar(x: T) -> NoneType: ...
+        def foo(a: Union[str, int]) -> C: ...
         """)
     tree1 = parser.parse_string(src1, python_version=self.python_version)
     tree2 = parser.parse_string(src2, python_version=self.python_version)
@@ -427,6 +399,7 @@ class TestUtils(parser_test_base.ParserTest):
     self.assertIsInstance(tree2, pytd.TypeDeclUnit)
     # For the ==, != tests, TypeDeclUnit uses identity
     # pylint: disable=g-generic-assert
+    # pylint: disable=comparison-with-itself
     self.assertTrue(tree1 == tree1)
     self.assertTrue(tree2 == tree2)
     self.assertFalse(tree1 == tree2)
@@ -436,6 +409,7 @@ class TestUtils(parser_test_base.ParserTest):
     self.assertTrue(tree1 != tree2)
     self.assertTrue(tree2 != tree1)
     # pylint: enable=g-generic-assert
+    # pylint: enable=comparison-with-itself
     self.assertEqual(tree1, tree1)
     self.assertEqual(tree2, tree2)
     self.assertNotEqual(tree1, tree2)
@@ -471,9 +445,9 @@ class TestDataFiles(parser_test_base.ParserTest):
   def test_get_predefined_file_basic(self):
     # smoke test, only checks that it doesn't throw, the filepath is correct,
     # and the result is a string
-    path, src = pytd_utils.GetPredefinedFile(self.BUILTINS, "__builtin__")
-    self.assertEqual(path, "pytd/builtins/3/__builtin__.pytd")
-    self.assertIsInstance(src, bytes)
+    path, src = pytd_utils.GetPredefinedFile(self.BUILTINS, "builtins")
+    self.assertEqual(path, "stubs/builtins/3/builtins.pytd")
+    self.assertIsInstance(src, str)
 
   def test_get_predefined_file_throws(self):
     # smoke test, only checks that it does throw
@@ -485,25 +459,25 @@ class TestDataFiles(parser_test_base.ParserTest):
   def test_pytd_builtin2(self):
     """Verify 'import sys' for python2."""
     subdir = "builtins/2"
-    _, import_contents = pytd_utils.GetPredefinedFile(subdir, "__builtin__")
-    with open(os.path.join(os.path.dirname(pytd.__file__),
-                           subdir, "__builtin__.pytd"), "r") as fi:
+    _, import_contents = pytd_utils.GetPredefinedFile(subdir, "builtins")
+    with open(os.path.join(os.path.dirname(file_utils.__file__), "stubs",
+                           subdir, "builtins.pytd"), "r") as fi:
       file_contents = fi.read()
-    self.assertMultiLineEqual(import_contents.decode("utf-8"), file_contents)
+    self.assertMultiLineEqual(import_contents, file_contents)
 
   def test_pytd_builtin3(self):
     """Verify 'import sys' for python3."""
     subdir = "builtins/3"
-    _, import_contents = pytd_utils.GetPredefinedFile(subdir, "__builtin__")
-    with open(os.path.join(os.path.dirname(pytd.__file__),
-                           subdir, "__builtin__.pytd"), "r") as fi:
+    _, import_contents = pytd_utils.GetPredefinedFile(subdir, "builtins")
+    with open(os.path.join(os.path.dirname(file_utils.__file__), "stubs",
+                           subdir, "builtins.pytd"), "r") as fi:
       file_contents = fi.read()
-    self.assertMultiLineEqual(import_contents.decode("utf-8"), file_contents)
+    self.assertMultiLineEqual(import_contents, file_contents)
 
   def test_pytd_builtin_is_package(self):
     subdir = "builtins/2and3"
     path, _ = pytd_utils.GetPredefinedFile(subdir, "attr", as_package=True)
-    self.assertEqual(path, "pytd/builtins/2and3/attr/__init__.pytd")
+    self.assertEqual(path, "stubs/builtins/2and3/attr/__init__.pytd")
 
   def test_type_builder(self):
     t = pytd_utils.TypeBuilder()
@@ -512,5 +486,59 @@ class TestDataFiles(parser_test_base.ParserTest):
     self.assertTrue(t)
 
 
+class PrintTest(parser_test_base.ParserTest):
+  """Test pytd_utils.Print."""
+
+  def test_smoke(self):
+    """Smoketest for printing pytd."""
+    ast = self.Parse("""
+      from typing import Any, Union
+      c1 = ...  # type: int
+      T = TypeVar('T')
+      class A(typing.Generic[T], object):
+        bar = ...  # type: T
+        def foo(self, x: list[int], y: T) -> Union[list[T], float]:
+          raise ValueError()
+      X = TypeVar('X')
+      Y = TypeVar('Y')
+      def bar(x: Union[X, Y]) -> Any: ...
+    """)
+    # TODO(b/159051689): Do more extensive testing.
+    pytd_utils.Print(ast)
+
+  def test_literal(self):
+    ast = self.Parse("""
+      from typing import Literal
+      x1: Literal[""]
+      x2: Literal[b""]
+      x3: Literal[u""]
+      x4: Literal[0]
+      x5: Literal[True]
+      x6: Literal[None]
+    """)
+    ast = ast.Visit(visitors.LookupBuiltins(self.loader.builtins))
+    self.assertMultiLineEqual(pytd_utils.Print(ast), textwrap.dedent("""
+      from typing import Literal
+
+      x1: Literal[""]
+      x2: Literal[b""]
+      x3: Literal[u""]
+      x4: Literal[0]
+      x5: Literal[True]
+      x6: None
+    """).strip())
+
+  def test_literal_union(self):
+    ast = self.Parse("""
+      from typing import Literal, Union
+      x: Union[Literal["x"], Literal["y"]]
+    """)
+    self.assertMultiLineEqual(pytd_utils.Print(ast), textwrap.dedent("""
+      from typing import Literal
+
+      x: Literal["x", "y"]
+    """).strip())
+
+
 if __name__ == "__main__":
   unittest.main()
