@@ -1,19 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 """Visitor(s) for walking ASTs.
 
 This module contains broadly useful basic visitors. Visitors that are more
@@ -23,175 +7,19 @@ like to use, feel free to propose moving it here.
 
 import collections
 import logging
+import re
 
+from pytype.pytd import base_visitor
+from pytype.pytd import pep484
 from pytype.pytd import pytd
 from pytype.pytd.parse import parser_constants
-from pytype.typegraph import cfg_utils
 
 
-# A convenient value for unchecked_node_classnames if a visitor wants to
-# use unchecked nodes everywhere.
-ALL_NODE_NAMES = type(
-    "contains_everything",
-    (),
-    {"__contains__": lambda *args: True})()
-
-
-class _NodeClassInfo(object):
-  """Representation of a node class in the precondition graph."""
-
-  def __init__(self, cls):
-    self.cls = cls  # The class object.
-    self.name = cls.__name__
-    # The set of NodeClassInfo objects that may appear below this particular
-    # type of node.  Initially empty, filled in by examining preconditions.
-    self.outgoing = set()
-
-
-def _FindNodeClasses():
-  """Yields _NodeClassInfo objects for each node found in pytd."""
-  for name in dir(pytd):
-    value = getattr(pytd, name)
-    if isinstance(value, type) and hasattr(value, "_CHECKER"):
-      yield _NodeClassInfo(value)
-
-
-_IGNORED_TYPENAMES = set(["str", "bool", "int", "NoneType"])
-_ancestor_map = None  # Memoized ancestors map.
-
-
-def _GetAncestorMap():
-  """Return a map of node class names to a set of ancestor class names."""
-
-  global _ancestor_map
-  if _ancestor_map is None:
-    # Map from name to _NodeClassInfo.
-    node_classes = {i.name: i for i in _FindNodeClasses()}
-
-    # Update _NodeClassInfo.outgoing based on preconditions.
-    for info in node_classes.values():
-      for allowed in info.cls._CHECKER.allowed_types():  # pylint: disable=protected-access
-        if isinstance(allowed, type):
-          # All subclasses of the type are allowed.
-          info.outgoing.update(
-              [i for i in node_classes.values() if issubclass(i.cls, allowed)])
-        elif allowed in node_classes:
-          info.outgoing.add(node_classes[allowed])
-        elif allowed not in _IGNORED_TYPENAMES:
-          # This means preconditions list a typename that is unknown.  If it
-          # is a node then make sure _FindNodeClasses() can discover it.  If it
-          # is not a node, then add the typename to _IGNORED_TYPENAMES.
-          raise AssertionError("Unknown precondition typename: %s" % allowed)
-
-    predecessors = cfg_utils.compute_predecessors(node_classes.values())
-    # Convert predecessors keys and values to use names instead of info objects.
-    get_names = lambda v: {n.name for n in v}
-    _ancestor_map = {k.name: get_names(v) for k, v in predecessors.items()}
-  return _ancestor_map
-
-
-class Visitor(object):
-  """Base class for visitors.
-
-  Each class inheriting from visitor SHOULD have a fixed set of methods,
-  otherwise it might break the caching in this class.
-
-  Attributes:
-    visits_all_node_types: Whether the visitor can visit every node type.
-    unchecked_node_names: Contains the names of node classes that are unchecked
-      when constructing a new node from visited children.  This is useful
-      if a visitor returns data in part or all of its walk that would violate
-      node preconditions.
-    enter_functions: A dictionary mapping node class names to the
-      corresponding Enter functions.
-    visit_functions: A dictionary mapping node class names to the
-      corresponding Visit functions.
-    leave_functions: A dictionary mapping node class names to the
-      corresponding Leave functions.
-    visit_class_names: A set of node class names that must be visited.  This is
-      constructed based on the enter/visit/leave functions and precondition
-      data about legal ASTs.  As an optimization, the visitor will only visit
-      nodes under which some actionable node can appear.
-  """
-  visits_all_node_types = False
-  unchecked_node_names = set()
-
-  _visitor_functions_cache = {}
-
-  def __init__(self):
-    cls = self.__class__
-
-    # The set of method names for each visitor implementation is assumed to
-    # be fixed. Therefore this introspection can be cached.
-    if cls in Visitor._visitor_functions_cache:
-      enter_fns, visit_fns, leave_fns, visit_class_names = (
-          Visitor._visitor_functions_cache[cls])
-    else:
-      enter_fns = {}
-      enter_prefix = "Enter"
-      enter_len = len(enter_prefix)
-
-      visit_fns = {}
-      visit_prefix = "Visit"
-      visit_len = len(visit_prefix)
-
-      leave_fns = {}
-      leave_prefix = "Leave"
-      leave_len = len(leave_prefix)
-
-      for attr in dir(cls):
-        if attr.startswith(enter_prefix):
-          enter_fns[attr[enter_len:]] = getattr(cls, attr)
-        elif attr.startswith(visit_prefix):
-          visit_fns[attr[visit_len:]] = getattr(cls, attr)
-        elif attr.startswith(leave_prefix):
-          leave_fns[attr[leave_len:]] = getattr(cls, attr)
-
-      ancestors = _GetAncestorMap()
-      visit_class_names = set()
-      # A custom Enter/Visit/Leave requires visiting all types of nodes.
-      visit_all = (cls.Enter != Visitor.Enter or
-                   cls.Visit != Visitor.Visit or
-                   cls.Leave != Visitor.Leave)
-      for node in set(enter_fns) | set(visit_fns) | set(leave_fns):
-        if node in ancestors:
-          visit_class_names.update(ancestors[node])
-        elif node:
-          # Visiting an unknown non-empty node means the visitor has defined
-          # behavior on nodes that are unknown to the ancestors list.  To be
-          # safe, visit everything.
-          #
-          # TODO(dbaum): Consider making this an error.  The only wrinkle is
-          # that StrictType is unknown to _FindNodeClasses(), does not appear
-          # in any preconditions, but has defined behavior in PrintVisitor.
-          visit_all = True
-      if visit_all:
-        visit_class_names = ALL_NODE_NAMES
-      Visitor._visitor_functions_cache[cls] = (
-          enter_fns, visit_fns, leave_fns, visit_class_names)
-
-    self.enter_functions = enter_fns
-    self.visit_functions = visit_fns
-    self.leave_functions = leave_fns
-    self.visit_class_names = visit_class_names
-
-  def Enter(self, node, *args, **kwargs):
-    return self.enter_functions[node.__class__.__name__](
-        self, node, *args, **kwargs)
-
-  def Visit(self, node, *args, **kwargs):
-    return self.visit_functions[node.__class__.__name__](
-        self, node, *args, **kwargs)
-
-  def Leave(self, node, *args, **kwargs):
-    self.leave_functions[node.__class__.__name__](self, node, *args, **kwargs)
-
-
-class CanonicalOrderingVisitor(Visitor):
+class CanonicalOrderingVisitor(base_visitor.Visitor):
   """Visitor for converting ASTs back to canonical (sorted) ordering."""
 
   def __init__(self, sort_signatures=False):
-    super(CanonicalOrderingVisitor, self).__init__()
+    super().__init__()
     self.sort_signatures = sort_signatures
 
   def VisitTypeDeclUnit(self, node):
@@ -209,6 +37,7 @@ class CanonicalOrderingVisitor(Visitor):
         parents=node.parents,
         methods=tuple(sorted(node.methods)),
         constants=tuple(sorted(node.constants)),
+        decorators=tuple(sorted(node.decorators)),
         classes=tuple(sorted(node.classes)),
         slots=tuple(sorted(node.slots)) if node.slots is not None else None,
         template=node.template)
@@ -231,25 +60,28 @@ class CanonicalOrderingVisitor(Visitor):
     return pytd.UnionType(tuple(sorted(node.type_list)))
 
 
-class ClassTypeToNamedType(Visitor):
+class ClassTypeToNamedType(base_visitor.Visitor):
   """Change all ClassType objects to NameType objects."""
 
   def VisitClassType(self, node):
     return pytd.NamedType(node.name)
 
 
-class CollectTypeParameters(Visitor):
+class CollectTypeParameters(base_visitor.Visitor):
   """Visitor that accumulates type parameters in its "params" attribute."""
 
   def __init__(self):
-    super(CollectTypeParameters, self).__init__()
-    self.params = set()
+    super().__init__()
+    self._seen = set()
+    self.params = []
 
   def EnterTypeParameter(self, p):
-    self.params.add(p)
+    if p.name not in self._seen:
+      self.params.append(p)
+      self._seen.add(p.name)
 
 
-class ExtractSuperClasses(Visitor):
+class ExtractSuperClasses(base_visitor.Visitor):
   """Visitor for extracting all superclasses (i.e., the class hierarchy).
 
   When called on a TypeDeclUnit, this yields a dictionary mapping pytd.Class
@@ -257,10 +89,11 @@ class ExtractSuperClasses(Visitor):
   """
 
   def __init__(self):
-    super(ExtractSuperClasses, self).__init__()
+    super().__init__()
     self._superclasses = {}
 
   def _Key(self, node):
+    # This method should be implemented by subclasses.
     return node
 
   def VisitTypeDeclUnit(self, module):
@@ -273,62 +106,35 @@ class ExtractSuperClasses(Visitor):
       parent = self._Key(p)
       if parent is not None:
         parents.append(parent)
-    # TODO(kramm): This uses the entire class node as a key, instead of just
-    # its id.
     self._superclasses[self._Key(cls)] = parents
 
 
-class PrintVisitor(Visitor):
+class PrintVisitor(base_visitor.Visitor):
   """Visitor for converting ASTs back to pytd source code."""
   visits_all_node_types = True
-  unchecked_node_names = ALL_NODE_NAMES
+  unchecked_node_names = base_visitor.ALL_NODE_NAMES
 
   INDENT = " " * 4
   _RESERVED = frozenset(parser_constants.RESERVED +
                         parser_constants.RESERVED_PYTHON)
 
   def __init__(self, multiline_args=False):
-    super(PrintVisitor, self).__init__()
+    super().__init__()
     self.class_names = []  # allow nested classes
     self.imports = collections.defaultdict(set)
     self.in_alias = False
     self.in_parameter = False
+    self.in_literal = False
     self._unit_name = None
     self._local_names = set()
     self._class_members = set()
     self._typing_import_counts = collections.defaultdict(int)
     self.multiline_args = multiline_args
 
-  def _EscapedName(self, name):
-    """Name, possibly escaped with backticks.
-
-    If a name is a reserved PyTD token or contains special characters, it is
-    enclosed in backticks.  See parser.Pylexer.t_NAME for legal names that
-    require backticks.
-
-    Args:
-      name: A name, typically an identifier in the PyTD.
-
-    Returns:
-      The escaped name, or the original name if it doesn't need escaping.
-    """
-    if parser_constants.BACKTICK_NAME.search(name) or name in self._RESERVED:
-      # We can do this because name will never contain backticks. Everything
-      # we process here came in through the pytd parser, and the pytd syntax
-      # doesn't allow escaping backticks themselves.
-      return r"`" + name + "`"
-    else:
-      return name
-
-  def _SafeName(self, name):
-    split_name = name.split(".")
-    split_result = (self._EscapedName(piece) for piece in split_name)
-    return ".".join(split_result)
-
   def _IsEmptyTuple(self, t):
     """Check if it is an empty tuple."""
     assert isinstance(t, pytd.GenericType)
-    return t.base_type == "tuple" and t.parameters == ("nothing",)
+    return isinstance(t, pytd.TupleType) and not t.parameters
 
   def _NeedsTupleEllipsis(self, t):
     """Do we need to use Tuple[x, ...] instead of Tuple[x]?"""
@@ -381,7 +187,7 @@ class PrintVisitor(Visitor):
     return ret
 
   def _IsBuiltin(self, module):
-    return module == "__builtin__"
+    return module == "builtins"
 
   def _FormatTypeParams(self, type_params):
     formatted_type_params = []
@@ -433,7 +239,13 @@ class PrintVisitor(Visitor):
 
   def VisitConstant(self, node):
     """Convert a class-level or module-level constant to a string."""
-    return self._SafeName(node.name) + ": " + node.type
+    if self.in_literal:
+      module, _, name = node.name.partition(".")
+      assert module == "builtins", module
+      assert name in ("True", "False"), name
+      return name
+    else:
+      return node.name + ": " + node.type
 
   def EnterAlias(self, _):
     self.old_imports = self.imports.copy()
@@ -451,11 +263,11 @@ class PrintVisitor(Visitor):
         return "from " + module + " import " + name + suffix
     elif isinstance(self.old_node.type, pytd.Module):
       return node.type
-    return self._SafeName(node.name) + " = " + node.type
+    return node.name + " = " + node.type
 
   def EnterClass(self, node):
     """Entering a class - record class name for children's use."""
-    n = self._SafeName(node.name)
+    n = node.name
     if node.template:
       n += "[{}]".format(
           ", ".join(t.Visit(PrintVisitor()) for t in node.template))
@@ -476,12 +288,13 @@ class PrintVisitor(Visitor):
     if node.metaclass is not None:
       parents += ("metaclass=" + node.metaclass,)
     parents_str = "(" + ", ".join(parents) + ")" if parents else ""
-    header = ["class " + self._SafeName(node.name) + parents_str + ":"]
+    header = ["class " + node.name + parents_str + ":"]
     if node.slots is not None:
       slots_str = ", ".join("\"%s\"" % s for s in node.slots)
       slots = [self.INDENT + "__slots__ = [" + slots_str + "]"]
     else:
       slots = []
+    decorators = ["@" + d for d in node.decorators]
     if node.classes or node.methods or node.constants or slots:
       # We have multiple methods, and every method has multiple signatures
       # (i.e., the method string will have multiple lines). Combine this into
@@ -496,15 +309,16 @@ class PrintVisitor(Visitor):
       constants = []
       classes = []
       methods = []
-    return "\n".join(header + slots + classes + constants + methods) + "\n"
+    lines = decorators + header + slots + classes + constants + methods
+    return "\n".join(lines) + "\n"
 
   def VisitFunction(self, node):
     """Visit function, producing multi-line string (one for each signature)."""
-    function_name = self._EscapedName(node.name)
+    function_name = node.name
     decorators = ""
     if node.kind == pytd.STATICMETHOD and function_name != "__new__":
       decorators += "@staticmethod\n"
-    elif node.kind == pytd.CLASSMETHOD:
+    elif node.kind == pytd.CLASSMETHOD and function_name != "__init_subclass__":
       decorators += "@classmethod\n"
     elif node.kind == pytd.PROPERTY:
       decorators += "@property\n"
@@ -611,16 +425,16 @@ class PrintVisitor(Visitor):
     # Its name is `ClsName` before `[`.
     elif node.name == "self" and self.class_names and (
         self.class_names[-1].split("[")[0] == node.type):
-      return self._SafeName(node.name) + suffix
+      return node.name + suffix
     elif node.name == "cls" and self.class_names and (
         node.type == "Type[%s]" % self.class_names[-1]):
       self._typing_import_counts["Type"] -= 1
-      return self._SafeName(node.name) + suffix
+      return node.name + suffix
     elif node.type is None:
       logging.warning("node.type is None")
-      return self._SafeName(node.name)
+      return node.name
     else:
-      return self._SafeName(node.name) + ": " + node.type + suffix
+      return node.name + ": " + node.type + suffix
 
   def VisitTemplateItem(self, node):
     """Convert a template to a string."""
@@ -651,7 +465,7 @@ class PrintVisitor(Visitor):
       # PEP 484 allows this special abbreviation.
       return "None"
     else:
-      return self._SafeName(node_name)
+      return node_name
 
   def VisitLateType(self, node):
     return self.VisitNamedType(node)
@@ -677,7 +491,7 @@ class PrintVisitor(Visitor):
     return "nothing"
 
   def VisitTypeParameter(self, node):
-    return self._SafeName(node.name)
+    return node.name
 
   def VisitModule(self, node):
     if node.is_aliased:
@@ -687,8 +501,6 @@ class PrintVisitor(Visitor):
 
   def MaybeCapitalize(self, name):
     """Capitalize a generic type, if necessary."""
-    # Import here due to circular import.
-    from pytype.pytd import pep484  # pylint: disable=g-import-not-at-top
     capitalized = pep484.PEP484_MaybeCapitalize(name)
     if capitalized:
       return self._FromTyping(capitalized)
@@ -730,8 +542,6 @@ class PrintVisitor(Visitor):
     if self.in_parameter:
       # Parameter's set types are merged after as a follow up to the
       # ExpandCompatibleBuiltins visitor.
-      # Import here due to circular import.
-      from pytype.pytd import pep484  # pylint: disable=g-import-not-at-top
       for compat, name in pep484.COMPAT_ITEMS:
         # name can replace compat.
         if compat in type_list and name in type_list:
@@ -748,14 +558,25 @@ class PrintVisitor(Visitor):
       A string representing the union of the types in type_list. Simplifies
       Union[X] to X and Union[X, None] to Optional[X].
     """
-    type_list = tuple(type_list)
-    if len(type_list) == 1:
-      return type_list[0]
-    elif "None" in type_list:
+    # Collect all literals, so we can print them using the Literal[x1, ..., xn]
+    # syntactic sugar.
+    literals = []
+    new_type_list = []
+    for t in type_list:
+      match = re.fullmatch(r"Literal\[(?P<content>.*)\]", t)
+      if match:
+        literals.append(match.group("content"))
+      else:
+        new_type_list.append(t)
+    if literals:
+      new_type_list.append("Literal[%s]" % ", ".join(literals))
+    if len(new_type_list) == 1:
+      return new_type_list[0]
+    elif "None" in new_type_list:
       return (self._FromTyping("Optional") + "[" +
-              self._BuildUnion(t for t in type_list if t != "None") + "]")
+              self._BuildUnion(t for t in new_type_list if t != "None") + "]")
     else:
-      return self._FromTyping("Union") + "[" + ", ".join(type_list) + "]"
+      return self._FromTyping("Union") + "[" + ", ".join(new_type_list) + "]"
 
   def _BuildIntersection(self, type_list):
     """Builds a intersection of the types in type_list.
@@ -773,6 +594,14 @@ class PrintVisitor(Visitor):
     else:
       return " and ".join(type_list)
 
+  def EnterLiteral(self, _):
+    assert not self.in_literal
+    self.in_literal = True
+
+  def LeaveLiteral(self, _):
+    assert self.in_literal
+    self.in_literal = False
+
   def VisitLiteral(self, node):
     base = "Literal"
     # Check whether Literal is already imported from typing_extensions.
@@ -781,7 +610,7 @@ class PrintVisitor(Visitor):
     return "%s[%s]" % (base, node.value)
 
 
-class RenameModuleVisitor(Visitor):
+class RenameModuleVisitor(base_visitor.Visitor):
   """Renames a TypeDeclUnit."""
 
   def __init__(self, old_module_name, new_module_name):
@@ -796,7 +625,7 @@ class RenameModuleVisitor(Visitor):
     Raises:
       ValueError: If the old_module name is an empty string.
     """
-    super(RenameModuleVisitor, self).__init__()
+    super().__init__()
     if not old_module_name:
       raise ValueError("old_module_name must be a non empty string.")
     assert not old_module_name.endswith(".")
