@@ -1,19 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-
 """Match pytd types against each other.
 
 "Matching" x against y means roughly: If we have a function f(param: y) and
@@ -30,6 +14,7 @@ import logging
 
 from pytype import utils
 from pytype.pytd import booleq
+from pytype.pytd import escape
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
@@ -38,44 +23,20 @@ from pytype.pytd.parse import node
 log = logging.getLogger(__name__)
 
 
+is_complete = escape.is_complete
+
+
 # Might not be needed anymore once pytd has builtin support for ~unknown.
 def is_unknown(t):
   """Return True if this is an ~unknown."""
   if isinstance(t, (pytd.ClassType, pytd.NamedType, pytd.Class, StrictType)):
-    return t.name.startswith("~unknown")
+    return escape.is_unknown(t.name)
   elif isinstance(t, str):
-    return t.startswith("~unknown")
+    return escape.is_unknown(t)
   else:
     return False
 
 
-# Might not be needed anymore once pytd has Interface types.
-def is_complete(cls):
-  """Return True if this class is complete."""
-  # Incomplete classes are marked with "~". E.g. "class ~int".
-  if isinstance(cls, str):
-    return not cls.startswith("~")
-  else:
-    return not cls.name.startswith("~")
-
-
-def is_partial(cls):
-  """Returns True if this is a partial class, e.g. "~list"."""
-  if isinstance(cls, str):
-    return cls.startswith("~")
-  elif hasattr(cls, "name"):
-    return cls.name.startswith("~")
-  else:
-    return False
-
-
-def unpack_name_of_partial(name):
-  """Convert e.g. "~int" to "int"."""
-  assert isinstance(name, str)
-  assert name.startswith("~")
-  return name.lstrip("~").replace("~", ".")
-
-
 def get_all_subclasses(asts):
   """Compute a class->subclasses mapping.
 
@@ -145,12 +106,12 @@ class TypeMatch(pytd_utils.TypeMatcher):
       return sum((self.get_superclasses(c) for c in t.cls.parents), [t])
     elif isinstance(t, pytd.AnythingType):
       # All types, even "?", inherit from object.
-      return [pytd.NamedType("__builtin__.object")]
+      return [pytd.NamedType("builtins.object")]
     elif isinstance(t, pytd.GenericType):
       return self.get_superclasses(t.base_type)
     else:
       log.warning("Can't extract superclasses from %s", type(t))
-      return [pytd.NamedType("__builtin__.object")]
+      return [pytd.NamedType("builtins.object")]
 
   def get_subclasses(self, t):
     """Get all classes derived from this type.
@@ -205,18 +166,18 @@ class TypeMatch(pytd_utils.TypeMatcher):
       # compatible base types, t1 is a homogeneous tuple here.
       return (t1.element_type,) * len(t2.parameters), t2.parameters
     elif isinstance(t1, pytd.TupleType):
-      return (pytd.UnionType(type_list=t1.parameters),), t2.parameters
+      return (pytd_utils.JoinTypes(t1.parameters),), t2.parameters
     elif (isinstance(t1, pytd.CallableType) and
           isinstance(t2, pytd.CallableType)):
       # Flip the arguments, since argument types are contravariant.
       return t2.args + (t1.ret,), t1.args + (t2.ret,)
-    elif (t1.base_type.cls.name == "__builtin__.type" and
+    elif (t1.base_type.cls.name == "builtins.type" and
           t2.base_type.cls.name == "typing.Callable"):
       # We'll only check the return type, since getting the argument types for
       # initializing a class is tricky.
       return t1.parameters, (t2.parameters[-1],)
     elif (t1.base_type.cls.name == "typing.Callable" and
-          t2.base_type.cls.name == "__builtin__.type"):
+          t2.base_type.cls.name == "builtins.type"):
       return (t1.parameters[-1],), t2.parameters
     elif isinstance(t1, pytd.CallableType):
       # We're matching against GenericType(Callable, (Any, _RET)), so we don't
@@ -260,7 +221,7 @@ class TypeMatch(pytd_utils.TypeMatcher):
     for type_param in type_params:
       self.solver.register_variable(type_param.name)
     if isinstance(t2, pytd.TupleType):
-      t2_parameters = (pytd.UnionType(type_list=t2.parameters),)
+      t2_parameters = (pytd_utils.JoinTypes(t2.parameters),)
     else:
       t2_parameters = t2.parameters
     params = [self.match_type_against_type(p1, p2, subst)
@@ -269,7 +230,7 @@ class TypeMatch(pytd_utils.TypeMatcher):
 
   def match_Generic_against_Unknown(self, t1, t2, subst):  # pylint: disable=invalid-name
     # Note: This flips p1 and p2 above.
-    return self.match_Unknown_against_Generic(t2, t1, subst)
+    return self.match_Unknown_against_Generic(t2, t1, subst)  # pylint: disable=arguments-out-of-order
 
   def maybe_lookup_type_param(self, t, subst):
     while isinstance(t, pytd.TypeParameter):
@@ -287,7 +248,7 @@ class TypeMatch(pytd_utils.TypeMatcher):
   def unclass(self, t):
     """Prevent further subclass or superclass expansion for this type."""
     if isinstance(t, pytd.ClassType):
-      # When t.name and t.cls.name differ (e.g., int vs. __builtin__.int), the
+      # When t.name and t.cls.name differ (e.g., int vs. builtins.int), the
       # latter is the complete name.
       return pytd.NamedType(t.cls.name)
     else:
@@ -345,11 +306,11 @@ class TypeMatch(pytd_utils.TypeMatcher):
       # For strict types, avoid subclasses of the left side.
       return booleq.Eq(self._full_name(t1), self._full_name(t2))
     elif (isinstance(t1, pytd.ClassType) and hasattr(t2, "name") and
-          t2.name == "__builtin__.object"):
+          t2.name == "builtins.object"):
       return booleq.TRUE
     elif (hasattr(t1, "name") and hasattr(t2, "name") and
-          t1.name in ("__builtin__.type", "typing.Callable") and
-          t2.name in ("__builtin__.type", "typing.Callable")):
+          t1.name in ("builtins.type", "typing.Callable") and
+          t2.name in ("builtins.type", "typing.Callable")):
       return booleq.TRUE
     elif isinstance(t1, pytd.ClassType):
       # ClassTypes are similar to Unions, except they're disjunctions: We can
@@ -485,7 +446,7 @@ class TypeMatch(pytd_utils.TypeMatcher):
             values = tuple(pytd.AnythingType() for _ in cls.template)
           elif isinstance(base, pytd.TupleType):
             cls = base.base_type.cls
-            values = (pytd.UnionType(type_list=base.parameters),)
+            values = (pytd_utils.JoinTypes(base.parameters),)
           else:
             cls = base.base_type.cls
             values = base.parameters
