@@ -1,24 +1,10 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 """Tests for type_match.py."""
 
 import textwrap
 
 from pytype.pyi import parser
 from pytype.pytd import booleq
+from pytype.pytd import escape
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import type_match
@@ -34,13 +20,17 @@ _BUILTINS = """
 """
 
 
+def pytd_src(text):
+  return textwrap.dedent(escape.preprocess_pytd(text))
+
+
 class TestTypeMatch(parser_test_base.ParserTest):
   """Test algorithms and datastructures of booleq.py."""
 
   def setUp(self):
-    super(TestTypeMatch, self).setUp()
+    super().setUp()
     builtins = parser.parse_string(textwrap.dedent(_BUILTINS),
-                                   name="__builtin__",
+                                   name="builtins",
                                    python_version=self.python_version)
     typing = parser.parse_string("class Generic: ...", name="typing",
                                  python_version=self.python_version)
@@ -132,8 +122,8 @@ class TestTypeMatch(parser_test_base.ParserTest):
 
   def test_function(self):
     ast = parser.parse_string(textwrap.dedent("""
-      def left(a: int) -> int
-      def right(a: int) -> int
+      def left(a: int) -> int: ...
+      def right(a: int) -> int: ...
     """), python_version=self.python_version)
     m = type_match.TypeMatch()
     self.assertEqual(m.match(ast.Lookup("left"), ast.Lookup("right"), {}),
@@ -141,8 +131,8 @@ class TestTypeMatch(parser_test_base.ParserTest):
 
   def test_return(self):
     ast = parser.parse_string(textwrap.dedent("""
-      def left(a: int) -> float
-      def right(a: int) -> int
+      def left(a: int) -> float: ...
+      def right(a: int) -> int: ...
     """), python_version=self.python_version)
     m = type_match.TypeMatch()
     self.assertNotEqual(m.match(ast.Lookup("left"), ast.Lookup("right"), {}),
@@ -150,8 +140,8 @@ class TestTypeMatch(parser_test_base.ParserTest):
 
   def test_optional(self):
     ast = parser.parse_string(textwrap.dedent("""
-      def left(a: int) -> int
-      def right(a: int, ...) -> int
+      def left(a: int) -> int: ...
+      def right(a: int, *args) -> int: ...
     """), python_version=self.python_version)
     m = type_match.TypeMatch()
     self.assertEqual(m.match(ast.Lookup("left"), ast.Lookup("right"), {}),
@@ -159,11 +149,12 @@ class TestTypeMatch(parser_test_base.ParserTest):
 
   def test_generic(self):
     ast = parser.parse_string(textwrap.dedent("""
+      from typing import Any
       T = TypeVar('T')
       class A(typing.Generic[T], object):
         pass
-      left = ...  # type: A[?]
-      right = ...  # type: A[?]
+      left = ...  # type: A[Any]
+      right = ...  # type: A[Any]
     """), python_version=self.python_version)
     ast = self.LinkAgainstSimpleBuiltins(ast)
     m = type_match.TypeMatch()
@@ -173,11 +164,12 @@ class TestTypeMatch(parser_test_base.ParserTest):
 
   def test_class_match(self):
     ast = parser.parse_string(textwrap.dedent("""
+      from typing import Any
       class Left():
-        def method(self) -> ?
+        def method(self) -> Any: ...
       class Right():
-        def method(self) -> ?
-        def method2(self) -> ?
+        def method(self) -> Any: ...
+        def method2(self) -> Any: ...
     """), python_version=self.python_version)
     ast = visitors.LookupClasses(ast, self.mini_builtins)
     m = type_match.TypeMatch()
@@ -192,8 +184,8 @@ class TestTypeMatch(parser_test_base.ParserTest):
       class B(A):
         pass
       a = ...  # type: A
-      def left(a: B) -> B
-      def right(a: A) -> A
+      def left(a: B) -> B: ...
+      def right(a: A) -> A: ...
     """), python_version=self.python_version)
     ast = visitors.LookupClasses(ast, self.mini_builtins)
     m = type_match.TypeMatch(type_match.get_all_subclasses([ast]))
@@ -202,25 +194,26 @@ class TestTypeMatch(parser_test_base.ParserTest):
     self.assertNotEqual(m.match(right, left, {}), booleq.TRUE)
 
   def _TestTypeParameters(self, reverse=False):
-    ast = parser.parse_string(textwrap.dedent("""
-      import typing
+    ast = parser.parse_string(pytd_src("""
+      from typing import Any, Generic
       class `~unknown0`():
-        def next(self) -> ?
+        def next(self) -> Any: ...
       T = TypeVar('T')
-      class A(typing.Generic[T], object):
-        def next(self) -> ?
+      class A(Generic[T], object):
+        def next(self) -> Any: ...
       class B():
         pass
-      def left(x: `~unknown0`) -> ?
-      def right(x: A[B]) -> ?
+      def left(x: `~unknown0`) -> Any: ...
+      def right(x: A[B]) -> Any: ...
     """), python_version=self.python_version)
     ast = self.LinkAgainstSimpleBuiltins(ast)
     m = type_match.TypeMatch()
     left, right = ast.Lookup("left"), ast.Lookup("right")
     match = m.match(right, left, {}) if reverse else m.match(left, right, {})
-    self.assertEqual(match, booleq.And((booleq.Eq("~unknown0", "A"),
-                                        booleq.Eq("~unknown0.A.T", "B"))))
-    self.assertIn("~unknown0.A.T", m.solver.variables)
+    unknown0 = escape.unknown(0)
+    self.assertEqual(match, booleq.And((booleq.Eq(unknown0, "A"),
+                                        booleq.Eq(f"{unknown0}.A.T", "B"))))
+    self.assertIn(f"{unknown0}.A.T", m.solver.variables)
 
   def test_unknown_against_generic(self):
     self._TestTypeParameters()
@@ -229,7 +222,7 @@ class TestTypeMatch(parser_test_base.ParserTest):
     self._TestTypeParameters(reverse=True)
 
   def test_strict(self):
-    ast = parser.parse_string(textwrap.dedent("""
+    ast = parser.parse_string(pytd_src("""
       import typing
 
       T = TypeVar('T')
@@ -242,25 +235,26 @@ class TestTypeMatch(parser_test_base.ParserTest):
       class `~unknown0`():
         pass
       a = ...  # type: A
-      def left() -> `~unknown0`
-      def right() -> list[A]
+      def left() -> `~unknown0`: ...
+      def right() -> list[A]: ...
     """), python_version=self.python_version)
     ast = self.LinkAgainstSimpleBuiltins(ast)
     m = type_match.TypeMatch(type_match.get_all_subclasses([ast]))
     left, right = ast.Lookup("left"), ast.Lookup("right")
+    unknown0 = escape.unknown(0)
     self.assertEqual(m.match(left, right, {}),
-                     booleq.And((booleq.Eq("~unknown0", "list"),
-                                 booleq.Eq("~unknown0.list.T", "A"))))
+                     booleq.And((booleq.Eq(unknown0, "list"),
+                                 booleq.Eq(f"{unknown0}.list.T", "A"))))
 
   def test_base_class(self):
     ast = parser.parse_string(textwrap.dedent("""
       class Base():
-        def f(self, x:Base) -> Base
+        def f(self, x:Base) -> Base: ...
       class Foo(Base):
         pass
 
       class Match():
-        def f(self, x:Base) -> Base
+        def f(self, x:Base) -> Base: ...
     """), python_version=self.python_version)
     ast = self.LinkAgainstSimpleBuiltins(ast)
     m = type_match.TypeMatch(type_match.get_all_subclasses([ast]))
@@ -321,20 +315,21 @@ class TestTypeMatch(parser_test_base.ParserTest):
     self.assertEqual(m.match_Generic_against_Generic(y1, x2, {}), booleq.TRUE)
 
   def test_unknown_against_tuple(self):
-    ast = self.ParseWithBuiltins("""
+    ast = self.ParseWithBuiltins(pytd_src("""
       from typing import Tuple
       class `~unknown0`():
         pass
       x = ...  # type: Tuple[int, str]
-    """)
-    unk = ast.Lookup("~unknown0")
+    """))
+    unknown0 = escape.unknown(0)
+    unk = ast.Lookup(unknown0)
     tup = ast.Lookup("x").type
     m = type_match.TypeMatch(type_match.get_all_subclasses([ast]))
     match = m.match_Unknown_against_Generic(unk, tup, {})
-    self.assertListEqual(sorted(match.extract_equalities()),
-                         [("~unknown0", "__builtin__.tuple"),
-                          ("~unknown0.__builtin__.tuple._T", "int"),
-                          ("~unknown0.__builtin__.tuple._T", "str")])
+    self.assertCountEqual(sorted(match.extract_equalities()),
+                          [(unknown0, "builtins.tuple"),
+                           (f"{unknown0}.builtins.tuple._T", "int"),
+                           (f"{unknown0}.builtins.tuple._T", "str")])
 
   def test_function_against_tuple_subclass(self):
     ast = self.ParseWithBuiltins("""
