@@ -6,7 +6,7 @@ from pytype import module_utils
 from pytype import pytype_source_utils
 from pytype import utils
 from pytype.pyi import parser
-from pytype.pytd.parse import builtins
+from pytype.pytd import builtins
 
 
 def _get_module_names_in_path(lister, path):
@@ -21,7 +21,7 @@ def _get_module_names_in_path(lister, path):
   return names
 
 
-class Typeshed(object):
+class Typeshed:
   """A typeshed installation.
 
   The location is either retrieved from the environment variable
@@ -50,18 +50,17 @@ class Typeshed(object):
   def _load_file(self, path):
     if self._env_home:
       filename = os.path.join(self._env_home, path)
-      with open(filename, "rb") as f:
+      with open(filename, "r") as f:
         return filename, f.read()
     else:
       filepath = os.path.join(self._root, path)
-      data = pytype_source_utils.load_pytype_file(filepath)
-      return filepath, data
+      return filepath, pytype_source_utils.load_text_file(filepath)
 
   def _load_missing(self):
     if not self.MISSING_FILE:
       return set()
-    _, data = self._load_file(self.MISSING_FILE)
-    return {line.decode("utf-8").strip() for line in data.split(b"\n") if line}
+    _, text = self._load_file(self.MISSING_FILE)
+    return {line.strip() for line in text.split("\n") if line}
 
   @property
   def missing(self):
@@ -146,8 +145,8 @@ class Typeshed(object):
     """Gets the paths to pytype's version-specific pytd files."""
     # TODO(mdemello): Should we add 2and3 here too and stop symlinking?
     return [pytype_source_utils.get_full_path(d) for d in [
-        "pytd/builtins/%d" % python_version[0],
-        "pytd/stdlib/%d" % python_version[0]]]
+        "stubs/builtins/%d" % python_version[0],
+        "stubs/stdlib/%d" % python_version[0]]]
 
   def get_all_module_names(self, python_version):
     """Get the names of all modules in typeshed or bundled with pytype."""
@@ -176,10 +175,8 @@ class Typeshed(object):
 
   def read_blacklist(self):
     """Read the typeshed blacklist."""
-    _, data = self._load_file(os.path.join("tests", "pytype_blacklist.txt"))
-    # |data| is raw byte data.
-    for line in data.splitlines():
-      line = line.decode("utf-8")
+    _, text = self._load_file(os.path.join("tests", "pytype_exclude_list.txt"))
+    for line in text.splitlines():
       line = line[:line.find("#")].strip()
       if line:
         yield line
