@@ -1,19 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 """Visitor(s) for walking ASTs."""
 
 # Because pytype takes too long:
@@ -27,14 +11,15 @@ import re
 from pytype import datatypes
 from pytype import module_utils
 from pytype import utils
+from pytype.pytd import base_visitor
+from pytype.pytd import escape
 from pytype.pytd import mro
+from pytype.pytd import pep484
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import pytd_visitors
 from pytype.pytd.parse import parser_constants  # pylint: disable=g-importing-member
 
-import six
-
 
 class ContainerError(Exception):
   pass
@@ -46,8 +31,8 @@ class SymbolLookupError(Exception):
 
 # All public elements of pytd_visitors are aliased here so that we can maintain
 # the conceptually simpler illusion of having a single visitors module.
-ALL_NODE_NAMES = pytd_visitors.ALL_NODE_NAMES
-Visitor = pytd_visitors.Visitor
+ALL_NODE_NAMES = base_visitor.ALL_NODE_NAMES
+Visitor = base_visitor.Visitor
 CanonicalOrderingVisitor = pytd_visitors.CanonicalOrderingVisitor
 ClassTypeToNamedType = pytd_visitors.ClassTypeToNamedType
 CollectTypeParameters = pytd_visitors.CollectTypeParameters
@@ -107,7 +92,7 @@ class FillInLocalPointers(Visitor):
         "Lookup" function).
       fallback: A symbol table to be tried if lookup otherwise fails.
     """
-    super(FillInLocalPointers, self).__init__()
+    super().__init__()
     if fallback is not None:
       lookup_map["*"] = fallback
     self._lookup_map = lookup_map
@@ -119,9 +104,9 @@ class FillInLocalPointers(Visitor):
       modules_to_try = [("", module)]
     else:
       modules_to_try = [("", ""),
-                        ("", "__builtin__"),
-                        ("__builtin__.", "__builtin__")]
-    modules_to_try += [("", "*"), ("__builtin__.", "*")]
+                        ("", "builtins"),
+                        ("builtins.", "builtins")]
+    modules_to_try += [("", "*"), ("builtins.", "*")]
     for prefix, module in modules_to_try:
       mod_ast = self._lookup_map.get(module)
       if mod_ast:
@@ -187,7 +172,7 @@ class DefaceUnresolved(RemoveTypeParametersFromGenericAny):
       do_not_log_prefix: If given, don't log error messages for classes with
         this prefix.
     """
-    super(DefaceUnresolved, self).__init__()
+    super().__init__()
     self._lookup_list = lookup_list
     self._do_not_log_prefix = do_not_log_prefix
 
@@ -206,7 +191,7 @@ class DefaceUnresolved(RemoveTypeParametersFromGenericAny):
     else:
       if (self._do_not_log_prefix is None or
           not name.startswith(self._do_not_log_prefix)):
-        logging.warning("Setting %s to ?", name)
+        logging.warning("Setting %s to Any", name)
       return pytd.AnythingType()
 
   def VisitCallableType(self, node):
@@ -236,25 +221,27 @@ class NamedTypeToClassType(Visitor):
 
 
 class DropBuiltinPrefix(Visitor):
-  """Drop '__builtin__.' prefix."""
+  """Drop 'builtins.' prefix."""
 
   def VisitClassType(self, node):
-    _, _, name = node.name.rpartition("__builtin__.")
+    _, _, name = node.name.rpartition("builtins.")
     return pytd.NamedType(name)
 
   def VisitNamedType(self, node):
     return self.VisitClassType(node)
 
 
-# TODO(b/110164593): Get rid of this hack.
+# TODO(rechen): Get rid of this hack once we drop Python 2 support and all
+# typeshed references to __builtin__ are gone.
 def RenameBuiltinsPrefixInName(name):
-  if name.startswith("builtins."):
-    name = "__builtin__." + name[len("builtins."):]
+  module, _, basename = name.partition(".")
+  if module == "__builtin__":
+    return f"builtins.{basename}" if basename else "builtins"
   return name
 
 
 class RenameBuiltinsPrefix(Visitor):
-  """Rename 'builtins' to '__builtin__' at import time."""
+  """Rename '__builtin__' to 'builtins' at import time."""
 
   def VisitClassType(self, node):
     return pytd.NamedType(RenameBuiltinsPrefixInName(node.name))
@@ -295,7 +282,7 @@ class VerifyLookup(Visitor):
   """Utility class for testing visitors.LookupClasses."""
 
   def __init__(self, ignore_late_types=False):
-    super(VerifyLookup, self).__init__()
+    super().__init__()
     self.ignore_late_types = ignore_late_types
 
   def EnterLateType(self, node):
@@ -320,7 +307,7 @@ class LookupBuiltins(Visitor):
       builtins: The builtins module.
       full_names: Whether to use fully qualified names for lookup.
     """
-    super(LookupBuiltins, self).__init__()
+    super().__init__()
     self._builtins = builtins
     self._full_names = full_names
 
@@ -383,7 +370,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny):
         names. If the source contains "import X as Y", module_alias_map should
         contain an entry mapping "Y": "X".
     """
-    super(LookupExternalTypes, self).__init__()
+    super().__init__()
     self._module_map = module_map
     self._module_alias_map = module_alias_map or {}
     self.name = self_name
@@ -455,7 +442,8 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny):
     if t.name in self._module_map:
       if self._alias_name and "." in self._alias_name:
         # Module aliases appear only in asts that use fully-qualified names.
-        return pytd.ToType(pytd.Module(name=t.name, module_name=t.name))
+        return pytd.ToType(
+            pytd.Module(name=self._alias_name, module_name=t.name))
       else:
         # We have a class with the same name as a module.
         return t
@@ -474,10 +462,10 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny):
         item = t  # VisitTypeDeclUnit will remove this unneeded item.
       else:
         item = self._LookupItemRecursive(module, module_name, name)
-    except KeyError:
+    except KeyError as e:
       item = self._ResolveUsingGetattr(module_name, module)
       if item is None:
-        raise KeyError("No %s in module %s" % (name, module_name))
+        raise KeyError("No %s in module %s" % (name, module_name)) from e
     if not self._in_generic_type and isinstance(item, pytd.Alias):
       # If `item` contains type parameters and is not inside a GenericType, then
       # we replace the parameters with Any.
@@ -497,7 +485,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny):
       try:
         t = MaybeSubstituteParameters(t.base_type, t.parameters) or t
       except ValueError as e:
-        raise KeyError(str(e))
+        raise KeyError(str(e)) from e
     return t
 
   def _ModulePrefix(self):
@@ -628,9 +616,9 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny):
         # Happens for infer calling load_pytd.resolve_ast() for the final pyi
         try:
           item = self.unit.Lookup(node.name)
-        except KeyError:
+        except KeyError as e:
           raise SymbolLookupError("Couldn't find %s in %s" % (
-              node.name, self.unit.name))
+              node.name, self.unit.name)) from e
       return pytd.ToType(item, allow_constants=False)
     elif module_name == self.unit.name:
       return pytd.ToType(self.unit.Lookup(node.name), allow_constants=False)
@@ -655,7 +643,7 @@ class ReplaceTypes(Visitor):
       record: Optional. A set. If given, this records which entries in
         the map were used.
     """
-    super(ReplaceTypes, self).__init__()
+    super().__init__()
     self.mapping = mapping
     self.record = record
 
@@ -696,7 +684,7 @@ class ReplaceTypeParameters(Visitor):
   """Visitor for replacing type parameters with actual types."""
 
   def __init__(self, mapping):
-    super(ReplaceTypeParameters, self).__init__()
+    super().__init__()
     self.mapping = mapping
 
   def VisitTypeParameter(self, p):
@@ -727,7 +715,7 @@ class AdjustSelf(Visitor):
   """
 
   def __init__(self, force=False):
-    super(AdjustSelf, self).__init__()
+    super().__init__()
     self.class_types = []  # allow nested classes
     self.force = force
 
@@ -744,7 +732,7 @@ class AdjustSelf(Visitor):
     """Adjust all parameters called "self" to have their parent class type.
 
     But do this only if their original type is unoccupied ("object" or,
-    if configured, "?").
+    if configured, "Any").
 
     Args:
       p: pytd.Parameter instance.
@@ -772,11 +760,11 @@ class RemoveUnknownClasses(Visitor):
     class ~unknown2:
       ...
   to
-    def f(x) -> ?
+    def f(x) -> Any
   """
 
   def __init__(self):
-    super(RemoveUnknownClasses, self).__init__()
+    super().__init__()
     self.parameter = None
 
   def EnterParameter(self, p):
@@ -787,32 +775,32 @@ class RemoveUnknownClasses(Visitor):
     self.parameter = None
 
   def VisitClassType(self, t):
-    if t.name.startswith("~unknown"):
+    if escape.is_unknown(t.name):
       return pytd.AnythingType()
     else:
       return t
 
   def VisitNamedType(self, t):
-    if t.name.startswith("~unknown"):
+    if escape.is_unknown(t.name):
       return pytd.AnythingType()
     else:
       return t
 
   def VisitTypeDeclUnit(self, u):
     return u.Replace(classes=tuple(
-        cls for cls in u.classes if not cls.name.startswith("~unknown")))
+        cls for cls in u.classes if not escape.is_unknown(cls.name)))
 
 
 class _CountUnknowns(Visitor):
   """Visitor for counting how often given unknowns occur in a type."""
 
   def __init__(self):
-    super(_CountUnknowns, self).__init__()
+    super().__init__()
     self.counter = collections.Counter()
     self.position = {}
 
   def EnterNamedType(self, t):
-    _, is_unknown, suffix = t.name.partition("~unknown")
+    _, is_unknown, suffix = t.name.partition(escape.UNKNOWN)
     if is_unknown:
       if suffix not in self.counter:
         # Also record the order in which we see the ~unknowns
@@ -856,14 +844,11 @@ class CreateTypeParametersForSignatures(Visitor):
   PREFIX = "_T"  # Prefix for new type params
 
   def __init__(self):
-    super(CreateTypeParametersForSignatures, self).__init__()
+    super().__init__()
     self.parameter = None
     self.class_name = None
     self.function_name = None
 
-  def _IsIncomplete(self, name):
-    return name and name.startswith("~")
-
   def EnterClass(self, node):
     self.class_name = node.name
 
@@ -885,7 +870,7 @@ class CreateTypeParametersForSignatures(Visitor):
       (self: X[, ...]) -> X
     so that we can replace X with a bounded TypeVar. This heuristic
     isn't perfect; for example, in this naive copy method:
-      class X(object):
+      class X:
         def copy(self):
           return X()
     we should have left X alone. But it prevents a number of false
@@ -908,8 +893,8 @@ class CreateTypeParametersForSignatures(Visitor):
 
   def VisitSignature(self, sig):
     """Potentially replace ~unknowns with type parameters, in a signature."""
-    if (self._IsIncomplete(self.class_name) or
-        self._IsIncomplete(self.function_name)):
+    if (escape.is_partial(self.class_name) or
+        escape.is_partial(self.function_name)):
       # Leave unknown classes and call traces as-is, they'll never be part of
       # the output.
       return sig
@@ -922,7 +907,8 @@ class CreateTypeParametersForSignatures(Visitor):
         # way, e.g. "def f(Dict[T, T])" works, too.
         type_param = pytd.TypeParameter(
             self.PREFIX + str(counter.position[suffix]))
-        replacements["~unknown"+suffix] = type_param
+        key = escape.UNKNOWN + suffix
+        replacements[key] = type_param
     if self._NeedsClassParam(sig):
       type_param = pytd.TypeParameter(
           self.PREFIX + self.class_name, bound=pytd.NamedType(self.class_name))
@@ -957,11 +943,11 @@ class RaiseIfContainsUnknown(Visitor):
   # COV_NF_END
 
   def EnterClassType(self, t):
-    if t.name.startswith("~unknown"):
+    if escape.is_unknown(t.name):
       raise RaiseIfContainsUnknown.HasUnknown()
 
   def EnterClass(self, cls):
-    if cls.name.startswith("~unknown"):
+    if escape.is_unknown(cls.name):
       raise RaiseIfContainsUnknown.HasUnknown()
 
 
@@ -969,11 +955,11 @@ class VerifyVisitor(Visitor):
   """Visitor for verifying pytd ASTs. For tests."""
 
   def __init__(self):
-    super(VerifyVisitor, self).__init__()
+    super().__init__()
     self._valid_param_name = re.compile(r"[a-zA-Z_]\w*$")
 
   def Enter(self, node):
-    super(VerifyVisitor, self).Enter(node)
+    super().Enter(node)
     node.Validate()
 
   def _AssertNoDuplicates(self, node, attrs):
@@ -1020,9 +1006,6 @@ class VerifyVisitor(Visitor):
   def EnterCallableType(self, node):
     self.EnterGenericType(node)
 
-  def EnterTupleType(self, node):
-    self.EnterGenericType(node)
-
   def EnterGenericType(self, node):
     assert node.parameters, node
 
@@ -1031,7 +1014,7 @@ class RemoveFunctionsAndClasses(Visitor):
   """Visitor for removing unwanted functions or classes."""
 
   def __init__(self, names):
-    super(RemoveFunctionsAndClasses, self).__init__()
+    super().__init__()
     self.names = names
 
   def VisitTypeDeclUnit(self, node):
@@ -1067,7 +1050,7 @@ class AddNamePrefix(Visitor):
   """
 
   def __init__(self):
-    super(AddNamePrefix, self).__init__()
+    super().__init__()
     self.cls_stack = []
     self.classes = None
     self.prefix = None
@@ -1158,7 +1141,7 @@ class CollectDependencies(Visitor):
   """
 
   def __init__(self):
-    super(CollectDependencies, self).__init__()
+    super().__init__()
     self.dependencies = {}
     self.late_dependencies = {}
 
@@ -1225,12 +1208,12 @@ class ExpandSignatures(Visitor):
   """Expand to Cartesian product of parameter types.
 
   For example, this transforms
-    def f(x: int or float, y: int or float) -> str or unicode
+    def f(x: Union[int, float], y: Union[int, float]) -> Union[str, unicode]
   to
-    def f(x: int, y: int) -> str or unicode
-    def f(x: int, y: float) -> str or unicode
-    def f(x: float, y: int) -> str or unicode
-    def f(x: float, y: float) -> str or unicode
+    def f(x: int, y: int) -> Union[str, unicode]
+    def f(x: int, y: float) -> Union[str, unicode]
+    def f(x: float, y: int) -> Union[str, unicode]
+    def f(x: float, y: float) -> Union[str, unicode]
 
   The expansion by this class is typically *not* an optimization. But it can be
   the precursor for optimizations that need the expanded signatures, and it can
@@ -1252,9 +1235,9 @@ class ExpandSignatures(Visitor):
       Function with the new signatures.
     """
 
-    # concatenate return value(s) from VisitSignature
-    signatures = sum([ExpandSignature(s) for s in f.signatures], [])
-    return f.Replace(signatures=tuple(signatures))
+    # flatten return value(s) from VisitSignature
+    signatures = tuple(ex for s in f.signatures for ex in ExpandSignature(s))  # pylint: disable=g-complex-comprehension
+    return f.Replace(signatures=signatures)
 
 
 class AdjustTypeParameters(Visitor):
@@ -1266,7 +1249,7 @@ class AdjustTypeParameters(Visitor):
   """
 
   def __init__(self):
-    super(AdjustTypeParameters, self).__init__()
+    super().__init__()
     self.class_typeparams = set()
     self.function_typeparams = None
     self.class_template = []
@@ -1274,6 +1257,7 @@ class AdjustTypeParameters(Visitor):
     self.function_name = None
     self.constant_name = None
     self.all_typeparams = set()
+    self.generic_level = 0
 
   def _GetTemplateItems(self, param):
     """Get a list of template items from a parameter."""
@@ -1341,9 +1325,9 @@ class AdjustTypeParameters(Visitor):
 
     try:
       template = mro.MergeSequences(templates)
-    except ValueError:
+    except ValueError as e:
       raise ContainerError(
-          "Illegal type parameter order in class %s" % node.name)
+          "Illegal type parameter order in class %s" % node.name) from e
 
     self.class_template.append(template)
 
@@ -1396,6 +1380,30 @@ class AdjustTypeParameters(Visitor):
   def LeaveConstant(self, unused_node):
     self.constant_name = None
 
+  def EnterGenericType(self, unused_node):
+    self.generic_level += 1
+
+  def LeaveGenericType(self, unused_node):
+    self.generic_level -= 1
+
+  def EnterCallableType(self, node):
+    self.EnterGenericType(node)
+
+  def LeaveCallableType(self, node):
+    self.LeaveGenericType(node)
+
+  def EnterTupleType(self, node):
+    self.EnterGenericType(node)
+
+  def LeaveTupleType(self, node):
+    self.LeaveGenericType(node)
+
+  def EnterUnionType(self, node):
+    self.EnterGenericType(node)
+
+  def LeaveUnionType(self, node):
+    self.LeaveGenericType(node)
+
   def _GetFullName(self, name):
     return ".".join(n for n in [self.class_name, name] if n)
 
@@ -1404,10 +1412,13 @@ class AdjustTypeParameters(Visitor):
       return self.class_name
     return self._GetFullName(self.function_name)
 
+  def _IsBoundTypeParam(self, node):
+    in_class = self.class_name and node.name in self.class_typeparams
+    return in_class or self.generic_level
+
   def VisitTypeParameter(self, node):
     """Add scopes to type parameters, track unbound params."""
-    if self.constant_name and (not self.class_name or
-                               node.name not in self.class_typeparams):
+    if self.constant_name and not self._IsBoundTypeParam(node):
       raise ContainerError("Unbound type parameter %s in %s" % (
           node.name, self._GetFullName(self.constant_name)))
     scope = self._GetScope(node.name)
@@ -1585,9 +1596,9 @@ class ExpandCompatibleBuiltins(Visitor):
   """Ad-hoc inheritance.
 
   In parameters, replaces
-    ClassType('__builtin__.float')
+    ClassType('builtins.float')
   with
-    Union[ClassType('__builtin__.float'), ClassType('__builtin__.int')]
+    Union[ClassType('builtins.float'), ClassType('builtins.int')]
 
   And similarly for unicode->(unicode, str, bytes) and bool->(bool, None).
 
@@ -1600,7 +1611,7 @@ class ExpandCompatibleBuiltins(Visitor):
   """
 
   def __init__(self, builtins):
-    super(ExpandCompatibleBuiltins, self).__init__()
+    super().__init__()
     self.in_parameter = False
     self.in_type_parameter = False
     self.replacements = self._BuildReplacementMap(builtins)
@@ -1610,8 +1621,6 @@ class ExpandCompatibleBuiltins(Visitor):
     """Dict[str, UnionType[ClassType, ...]] map."""
     prefix = builtins.name + "."
     rmap = collections.defaultdict(list)
-    # Import here due to circular import.
-    from pytype.pytd import pep484  # pylint: disable=g-import-not-at-top
 
     # compat_list :: [(compat, name)], where name is the more generalized
     # type and compat is the less generalized one. (eg: name = float, compat =
@@ -1628,7 +1637,7 @@ class ExpandCompatibleBuiltins(Visitor):
         # want it in our union
         rmap[prefix + name].append(pytd.ClassType(full_name, t))
 
-    return {k: pytd.UnionType(tuple(v)) for k, v in six.iteritems(rmap)}
+    return {k: pytd.UnionType(tuple(v)) for k, v in rmap.items()}
 
   def EnterParameter(self, _):
     assert not self.in_parameter
@@ -1672,7 +1681,7 @@ class ReplaceModulesWithAny(RemoveTypeParametersFromGenericAny):
   """Replace all references to modules in a list with AnythingType."""
 
   def __init__(self, module_list):
-    super(ReplaceModulesWithAny, self).__init__()
+    super().__init__()
     assert isinstance(module_list, list)
     self._any_modules = module_list
 
@@ -1702,11 +1711,11 @@ class ClassTypeToLateType(Visitor):
 
     Args:
       ignore: A list of prefixes to ignore. Typically, this list includes
-        things something like like "__builtin__.", since we don't want to
+        things something like like "builtins.", since we don't want to
         convert builtin types to late types. (And, more generally, types of
         modules that are always loaded by pytype don't need to be late types)
     """
-    super(ClassTypeToLateType, self).__init__()
+    super().__init__()
     self._ignore = ignore
 
   def VisitClassType(self, n):
