@@ -1,21 +1,6 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-
 import textwrap
 
+from pytype.pytd import escape
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
@@ -29,6 +14,16 @@ import unittest
 # parser_test_base.AssertSourceEquals() uses pytd_utils.Print.
 
 
+DEFAULT_PYI = """
+from typing import Any
+def __getattr__(name) -> Any: ...
+"""
+
+
+def pytd_src(text):
+  return textwrap.dedent(escape.preprocess_pytd(text))
+
+
 class TestVisitors(parser_test_base.ParserTest):
   """Tests the classes in parse/visitors."""
 
@@ -44,16 +39,17 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_lookup_classes(self):
     src = textwrap.dedent("""
-        class object(object):
+        from typing import Union
+        class object:
             pass
 
-        class A(object):
-            def a(self, a: A, b: B) -> A or B:
+        class A:
+            def a(self, a: A, b: B) -> Union[A, B]:
                 raise A()
                 raise B()
 
-        class B(object):
-            def b(self, a: A, b: B) -> A or B:
+        class B:
+            def b(self, a: A, b: B) -> Union[A, B]:
                 raise A()
                 raise B()
     """)
@@ -64,8 +60,9 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_maybe_fill_in_local_pointers(self):
     src = textwrap.dedent("""
-        class A(object):
-            def a(self, a: A, b: B) -> A or B:
+        from typing import Union
+        class A:
+            def a(self, a: A, b: B) -> Union[A, B]:
                 raise A()
                 raise B()
     """)
@@ -86,20 +83,21 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_deface_unresolved(self):
     builtins = self.Parse(textwrap.dedent("""
-      class int(object):
+      class int:
         pass
     """))
     src = textwrap.dedent("""
         class A(X):
             def a(self, a: A, b: X, c: int) -> X:
                 raise X()
-            def b(self) -> X[int]
+            def b(self) -> X[int]: ...
     """)
     expected = textwrap.dedent("""
-        class A(?):
-            def a(self, a: A, b: ?, c: int) -> ?:
-                raise ?
-            def b(self) -> ?
+        from typing import Any
+        class A(Any):
+            def a(self, a: A, b: Any, c: int) -> Any:
+                raise Any
+            def b(self) -> Any: ...
     """)
     tree = self.Parse(src)
     new_tree = tree.Visit(visitors.DefaceUnresolved([tree, builtins]))
@@ -109,7 +107,7 @@ class TestVisitors(parser_test_base.ParserTest):
   def test_deface_unresolved2(self):
     builtins = self.Parse(textwrap.dedent("""
       from typing import Generic, TypeVar
-      class int(object):
+      class int:
         pass
       T = TypeVar("T")
       class list(Generic[T]):
@@ -120,14 +118,14 @@ class TestVisitors(parser_test_base.ParserTest):
         class A(X):
             def a(self, a: A, b: X, c: int) -> X:
                 raise X()
-            def c(self) -> Union[list[X], int]
+            def c(self) -> Union[list[X], int]: ...
     """)
     expected = textwrap.dedent("""
-        from typing import Union
-        class A(?):
-            def a(self, a: A, b: ?, c: int) -> ?:
-                raise ?
-            def c(self) -> Union[list[?], int]
+        from typing import Any, Union
+        class A(Any):
+            def a(self, a: A, b: Any, c: int) -> Any:
+                raise Any
+            def c(self) -> Union[list[Any], int]: ...
     """)
     tree = self.Parse(src)
     new_tree = tree.Visit(visitors.DefaceUnresolved([tree, builtins]))
@@ -136,14 +134,16 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_replace_types(self):
     src = textwrap.dedent("""
-        class A(object):
-            def a(self, a: A or B) -> A or B:
+        from typing import Union
+        class A:
+            def a(self, a: Union[A, B]) -> Union[A, B]:
                 raise A()
                 raise B()
     """)
     expected = textwrap.dedent("""
-        class A(object):
-            def a(self: A2, a: A2 or B) -> A2 or B:
+        from typing import Union
+        class A:
+            def a(self: A2, a: Union[A2, B]) -> Union[A2, B]:
                 raise A2()
                 raise B()
     """)
@@ -174,36 +174,38 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_strip_self(self):
     src = textwrap.dedent("""
-        def add(x: int, y: int) -> int
-        class A(object):
-            def bar(self, x: int) -> float
-            def baz(self) -> float
-            def foo(self, x: int, y: float) -> float
+        def add(x: int, y: int) -> int: ...
+        class A:
+            def bar(self, x: int) -> float: ...
+            def baz(self) -> float: ...
+            def foo(self, x: int, y: float) -> float: ...
     """)
     expected = textwrap.dedent("""
-        def add(x: int, y: int) -> int
+        def add(x: int, y: int) -> int: ...
 
-        class A(object):
-            def bar(x: int) -> float
-            def baz() -> float
-            def foo(x: int, y: float) -> float
+        class A:
+            def bar(x: int) -> float: ...
+            def baz() -> float: ...
+            def foo(x: int, y: float) -> float: ...
     """)
     tree = self.Parse(src)
     new_tree = tree.Visit(visitors.StripSelf())
     self.AssertSourceEquals(new_tree, expected)
 
   def test_remove_unknown_classes(self):
-    src = textwrap.dedent("""
+    src = pytd_src("""
+        from typing import Union
         class `~unknown1`():
             pass
         class `~unknown2`():
             pass
-        class A(object):
-            def foobar(x: `~unknown1`, y: `~unknown2`) -> `~unknown1` or int
+        class A:
+            def foobar(x: `~unknown1`, y: `~unknown2`) -> Union[`~unknown1`, int]: ...
     """)
     expected = textwrap.dedent("""
-        class A(object):
-            def foobar(x, y) -> ? or int
+        from typing import Any, Union
+        class A:
+            def foobar(x, y) -> Union[Any, int]: ...
     """)
     tree = self.Parse(src)
     tree = tree.Visit(visitors.RemoveUnknownClasses())
@@ -211,7 +213,8 @@ class TestVisitors(parser_test_base.ParserTest):
     self.AssertSourceEquals(tree, expected)
 
   def test_find_unknown_visitor(self):
-    src = textwrap.dedent("""
+    src = pytd_src("""
+        from typing import Any
         class object:
           pass
         class `~unknown1`():
@@ -221,9 +224,9 @@ class TestVisitors(parser_test_base.ParserTest):
         class `~int`():
           pass
         class A():
-          def foobar(self, x: `~unknown1`) -> ?
+          def foobar(self, x: `~unknown1`) -> Any: ...
         class B():
-          def foobar(self, x: `~int`) -> ?
+          def foobar(self, x: `~int`) -> Any: ...
         class C():
           x = ... # type: `~unknown_foobar`
         class D(`~unknown1`):
@@ -239,13 +242,13 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_in_place_lookup_external_classes(self):
     src1 = textwrap.dedent("""
-      def f1() -> bar.Bar
-      class Foo(object):
+      def f1() -> bar.Bar: ...
+      class Foo:
         pass
     """)
     src2 = textwrap.dedent("""
-      def f2() -> foo.Foo
-      class Bar(object):
+      def f2() -> foo.Foo: ...
+      class Bar:
         pass
     """)
     ast1 = self.Parse(src1, name="foo")
@@ -262,7 +265,7 @@ class TestVisitors(parser_test_base.ParserTest):
       Foo = ...  # type: type
     """)
     src2 = textwrap.dedent("""
-      class Bar(object):
+      class Bar:
         bar = ...  # type: foo.Foo
     """)
     ast1 = self.Parse(src1, name="foo")
@@ -275,7 +278,7 @@ class TestVisitors(parser_test_base.ParserTest):
     src1 = textwrap.dedent("""
       x = ...  # type: int
       T = TypeVar("T")
-      class A(object): ...
+      class A: ...
       def f(x: T) -> T: ...
       B = A
     """)
@@ -290,7 +293,7 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_lookup_star_alias_in_unnamed_module(self):
     src1 = textwrap.dedent("""
-      class A(object): ...
+      class A: ...
     """)
     src2 = "from foo import *"
     ast1 = self.Parse(src1).Replace(name="foo").Visit(visitors.AddNamePrefix())
@@ -306,8 +309,8 @@ class TestVisitors(parser_test_base.ParserTest):
     """).strip())
 
   def test_lookup_two_star_aliases(self):
-    src1 = "class A(object): ..."
-    src2 = "class B(object): ..."
+    src1 = "class A: ..."
+    src2 = "class B: ..."
     src3 = textwrap.dedent("""
       from foo import *
       from bar import *
@@ -320,8 +323,8 @@ class TestVisitors(parser_test_base.ParserTest):
     self.assertSetEqual({a.name for a in ast3.aliases}, {"baz.A", "baz.B"})
 
   def test_lookup_two_star_aliases_with_same_class(self):
-    src1 = "class A(object): ..."
-    src2 = "class A(object): ..."
+    src1 = "class A: ..."
+    src2 = "class A: ..."
     src3 = textwrap.dedent("""
       from foo import *
       from bar import *
@@ -333,10 +336,10 @@ class TestVisitors(parser_test_base.ParserTest):
         {"foo": ast1, "bar": ast2, "baz": ast3}, self_name="baz"))
 
   def test_lookup_star_alias_with_duplicate_class(self):
-    src1 = "class A(object): ..."
+    src1 = "class A: ..."
     src2 = textwrap.dedent("""
       from foo import *
-      class A(object):
+      class A:
         x = ...  # type: int
     """)
     ast1 = self.Parse(src1).Replace(name="foo").Visit(visitors.AddNamePrefix())
@@ -349,8 +352,8 @@ class TestVisitors(parser_test_base.ParserTest):
     """).strip())
 
   def test_lookup_two_star_aliases_with_default_pyi(self):
-    src1 = "def __getattr__(name) -> ?"
-    src2 = "def __getattr__(name) -> ?"
+    src1 = DEFAULT_PYI
+    src2 = DEFAULT_PYI
     src3 = textwrap.dedent("""
       from foo import *
       from bar import *
@@ -367,10 +370,11 @@ class TestVisitors(parser_test_base.ParserTest):
     """).strip())
 
   def test_lookup_star_alias_with_duplicate_getattr(self):
-    src1 = "def __getattr__(name) -> ?"
+    src1 = DEFAULT_PYI
     src2 = textwrap.dedent("""
+      from typing import Any
       from foo import *
-      def __getattr__(name) -> ?
+      def __getattr__(name) -> Any: ...
     """)
     ast1 = self.Parse(src1).Replace(name="foo").Visit(visitors.AddNamePrefix())
     ast2 = self.Parse(src2).Replace(name="bar").Visit(visitors.AddNamePrefix())
@@ -383,8 +387,8 @@ class TestVisitors(parser_test_base.ParserTest):
     """).strip())
 
   def test_lookup_two_star_aliases_with_different_getattrs(self):
-    src1 = "def __getattr__(name) -> int"
-    src2 = "def __getattr__(name) -> str"
+    src1 = "def __getattr__(name) -> int: ..."
+    src2 = "def __getattr__(name) -> str: ..."
     src3 = textwrap.dedent("""
       from foo import *
       from bar import *
@@ -396,10 +400,10 @@ class TestVisitors(parser_test_base.ParserTest):
         {"foo": ast1, "bar": ast2, "baz": ast3}, self_name="baz"))
 
   def test_lookup_star_alias_with_different_getattr(self):
-    src1 = "def __getattr__(name) -> int"
+    src1 = "def __getattr__(name) -> int: ..."
     src2 = textwrap.dedent("""
       from foo import *
-      def __getattr__(name) -> str
+      def __getattr__(name) -> str: ...
     """)
     ast1 = self.Parse(src1).Replace(name="foo").Visit(visitors.AddNamePrefix())
     ast2 = self.Parse(src2).Replace(name="bar").Visit(visitors.AddNamePrefix())
@@ -411,9 +415,10 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_collect_dependencies(self):
     src = textwrap.dedent("""
-      l = ... # type: list[int or baz.BigInt]
-      def f1() -> bar.Bar
-      def f2() -> foo.bar.Baz
+      from typing import Union
+      l = ... # type: list[Union[int, baz.BigInt]]
+      def f1() -> bar.Bar: ...
+      def f2() -> foo.bar.Baz: ...
     """)
     deps = visitors.CollectDependencies()
     self.Parse(src).Visit(deps)
@@ -430,15 +435,17 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_expand(self):
     src = textwrap.dedent("""
-        def foo(a: int or float, z: complex or str, u: bool) -> file
-        def bar(a: int) -> str or unicode
+        from typing import Union
+        def foo(a: Union[int, float], z: Union[complex, str], u: bool) -> file: ...
+        def bar(a: int) -> Union[str, unicode]: ...
     """)
     new_src = textwrap.dedent("""
-        def foo(a: int, z: complex, u: bool) -> file
-        def foo(a: int, z: str, u: bool) -> file
-        def foo(a: float, z: complex, u: bool) -> file
-        def foo(a: float, z: str, u: bool) -> file
-        def bar(a: int) -> str or unicode
+        from typing import Union
+        def foo(a: int, z: complex, u: bool) -> file: ...
+        def foo(a: int, z: str, u: bool) -> file: ...
+        def foo(a: float, z: complex, u: bool) -> file: ...
+        def foo(a: float, z: str, u: bool) -> file: ...
+        def bar(a: int) -> Union[str, unicode]: ...
     """)
     self.AssertSourceEquals(
         self.ApplyVisitorToString(src, visitors.ExpandSignatures()),
@@ -446,8 +453,8 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_print_imports(self):
     src = textwrap.dedent("""
-      from typing import List, Tuple, Union
-      def f(x: Union[int, slice]) -> List[?]: ...
+      from typing import Any, List, Tuple, Union
+      def f(x: Union[int, slice]) -> List[Any]: ...
       def g(x: foo.C.C2) -> None: ...
     """)
     expected = textwrap.dedent("""
@@ -481,10 +488,11 @@ class TestVisitors(parser_test_base.ParserTest):
     res = pytd_utils.Print(tree)
     self.assertMultiLineEqual(res, src)
 
+  @unittest.skip("depended on `or`")
   def test_print_union_name_conflict(self):
     src = textwrap.dedent("""
       class Union: ...
-      def g(x: Union) -> int or float: ...
+      def g(x: Union) -> Union[int, float]: ...
     """)
     tree = self.Parse(src)
     res = pytd_utils.Print(tree)
@@ -492,12 +500,13 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_adjust_type_parameters(self):
     ast = self.Parse("""
+      from typing import Union
       T = TypeVar("T")
       T2 = TypeVar("T2")
-      def f(x: T) -> T
+      def f(x: T) -> T: ...
       class A(Generic[T]):
         def a(self, x: T2) -> None:
-          self = A[T or T2]
+          self = A[Union[T, T2]]
     """)
 
     f = ast.Lookup("f")
@@ -596,7 +605,7 @@ class TestVisitors(parser_test_base.ParserTest):
                       lambda: ast4.Visit(visitors.VerifyContainers()))
 
   def test_clear_class_pointers(self):
-    cls = pytd.Class("foo", None, (), (), (), (), None, ())
+    cls = pytd.Class("foo", None, (), (), (), (), (), None, ())
     t = pytd.ClassType("foo", cls)
     t = t.Visit(visitors.ClearClassPointers())
     self.assertIsNone(t.cls)
@@ -663,8 +672,8 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_add_name_prefix_twice(self):
     src = textwrap.dedent("""
-      from typing import TypeVar
-      x = ...  # type: ?
+      from typing import Any, TypeVar
+      x = ...  # type: Any
       T = TypeVar("T")
       class X(Generic[T]): ...
     """)
@@ -778,13 +787,13 @@ class TestVisitors(parser_test_base.ParserTest):
   def test_verify_heterogeneous_tuple(self):
     # Error: does not inherit from Generic
     base = pytd.ClassType("tuple")
-    base.cls = pytd.Class("tuple", None, (), (), (), (), None, ())
+    base.cls = pytd.Class("tuple", None, (), (), (), (), (), None, ())
     t1 = pytd.TupleType(base, (pytd.NamedType("str"), pytd.NamedType("float")))
     self.assertRaises(visitors.ContainerError,
                       lambda: t1.Visit(visitors.VerifyContainers()))
     # Error: Generic[str, float]
     gen = pytd.ClassType("typing.Generic")
-    gen.cls = pytd.Class("typing.Generic", None, (), (), (), (), None, ())
+    gen.cls = pytd.Class("typing.Generic", None, (), (), (), (), (), None, ())
     t2 = pytd.TupleType(gen, (pytd.NamedType("str"), pytd.NamedType("float")))
     self.assertRaises(visitors.ContainerError,
                       lambda: t2.Visit(visitors.VerifyContainers()))
@@ -792,7 +801,8 @@ class TestVisitors(parser_test_base.ParserTest):
     param = pytd.TypeParameter("T")
     parent = pytd.GenericType(gen, (param,))
     base.cls = pytd.Class(
-        "tuple", None, (parent,), (), (), (), None, (pytd.TemplateItem(param),))
+        "tuple", None, (parent,), (), (), (), (), None,
+        (pytd.TemplateItem(param),))
     t3 = pytd.TupleType(base, (pytd.NamedType("str"), pytd.NamedType("float")))
     t3.Visit(visitors.VerifyContainers())
 
@@ -912,7 +922,7 @@ class TestVisitors(parser_test_base.ParserTest):
     assert node.cls
 
   def test_create_type_parameters_from_unknowns(self):
-    src = textwrap.dedent("""
+    src = pytd_src("""
       from typing import Dict
       def f(x: `~unknown1`) -> `~unknown1`: ...
       def g(x: `~unknown2`, y: `~unknown2`) -> None: ...
@@ -920,11 +930,11 @@ class TestVisitors(parser_test_base.ParserTest):
       def i(x: Dict[`~unknown4`, `~unknown4`]) -> None: ...
 
       # Should not be changed
-      class `~unknown5`(object):
+      class `~unknown5`:
         def __add__(self, x: `~unknown6`) -> `~unknown6`: ...
       def `~f`(x: `~unknown7`) -> `~unknown7`: ...
     """)
-    expected = textwrap.dedent("""
+    expected = pytd_src("""
       from typing import Dict
 
       _T0 = TypeVar('_T0')
@@ -934,7 +944,7 @@ class TestVisitors(parser_test_base.ParserTest):
       def h(x: `~unknown3`) -> None: ...
       def i(x: Dict[_T0, _T0]) -> None: ...
 
-      class `~unknown5`(object):
+      class `~unknown5`:
         def __add__(self, x: `~unknown6`) -> `~unknown6`: ...
       def `~f`(x: `~unknown7`) -> `~unknown7`: ...
     """)
@@ -942,10 +952,11 @@ class TestVisitors(parser_test_base.ParserTest):
     ast1 = ast1.Visit(visitors.CreateTypeParametersForSignatures())
     self.AssertSourceEquals(ast1, expected)
 
+  @unittest.skip("We no longer support redefining TypeVar")
   def test_redefine_typevar(self):
-    src = textwrap.dedent("""
+    src = pytd_src("""
       def f(x: `~unknown1`) -> `~unknown1`: ...
-      class `TypeVar`(object): ...
+      class `TypeVar`: ...
     """)
     ast = self.Parse(src).Visit(visitors.CreateTypeParametersForSignatures())
     self.assertMultiLineEqual(pytd_utils.Print(ast), textwrap.dedent("""
@@ -960,9 +971,9 @@ class TestVisitors(parser_test_base.ParserTest):
   def test_create_type_parameters_for_new(self):
     src = textwrap.dedent("""
       class Foo:
-          def __new__(cls: Type[Foo]) -> Foo
+          def __new__(cls: Type[Foo]) -> Foo: ...
       class Bar:
-          def __new__(cls: Type[Bar], x, y, z) -> Bar
+          def __new__(cls: Type[Bar], x, y, z) -> Bar: ...
     """)
     ast = self.Parse(src).Visit(visitors.CreateTypeParametersForSignatures())
     self.assertMultiLineEqual(pytd_utils.Print(ast), textwrap.dedent("""
@@ -1002,7 +1013,7 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_print_cls(self):
     src = textwrap.dedent("""
-      class A(object):
+      class A:
           def __new__(cls: Type[A]) -> A: ...
     """)
     self.assertMultiLineEqual(pytd_utils.Print(self.Parse(src)),
@@ -1013,7 +1024,7 @@ class TestVisitors(parser_test_base.ParserTest):
 
   def test_print_no_return(self):
     src = textwrap.dedent("""
-      def f() -> nothing
+      def f() -> nothing: ...
     """)
     self.assertMultiLineEqual(pytd_utils.Print(self.Parse(src)),
                               textwrap.dedent("""
@@ -1039,10 +1050,10 @@ class TestVisitors(parser_test_base.ParserTest):
         """).strip())
 
   def test_rename_builtins_prefix(self):
-    """builtins.foo should get rewritten to __builtin__.foo and then to foo."""
+    """__builtin__.foo should get rewritten to builtins.foo and then to foo."""
     src = textwrap.dedent("""
-      import builtins
-      class MyError(builtins.KeyError): ...
+      import __builtin__
+      class MyError(__builtin__.KeyError): ...
     """)
     self.assertMultiLineEqual(pytd_utils.Print(self.Parse(src)),
                               "class MyError(KeyError): ...")
