@@ -1,5 +1,3 @@
-# lint-as: python3
-
 """Custom implementations of builtin types."""
 
 from pytype import abstract
@@ -18,8 +16,7 @@ class TypeNew(abstract.PyTDFunction):
       try:
         bases = list(abstract_utils.get_atomic_python_constant(bases_var))
         if not bases:
-          bases = [
-              self.vm.convert.object_type.to_variable(self.vm.root_cfg_node)]
+          bases = [self.vm.convert.object_type.to_variable(self.vm.root_node)]
         node, variable = self.vm.make_class(
             node, name_var, bases, class_dict_var, cls)
       except abstract_utils.ConversionError:
@@ -40,7 +37,18 @@ class TypeNew(abstract.PyTDFunction):
       # bound to a class or function, so we'll go with Any.
       self.match_args(node, args)  # May raise FailedFunctionCall.
       return node, self.vm.new_unsolvable(node)
-    node, raw_ret = super(TypeNew, self).call(node, func, args)
+    elif args.posargs and all(
+        v.full_name == "typing.Protocol" for v in args.posargs[-1].data):
+      # type(Protocol) is a _ProtocolMeta class that inherits from abc.ABCMeta.
+      # Changing the definition of Protocol in typing.pytd to include this
+      # metaclass causes a bunch of weird breakages, so we instead return the
+      # metaclass when type() or __class__ is accessed on Protocol. For
+      # simplicity, we pretend the metaclass is ABCMeta rather than a subclass.
+      self.match_args(node, args)  # May raise FailedFunctionCall.
+      abc = self.vm.import_module("abc", "abc", 0).get_module("ABCMeta")
+      abc.load_lazy_attribute("ABCMeta")
+      return node, abc.members["ABCMeta"].AssignToNewVariable(node)
+    node, raw_ret = super().call(node, func, args)
     # Removes TypeVars from the return value.
     # See test_typevar.TypeVarTest.test_type_of_typevar(_error).
     ret = self.vm.program.NewVariable()
@@ -51,14 +59,14 @@ class TypeNew(abstract.PyTDFunction):
 
 
 class BuiltinFunction(abstract.PyTDFunction):
-  """Implementation of functions in __builtin__.pytd."""
+  """Implementation of functions in builtins.pytd."""
 
   name = None
 
   @classmethod
   def make(cls, vm):
     assert cls.name
-    return super(BuiltinFunction, cls).make(cls.name, vm, "__builtin__")
+    return super().make(cls.name, vm, "builtins")
 
   def get_underlying_method(self, node, receiver, method_name):
     """Get the bound method that a built-in function delegates to."""
@@ -82,28 +90,6 @@ def get_file_mode(sig, args):
     return ""
 
 
-class Open(BuiltinFunction):
-  """Implementation of open(...)."""
-
-  name = "open"
-
-  def call(self, node, func, args):
-    if self.vm.PY3:
-      # In Python 3, the type of IO object returned depends on the mode.
-      self.match_args(node, args)  # May raise FailedFunctionCall.
-      sig, = self.signatures
-      try:
-        mode = get_file_mode(sig, args)
-      except abstract_utils.ConversionError:
-        pass
-      else:
-        # The default mode is 'r'.
-        io_type = "Binary" if "b" in mode else "Text"
-        return node, self.vm.convert.constant_to_var(abstract_utils.AsInstance(
-            self.vm.lookup_builtin("typing.%sIO" % io_type)), {}, node)
-    return super(Open, self).call(node, func, args)
-
-
 class Abs(BuiltinFunction):
   """Implements abs."""
 
@@ -189,7 +175,7 @@ class Filter(BuiltinFunction):
   def call(self, node, func, args):
     self.match_args(node, args)
     if len(args.posargs) != 2:
-      return super(Filter, self).call(node, func, args)
+      return super().call(node, func, args)
     pred, seq = args.posargs
     # Special case filter(None, seq). We remove None from seq and then call the
     # regular filter() so we don't need to reimplement eveything.
@@ -202,7 +188,7 @@ class Filter(BuiltinFunction):
         else:
           result.PasteBinding(b)
       args = function.Args((pred, result))
-    return super(Filter, self).call(node, func, args)
+    return super().call(node, func, args)
 
 
 class ObjectPredicate(BuiltinFunction):
@@ -213,7 +199,7 @@ class ObjectPredicate(BuiltinFunction):
   """
 
   def __init__(self, name, signatures, kind, vm):
-    super(ObjectPredicate, self).__init__(name, signatures, kind, vm)
+    super().__init__(name, signatures, kind, vm)
     # Map of True/False/None (where None signals an ambiguous bool) to
     # vm values.
     self._vm_values = {
@@ -278,9 +264,9 @@ class HasAttr(BinaryPredicate):
 
     Args:
       node: The given node.
-      obj: An AtomicAbstractValue, generally the left hand side of a
+      obj: A BaseValue, generally the left hand side of a
           hasattr() call.
-      attr: An AtomicAbstractValue, generally the right hand side of a
+      attr: A BaseValue, generally the right hand side of a
           hasattr() call.
 
     Returns:
@@ -337,7 +323,7 @@ def _check_against_mro(vm, target, class_spec):
 
   Args:
     vm: The virtual machine.
-    target: An AtomicAbstractValue whose MRO will be checked.
+    target: A BaseValue whose MRO will be checked.
     class_spec: A Class or PythonConstant tuple of classes (i.e. the second
       argument to isinstance or issubclass).
 
@@ -369,9 +355,9 @@ class IsInstance(BinaryPredicate):
     """Check if the object matches a class specification.
 
     Args:
-      obj: An AtomicAbstractValue, generally the left hand side of an
+      obj: A BaseValue, generally the left hand side of an
           isinstance() call.
-      class_spec: An AtomicAbstractValue, generally the right hand side of an
+      class_spec: A BaseValue, generally the right hand side of an
           isinstance() call.
 
     Returns:
@@ -382,6 +368,8 @@ class IsInstance(BinaryPredicate):
     if (isinstance(obj, abstract.AMBIGUOUS_OR_EMPTY) or cls is None or
         isinstance(cls, abstract.AMBIGUOUS_OR_EMPTY)):
       return None
+    if isinstance(class_spec, abstract.AnnotationClass):
+      class_spec = class_spec.base_cls
     return _check_against_mro(self.vm, cls, class_spec)
 
 
@@ -397,8 +385,8 @@ class IsSubclass(BinaryPredicate):
     """Check if the given class is a subclass of a class specification.
 
     Args:
-      cls: An AtomicAbstractValue, the first argument to an issubclass call.
-      class_spec: An AtomicAbstractValue, the second issubclass argument.
+      cls: A BaseValue, the first argument to an issubclass call.
+      class_spec: A BaseValue, the second issubclass argument.
 
     Returns:
       True if the class is a subclass (or is a class) in the class_spec, False
@@ -424,7 +412,7 @@ class IsCallable(UnaryPredicate):
 
     Args:
       node: The given node.
-      obj: An AtomicAbstractValue, the arg of a callable() call.
+      obj: A BaseValue, the arg of a callable() call.
 
     Returns:
       (node, result) where result = True if the object is callable,
@@ -446,27 +434,27 @@ class IsCallable(UnaryPredicate):
 
 
 class BuiltinClass(abstract.PyTDClass):
-  """Implementation of classes in __builtin__.pytd.
+  """Implementation of classes in builtins.pytd.
 
   The module name is passed in to allow classes in other modules to subclass a
-  module in __builtin__ and inherit the custom behaviour.
+  module in builtins and inherit the custom behaviour.
   """
 
-  def __init__(self, vm, name, module="__builtin__"):
-    if module == "__builtin__":
-      pytd_cls = vm.lookup_builtin("__builtin__.%s" % name)
+  def __init__(self, vm, name, module="builtins"):
+    if module == "builtins":
+      pytd_cls = vm.lookup_builtin("builtins.%s" % name)
     else:
       ast = vm.loader.import_name(module)
       pytd_cls = ast.Lookup("%s.%s" % (module, name))
-    super(BuiltinClass, self).__init__(name, pytd_cls, vm)
+    super().__init__(name, pytd_cls, vm)
     self.module = module
 
 
-class SuperInstance(abstract.AtomicAbstractValue):
+class SuperInstance(abstract.BaseValue):
   """The result of a super() call, i.e., a lookup proxy."""
 
   def __init__(self, cls, obj, vm):
-    super(SuperInstance, self).__init__("super", vm)
+    super().__init__("super", vm)
     self.cls = self.vm.convert.super_type
     self.super_cls = cls
     self.super_obj = obj
@@ -497,8 +485,7 @@ class SuperInstance(abstract.AtomicAbstractValue):
       # superclass, fall back to returning self.
       return self.get.to_variable(node)
     else:
-      return super(SuperInstance, self).get_special_attribute(
-          node, name, valself)
+      return super().get_special_attribute(node, name, valself)
 
   def get_class(self):
     return self.cls
@@ -515,7 +502,7 @@ class Super(BuiltinClass):
   _SIGNATURE = function.Signature.from_param_names("super", ("cls", "self"))
 
   def __init__(self, vm):
-    super(Super, self).__init__(vm, "super")
+    super().__init__(vm, "super")
 
   def call(self, node, _, args):
     result = self.vm.program.NewVariable()
@@ -563,10 +550,10 @@ class Super(BuiltinClass):
 
 
 class Object(BuiltinClass):
-  """Implementation of __builtin__.object."""
+  """Implementation of builtins.object."""
 
   def __init__(self, vm):
-    super(Object, self).__init__(vm, "object")
+    super().__init__(vm, "object")
 
   def is_object_new(self, func):
     """Whether the given function is object.__new__.
@@ -594,7 +581,7 @@ class Object(BuiltinClass):
 
     Returns:
       True if the class's definition of the method is different from the
-      definition in __builtin__.object, False otherwise.
+      definition in builtins.object, False otherwise.
     """
     assert method in ("__new__", "__init__")
     if not isinstance(cls, mixin.Class):
@@ -618,14 +605,14 @@ class Object(BuiltinClass):
             self._has_own(node, val.cls, "__new__")):
         self.load_lazy_attribute("__init__extra_args")
         return self.members["__init__extra_args"]
-    return super(Object, self).get_special_attribute(node, name, valself)
+    return super().get_special_attribute(node, name, valself)
 
 
-class RevealType(abstract.AtomicAbstractValue):
+class RevealType(abstract.BaseValue):
   """For debugging. reveal_type(x) prints the type of "x"."""
 
   def __init__(self, vm):
-    super(RevealType, self).__init__("reveal_type", vm)
+    super().__init__("reveal_type", vm)
 
   def call(self, node, _, args):
     for a in args.posargs:
@@ -638,8 +625,8 @@ class PropertyTemplate(BuiltinClass):
 
   _KEYS = ["fget", "fset", "fdel", "doc"]
 
-  def __init__(self, vm, name, module="__builtin__"):  # pylint: disable=useless-super-delegation
-    super(PropertyTemplate, self).__init__(vm, name, module)
+  def __init__(self, vm, name, module="builtins"):  # pylint: disable=useless-super-delegation
+    super().__init__(vm, name, module)
 
   def signature(self):
     # Minimal signature, only used for constructing exceptions.
@@ -657,13 +644,11 @@ class PropertyTemplate(BuiltinClass):
     raise NotImplementedError()
 
 
-class PropertyInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
+class PropertyInstance(abstract.SimpleValue, mixin.HasSlots):
   """Property instance (constructed by Property.call())."""
 
-  CAN_BE_ABSTRACT = True
-
   def __init__(self, vm, name, cls, fget=None, fset=None, fdel=None, doc=None):
-    super(PropertyInstance, self).__init__("property", vm)
+    super().__init__("property", vm)
     mixin.HasSlots.init_mixin(self)
     self.name = name  # Reports the correct decorator in error messages.
     self.fget = fget
@@ -716,12 +701,15 @@ class PropertyInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
     result = self.vm.program.NewVariable([prop], fdel.bindings, node)
     return node, result
 
+  def isinstance_PropertyInstance(self):
+    return True
+
 
 class Property(PropertyTemplate):
   """Property method decorator."""
 
   def __init__(self, vm):
-    super(Property, self).__init__(vm, "property")
+    super().__init__(vm, "property")
 
   def call(self, node, funcv, args):
     property_args = self._get_args(args)
@@ -729,11 +717,11 @@ class Property(PropertyTemplate):
         self.vm, "property", self, **property_args).to_variable(node)
 
 
-class StaticMethodInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
+class StaticMethodInstance(abstract.SimpleValue, mixin.HasSlots):
   """StaticMethod instance (constructed by StaticMethod.call())."""
 
   def __init__(self, vm, cls, func):
-    super(StaticMethodInstance, self).__init__("staticmethod", vm)
+    super().__init__("staticmethod", vm)
     mixin.HasSlots.init_mixin(self)
     self.func = func
     self.cls = cls
@@ -745,6 +733,9 @@ class StaticMethodInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
   def func_slot(self, node, obj, objtype):
     return node, self.func
 
+  def isinstance_StaticMethodInstance(self):
+    return True
+
 
 class StaticMethod(BuiltinClass):
   """Static method decorator."""
@@ -753,7 +744,7 @@ class StaticMethod(BuiltinClass):
   _SIGNATURE = function.Signature.from_param_names("staticmethod", ("func",))
 
   def __init__(self, vm):
-    super(StaticMethod, self).__init__(vm, "staticmethod")
+    super().__init__(vm, "staticmethod")
 
   def call(self, node, funcv, args):
     if len(args.posargs) != 1:
@@ -766,11 +757,11 @@ class ClassMethodCallable(abstract.BoundFunction):
   """Tag a ClassMethod bound function so we can dispatch on it."""
 
 
-class ClassMethodInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
+class ClassMethodInstance(abstract.SimpleValue, mixin.HasSlots):
   """ClassMethod instance (constructed by ClassMethod.call())."""
 
   def __init__(self, vm, cls, func):
-    super(ClassMethodInstance, self).__init__("classmethod", vm)
+    super().__init__("classmethod", vm)
     mixin.HasSlots.init_mixin(self)
     self.cls = cls
     self.func = func
@@ -783,6 +774,9 @@ class ClassMethodInstance(abstract.SimpleAbstractValue, mixin.HasSlots):
     results = [ClassMethodCallable(objtype, b.data) for b in self.func.bindings]
     return node, self.vm.program.NewVariable(results, [], node)
 
+  def isinstance_ClassMethodInstance(self):
+    return True
+
 
 class ClassMethod(BuiltinClass):
   """Static method decorator."""
@@ -790,7 +784,7 @@ class ClassMethod(BuiltinClass):
   _SIGNATURE = function.Signature.from_param_names("classmethod", ("func",))
 
   def __init__(self, vm):
-    super(ClassMethod, self).__init__(vm, "classmethod")
+    super().__init__(vm, "classmethod")
 
   def call(self, node, funcv, args):
     if len(args.posargs) != 1:
