@@ -10,12 +10,12 @@ class PytdFileSystem(fs.ExtensionRemappingFileSystem):
   """File system that remaps .py file extensions to pytd."""
 
   def __init__(self, underlying):
-    super(PytdFileSystem, self).__init__(underlying, 'pytd')
+    super().__init__(underlying, 'pytd')
 
 
 def create_importlab_environment(conf, typeshed):
   """Create an importlab environment from the python version and path."""
-  python_version = utils.split_version(conf.python_version)
+  python_version = utils.version_from_string(conf.python_version)
   path = fs.Path()
   for p in conf.pythonpath:
     path.add_path(p, 'os')
